/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgParser.h,v 2.4 2000/07/23 20:36:43 jeffharris Exp $

*******************************************************************************/


#ifndef DlgParser_included
#define DlgParser_included

#include <stdio.h>
#include <qobject.h>
#include <qstring.h>
#include <qfileinf.h>

extern "C"
{
    int yylex();
    // The generated lexer
}


enum ParserRetCodes {
  EndOfInput,
  ProjectDataStart,
  ProjectData,
  ProjectDataEnd,
  DialogDataStart,
  DialogData,
  DialogDataEnd,
  LayoutDataStart,
  LayoutData,
  LayoutDataEnd,
  WidgetDataStart,
  WidgetData,
  WidgetDataEnd,
  LayoutItemDataStart,
  LayoutItemData,
  LayoutItemDataEnd,
  PixmapDataStart,
  PixmapData,
  PixmapDataEnd,

  ErrorInInput
};

class DlgParser : public QObject
{
  Q_OBJECT
public: 
  // Methods
  DlgParser( const QString &inFile, char *name = NULL );

  ~DlgParser();

  QString GetVersion();
  QString GetObjectType();

  void StartParsing();
  void StopParsing();

  bool IsInitialized() const;

  const QFileInfo &GetInfo() const;

signals:
  void NewProject();
  void ProjectKeyValue( QString &, QString & );
  void EndProject();
  void NewDialog();
  void DlgKeyValue( QString &, QString & );
  void EndDialog();
  void NewLayout();
  void LayoutKeyValue( QString &, QString & );
  void EndLayout();
  void NewWidget( QString & );
  void WidgetKeyValue( QString &, QString & );
  void EndWidget();
    void NewLayoutItem( QString& );
    void LayoutItemKeyValue( QString&, QString& );
    void EndLayoutItem();
  void NewPixmap();
  void PixmapKeyValue( QString &, QString & );
  void EndPixmap();
  
private:
  // Attributes
  QString type, version;
  // Type and version of the file 
  
  QString widgetType, key, value;
  int lineNumber, errorCount;
  // Attributes which are used by the generated lexer

    bool gridSizeSeen;
    // HACK!!!!!! Has the Key for the Grid size in the widget layout been seen
    // yet.  This is needed so that the QGrid widget can be called Grid like
    // it should.  When a better file format is used, xml???? this can go
    // away.

  FILE *input;
  // Input file

    friend int yylex();
    // The generated lexer

  bool action;
  // Action

  QFileInfo fileInfo;
  // some Infos about the file

  bool initialized;
  // ctor says everything's ok
};

extern DlgParser *gDlgParser;

inline bool DlgParser::IsInitialized() const
{
  return initialized;
}

inline const QFileInfo &DlgParser::GetInfo() const
{
  return fileInfo;
}

#endif
