/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: GuiTools.cpp,v 2.11 2000/07/23 20:36:43 jeffharris Exp $

*******************************************************************************/


#include <qapplication.h>
#include <qcdestyle.h>
#include <qmotifstyle.h>
#include <qplatinumstyle.h>
#include <qstrlist.h>
#include <qwindowsstyle.h>

#include "GuiTools.h"


QStrList GuiTools::itsQDialogSlots;


const QString GuiTools::platinumStyle = "Platinum";
const QString GuiTools::cdeStyle = "CDE";
const QString GuiTools::motifStyle = "Motif";
const QString GuiTools::windowsStyle = "Windows";


void GuiTools::SetStyle( const QString& style )
{
    if( style == platinumStyle )
    {
        QApplication::setStyle( new QPlatinumStyle() );
    }
    else if( style == cdeStyle )
    {
        QApplication::setStyle( new QCDEStyle( TRUE ) );
    }
    else if( style == motifStyle )
    {
        QApplication::setStyle( new QMotifStyle( TRUE ) );
    }
    else if( style == windowsStyle )
    {
        QApplication::setStyle( new QWindowsStyle() );
    }
}


void GuiTools::SetVisible( QWidget*	widget,
			   bool		visible )
{
    if( visible )
    {
	widget->show();
    }
    else
    {
	widget->hide();
    }
}


QRect GuiTools::GetGlobalGeometry
(
    const QWidget*	widget
)
{
    //const QRect& rect = widget->geometry();

//     return QRect( widget->mapToGlobal( rect.topLeft() ),
// 		  widget->mapToGlobal( rect.bottomRight() ) );

    return QRect( widget->mapToGlobal( QPoint(0,0) ),
		  widget->size() );
}


void GuiTools::ChangeColor( 
    QWidget*            widget,
    const QColor&       color 
    )
{
    widget->setPalette( QPalette( color, color ) );
}


void GuiTools::TransferListBox( const QListBox*	source,
				QListBox*	dest )
{
    dest->clear();
    for( unsigned int i = 0; i < (unsigned int)source->count(); i++ )
        dest->insertItem( source->text( i ) );
}    


void GuiTools::TransferListFromCombo( const QComboBox*	source,
				      QListBox*		dest )
{
    dest->clear();
    for( unsigned int i = 0; i < (unsigned int)source->count(); i++ )
        dest->insertItem( source->text( i ) );
}


void GuiTools::TransferListToCombo( QComboBox*		dest,
				    const QListBox*	source )
{
    dest->clear();
    for( unsigned int i = 0; i < (unsigned int)source->count(); i++ )
        dest->insertItem( source->text( i ) );
}


QString GuiTools::QFontStyleHintStr( QFont::StyleHint	hint )
{
    QString str;
    
    switch( hint )
    {
	case QFont::Helvetica:
	default:
	{
	    str = "Helvetica";
	    break;
	}
	case QFont::Times:
	{
	    str = "Times";
	    break;
	}
	case QFont::Courier:
	{
	    str = "Courier";
	    break;
	}
	case QFont::OldEnglish:
	{
	    str = "Old English";
	    break;
	}
	case QFont::System:
	{
	    str = "System";
	    break;
	}
	case QFont::AnyStyle:
	{
	    str = "AnyStyle";
	    break;
	}
    }

    return str;
}


QFont::StyleHint GuiTools::QFontStyleHint( QString	str )
{
    QFont::StyleHint hint = QFont::Helvetica;
    
    if( (str == "Helvetica") || (str == "Sans Serif") )
    {
	// already set to helvetica
    }
    else if( (str == "Times") || (str == "Serif") )
    {
	hint = QFont::Times;
    }
    else if( (str == "Courier") || (str == "Type Writer") )
    {
	hint = QFont::Courier;
    }
    else if( (str == "Old English") || (str == "Decorative") )
    {
	hint = QFont::OldEnglish;
    }
    else if( str == "System" )
    {
	hint = QFont::System;
    }
    else if( str == "Any Style" )
    {
	hint = QFont::AnyStyle;
    }

    return hint;
}



QString GuiTools::QFontWeightStr( int	weight )
{
    QString str;
    str.setNum( weight );
    
    switch( weight )
    {
	case QFont::Light:
	{
	    str = "Light";
	    break;
	}
	case QFont::Normal:
	{
	    str = "Normal";
	    break;
	}
	case QFont::DemiBold:
	{
	    str = "DemiBold";
	    break;
	}
	case QFont::Bold:
	{
	    str = "Bold";
	    break;
	}
	case QFont::Black:
	{
	    str = "Black";
	    break;
	}
    }

    return str;
}


QString GuiTools::QFontCharSetStr( QFont::CharSet	set )
{
    QString str;
    
    switch( set )
    {
	case QFont::Latin1:
	{
	    str = "Latin1";
	    break;
	}
	case QFont::AnyCharSet:
	{
	    str = "Any Char Set";
	    break;
	}
	case QFont::Latin2:
	{
	    str = "Latin2";
	    break;
	}
	case QFont::Latin3:
	{
	    str = "Latin3";
	    break;
	}
	case QFont::Latin4:
	{
	    str = "Latin4";
	    break;
	}
	case QFont::Latin6:
	{
	    str = "Latin6";
	    break;
	}
	case QFont::Latin7:
	{
	    str = "Latin7";
	    break;
	}
	case QFont::Latin8:
	{
	    str = "Latin8";
	    break;
	}
	case QFont::Latin9:
	{
	    str = "Latin9";
	    break;
	}
	case QFont::ISO_8859_5:
	{
	    str = "ISO_8859_5";
	    break;
	}
	case QFont::ISO_8859_6:
	{
	    str = "ISO_8859_6";
	    break;
	}
	case QFont::ISO_8859_7:
	{
	    str = "ISO_8859_7";
	    break;
	}
	case QFont::ISO_8859_8:
	{
	    str = "ISO_8859_8";
	    break;
	}
	case QFont::ISO_8859_9:
	{
	    str = "ISO_8859_9";
	    break;
	}
	case QFont::ISO_8859_11:
	{
	    str = "ISO_8859_11";
	    break;
	}
	case QFont::ISO_8859_12:
	{
	    str = "ISO_8859_12";
	    break;
	}
	case QFont::KOI8R:
	{
	    str = "KOI8R";
	    break;
	}
	case QFont::Set_Ja:
	{
	    str = "Set_Ja";
	    break;
	}
	case QFont::Set_Ko:
	{
	    str = "Set_Ko";
	    break;
	}
	case QFont::Set_Th_TH:
	{
	    str = "Set_Th_TH";
	    break;
	}
	case QFont::Set_Zh:
	{
	    str = "Set_Zh";
	    break;
	}
	case QFont::Set_Zh_TW:
	{
	    str = "Set_Zh_TW";
	    break;
	}
	case QFont::Unicode:
	{
	    str = "Unicode";
	    break;
	}
        case QFont::Set_GBK:
        {
            str = "Set_GBK";
            break;
        }
        case QFont::Set_Big5:
        {
            str = "Set_Big5";
            break;
        }
    }
    
    return str;
}


QFont::CharSet GuiTools::QFontCharSet( QString		set )
{
    QFont::CharSet charSet = QFont::Latin1;
    
    if( (set == "Latin1") || (set == "ISO_8859_1") )
    {
	// already set to Latin1
    }
    else if( set == "Any Char Set" )
    {
	charSet = QFont::AnyCharSet;
    }
    else if( (set == "Latin2") || (set == "ISO_8859_2") )
    {
	charSet = QFont::Latin2;
    }
    else if( (set == "Latin3") || (set == "ISO_8859_3") )
    {
	charSet = QFont::Latin3;
    }
    else if( (set == "Latin4") || (set == "ISO_8859_4") )
    {
	charSet = QFont::Latin4;
    }
    else if( (set == "Latin5") || (set == "ISO_8859_5") )
    {
	charSet = QFont::ISO_8859_5;
    }
    else if( (set == "Latin6") || (set == "ISO_8859_6") )
    {
	charSet = QFont::ISO_8859_6;
    }
    else if( (set == "Latin7") || (set == "ISO_8859_7") )
    {
	charSet = QFont::ISO_8859_7;
    }
    else if( (set == "Latin8") || (set == "ISO_8859_8") )
    {
	charSet = QFont::ISO_8859_8;
    }
    else if( (set == "Latin9") || (set == "ISO_8859_9") )
    {
	charSet = QFont::ISO_8859_9;
    }
    else if( set == "ISO_8859_10" )
    {
	charSet = QFont::ISO_8859_10;
    }
    else if( set == "ISO_8859_11" )
    {
	charSet = QFont::ISO_8859_11;
    }
    else if( set == "ISO_8859_12" )
    {
	charSet = QFont::ISO_8859_12;
    }
    else if( set == "ISO_8859_13" )
    {
	charSet = QFont::ISO_8859_13;
    }
    else if( set == "ISO_8859_14" )
    {
	charSet = QFont::ISO_8859_14;
    }
    else if( set == "ISO_8859_15" )
    {
	charSet = QFont::ISO_8859_15;
    }
    else if( set == "KOI8R" )
    {
	charSet = QFont::KOI8R;
    }
    else if( (set == "Set_Ja") || (set == "Set_1") )
    {
	charSet = QFont::Set_Ja;
    }
    else if( set == "Set_Ko" )
    {
	charSet = QFont::Set_Ko;
    }
    else if( set == "Set_Th_TH" )
    {
	charSet = QFont::Set_Th_TH;
    }
    else if( set == "Set_Zh" )
    {
	charSet = QFont::Set_Zh;
    }
    else if( (set == "Set_Zh_TW") || (set == "Set_N") )
    {
	charSet = QFont::Set_Zh_TW;
    }
    else if( set == "Unicode" )
    {
	charSet = QFont::Unicode;
    }
    else if( set == "Set_GBK" )
    {
        charSet = QFont::Set_GBK;
    }
    else if( set == "Set_Big5" )
    {
        charSet = QFont::Set_Big5;
    }
    
    return charSet;
}


QString GuiTools::QCursorStr( const QCursor&    cursor )
{
    QString str;
    
    switch( cursor.shape() )
    {
        case ArrowCursor:
        {
            str = "arrowCursor";
            break;
        }
        case UpArrowCursor:
        {
            str = "upArrowCursor";
            break;
        }
        case CrossCursor:
        {
            str = "crossCursor";
            break;
        }
        case WaitCursor:
        {
            str = "waitCursor";
            break;
        }
        case IbeamCursor:
        {
            str = "ibeamCursor";
            break;
        }
        case SizeVerCursor:
        {
            str = "sizeVerCursor";
            break;
        }
        case SizeHorCursor:
        {
            str = "sizeHorCursor";
            break;
        }
        case SizeBDiagCursor:
        {
            str = "sizeBDiagCursor";
            break;
        }
        case SizeFDiagCursor:
        {
            str = "sizeFDiagCursor";
            break;
        }
        case SizeAllCursor:
        {
            str = "sizeAllCursor";
            break;
        }
        case BlankCursor:
        {
            str = "blankCursor";
            break;
        }
        case SplitVCursor:
        {
            str = "splitVCursor";
            break;
        }
        case SplitHCursor:
        {
            str = "splitHCursor";
            break;
        }
        case PointingHandCursor:
        {
            str = "pointingHandCursor";
            break;
        }
    }
    
    return str;
}


const QCursor& GuiTools::QCursorVal( const QString&     str )
{
    if( str == "arrowCursor" )
    {
        return Qt::arrowCursor;
    }
    else if( str == "upArrowCursor" )
    {
        return Qt::upArrowCursor;
    }
    else if( str == "crossCursor" )
    {
        return Qt::crossCursor;
    }
    else if( str == "waitCursor" )
    {
        return Qt::waitCursor;
    }
    else if( str == "ibeamCursor" )
    {
        return Qt::ibeamCursor;
    }
    else if( str == "sizeVerCursor" )
    {
        return Qt::sizeVerCursor;
    }
    else if( str == "sizeHorCursor" )
    {
        return Qt::sizeHorCursor;
    }
    else if( str == "sizeBDiagCursor" )
    {
        return Qt::sizeBDiagCursor;
    }
    else if( str == "sizeFDiagCursor" )
    {
        return Qt::sizeFDiagCursor;
    }
    else if( str == "sizeAllCursor" )
    {
        return Qt::sizeAllCursor;
    }
    else if( str == "blankCursor" )
    {
        return Qt::blankCursor;
    }
    else if( str == "splitVCursor" )
    {
        return Qt::splitVCursor;
    }
    else if( str == "splitHCursor" )
    {
        return Qt::splitHCursor;
    }
    else if( str == "pointingHandCursor" )
    {
        return Qt::pointingHandCursor;
    }
    
    return Qt::arrowCursor;
}


void GuiTools::QStrListAppendAll( QStrList&		to,
				  const QStrList&	from )
{
    QStrListIterator iter( from );
    char* item;
    
    while( (item = iter()) )
    {
	to.append( item );
    }
}


const QStrList& GuiTools::GetQDialogSlots( void )
{
    if( itsQDialogSlots.isEmpty() )
    {
	itsQDialogSlots.append( "done(int)" );
	itsQDialogSlots.append( "accept()" );
	itsQDialogSlots.append( "reject()" );
    }
    
    return itsQDialogSlots;
}


int GuiTools::numSiblings(
    const QListViewItem*	theItem
    )
{
    // Need to count the item itself
    int numSiblings = 0;

    for( const QListViewItem* sibling = theItem; 
	 sibling;
	 sibling = sibling->nextSibling(), ++numSiblings );
    
    return numSiblings;
}


QString GuiTools::ArrowTypeStr(
    Qt::ArrowType	theArrowType
    )
{
    QString str;
    
    switch( theArrowType )
    {
	case Qt::UpArrow:
	{
	    str = "UpArrow";
	    break;
	}
	case Qt::DownArrow:
	{
	    str = "DownArrow";
	    break;
	}
	case Qt::LeftArrow:
	{
	    str = "LeftArrow";
	    break;
	}
	case Qt::RightArrow:
	{
	    str = "RightArrow";
	    break;
	}
    }
    
    return str;
}


Qt::ArrowType GuiTools::ArrowTypeVal(
    const QString&	theArrowTypeStr
    )
{
    Qt::ArrowType type = Qt::UpArrow;
    
    if( theArrowTypeStr == "UpArrow" )
    {
	type = Qt::UpArrow;
    }
    else if( theArrowTypeStr == "DownArrow" )
    {
	type = Qt::DownArrow;
    }
    else if( theArrowTypeStr == "LeftArrow" )
    {
	type = Qt::LeftArrow;
    }
    else if( theArrowTypeStr == "RightArrow" )
    {
	type = Qt::RightArrow;
    }
    
    return type;
}
