/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: PreferencesDialog.cpp,v 2.4 2000/07/23 20:36:43 jeffharris Exp $

*******************************************************************************/


#include <qframe.h>

#include "GridPrefPage.h"
#include "I18NPrefPage.h"
#include "ModulesPrefPage.h"
#include "PreferencesDialog.h"
#include "ProjectWnd.h"
#include "StylePrefPage.h"


PreferencesDialog::PreferencesDialog(
    QWidget*	theParent
    )
    :
    QTabDialog( theParent, "Preferences", TRUE )
{
    setApplyButton();
    setCancelButton();
    
    setCaption( tr( "Qt Architect - Preferences" ) );

    AppConfig& appConfig = ProjectWnd::GetAppConfig();

    ModulesPrefPage* modulesPage = new ModulesPrefPage( appConfig, this );
    modulesPage->connect( this,
			  SIGNAL(apply(AppConfig&)), 
			  SLOT(apply(AppConfig&)) );

    GridPrefPage* gridPage = new GridPrefPage( appConfig, this );
    gridPage->connect( this, 
		       SIGNAL(apply(AppConfig&)), 
		       SLOT(apply(AppConfig&)) );

    StylePrefPage* stylePage = new StylePrefPage( appConfig, this );
    stylePage->connect( this,
                        SIGNAL(apply(AppConfig&)),
                        SLOT(apply(AppConfig&)) );

    I18NPrefPage* i18nPage = new I18NPrefPage( appConfig, this );
    i18nPage->connect( this,
                       SIGNAL(apply(AppConfig&)),
                       SLOT(apply(AppConfig&)) );

    addTab( modulesPage, tr( "Widget Modules" ) );
    addTab( stylePage, tr( "Style" ) );
    addTab( i18nPage, tr( "Internationalization" ) );
    addTab( gridPage, tr( "Grid" ) );

    connect( this, SIGNAL(applyButtonPressed()), SLOT(apply()) );

    resize( 400, 300 );
}


PreferencesDialog::~PreferencesDialog()
{
}


void PreferencesDialog::apply()
{
    AppConfig& appConfig = ProjectWnd::GetAppConfig();
    
    emit apply( appConfig );
    
    appConfig.flush();
}
