/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: ProjectWnd.cpp,v 2.9 2000/07/23 20:36:43 jeffharris Exp $

*******************************************************************************/


#include <qapp.h>
#include <qclipbrd.h>
#include <qdir.h>
#include <qfile.h>
#include <qfiledlg.h>
#include <qfileinf.h>
#include <qiconset.h>
#include <qkeycode.h>
#include <qmenubar.h>
#include <qmsgbox.h>
#include <qpixmap.h>
#include <qpopmenu.h>
#include <qregexp.h>
#include <qtimer.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qtstream.h>

#include "Dialog.h"
#include "DlgParser.h"
#include "GenerateDialog.h"
#include "PreferencesDialog.h"
#include "ProjectWnd.h"

#include "pixmap/Pixmap.h"

#include "widget/DlgWidget.h"

// bitmaps
#include "bmp/PixmapFactory.h"

AppConfig ProjectWnd::itsConfig( "QtArchitect" );

#define Inherited QMainWindow

ProjectWnd::ProjectWnd( QWidget* parent,
		      const char* name )
    : 
    Inherited( parent, name ),
    itsObjectList( new ObjectList( this, "dialogList" ) ),
    autoGenerate( FALSE )
{
    setCaption( "Qt Architect" );

    setIcon( PixmapFactory::GetPixmap( QtArch_bmp ) );
    setIconText( caption() );

    // setup menu items
    QPopupMenu* file = new QPopupMenu;
    file->insertItem( tr( "&New Dialog..." ), 
                      this, 
                      SLOT(DialogNew()), 
                      CTRL+Key_N );
    file->insertItem( tr( "&Open Dialog..." ), 
                      this, 
                      SLOT(DialogOpen()), 
                      CTRL+Key_O );
    file->insertSeparator();
    file->insertItem( tr( "New Pixmap..." ), this, SLOT(PixmapNew()) );
    file->insertItem( tr( "Open Pixmap..." ), this, SLOT(PixmapOpen()) );
    file->insertSeparator();
    file->insertItem( tr( "New Project..." ), this, SLOT(ProjectNew()) );
    file->insertItem( tr( "Open Project..." ), this, SLOT(ProjectOpen()) );
    file->insertSeparator();
    file->insertItem( tr( "Save All" ), this, SLOT(ObjectSave()) );
    file->insertSeparator();
    file->insertItem( tr( "Generate Objects" ), 
                      this, 
                      SLOT(ObjectGenerate()), 
		      CTRL + Key_G );
    file->insertSeparator();
    file->insertItem( tr( "E&xit" ), this, SLOT(quit()), CTRL + Key_X );

    QPopupMenu* edit = new QPopupMenu;
    edit->insertItem( tr( "&Preferences" ), this, SLOT(EditPreferences()) );

    QPopupMenu* help = new QPopupMenu;
    help->insertItem( tr( "&About" ), this, SLOT(HelpAbout()) );

    QMenuBar& menu = *menuBar();

    menu.insertItem( tr( "&File" ), file );
    menu.insertItem( tr( "&Edit" ), edit );
    menu.insertSeparator();
    menu.insertItem( tr( "&Help" ), help );

    // Create initial layout (implizit new Window)
    QToolBar* toolBar = new QToolBar( "Main", this );

    itsNewProjectButton = 
	new QToolButton( QIconSet( PixmapFactory::GetPixmap( NewProject_bmp ), 
				   QIconSet::Small ),
			 tr( "New Project" ),
			 tr( "New Project" ),
			 this, 
			 SLOT(ProjectNew()),
			 toolBar );

    itsNewDialogButton = 
	new QToolButton( QIconSet( PixmapFactory::GetPixmap( NewDialog_bmp ),
				   QIconSet::Small ),
			 tr( "New Dialog" ),
			 tr( "New Dialog" ),
			 this,
			 SLOT(DialogNew()), 
			 toolBar );

    itsNewPixmapButton = 
	new QToolButton( QIconSet( PixmapFactory::GetPixmap( NewPixmap_bmp ),
				   QIconSet::Small ),
			 tr( "New Pixmap" ),
			 tr( "New Pixmap" ),
			 this,
			 SLOT(PixmapNew()), 
			 toolBar );

    toolBar->addSeparator();
    
    (void) new QToolButton( QIconSet( PixmapFactory::GetPixmap( OpenProject_bmp ),
				      QIconSet::Small ),
			    tr( "Open Project" ),
			    tr( "Open Project" ),
			    this,
			    SLOT(ProjectOpen()),
			    toolBar );

    (void) new QToolButton( QIconSet( PixmapFactory::GetPixmap( OpenDialog_bmp ),
				      QIconSet::Small ),
			    tr( "Open Dialog" ),
			    tr( "Open Dialog" ),
			    this,
			    SLOT(DialogOpen()),
			    toolBar );

    (void) new QToolButton( QIconSet( PixmapFactory::GetPixmap( OpenPixmap_bmp ),
				      QIconSet::Small ),
			    tr( "Open Pixmap" ),
			    tr( "Open Pixmap" ),
			    this, 
			    SLOT(PixmapOpen()),
			    toolBar );

    toolBar->addSeparator();
    
    itsSaveAllButton = 
	new QToolButton( QIconSet( PixmapFactory::GetPixmap( SaveDialogs_bmp ),
				   QIconSet::Small ),
			 tr( "Save All Open Files" ),
			 tr( "Save All Open Files" ),
			 this,
			 SLOT(ObjectSave()),
			 toolBar );

    toolBar->addSeparator();

    itsGenerateButton = 
	new QToolButton( QIconSet( PixmapFactory::GetPixmap( Generate_bmp ),
				   QIconSet::Small ),
			 tr( "Generate All" ),
			 tr( "Generate All" ),
			 this,
			 SLOT(ObjectGenerate()),
			 toolBar );

    //
    // Setup central widget
    //
    setCentralWidget( itsObjectList );
    itsObjectList->resize( 200,100 );

    // Setup a status bar
    statusBar();

    resize( 250, 150 );

    itsObjects.setAutoDelete( TRUE );

    connect( QApplication::clipboard(), SIGNAL(dataChanged()),
	     SLOT(ClipBoardChanged()) );

    Update();

    QTimer::singleShot( 0, this, SLOT(ProcessCommandArgs()) );
}


ProjectWnd::~ProjectWnd()
{
}


DlgObject* ProjectWnd::CreateNewObject( DlgObjectType type )
{
  static int     defNumber = 1;
  QString extension;

  DlgObject *obj = NULL;

  switch( type )
  {
  case DlgObject_Dialog:
    obj = new Dialog( *this, "" );
    break;
  case DlgObject_Pixmap:
    obj = new Pixmap( *this, "" );
    break;
  case DlgObject_Project:
    obj = new Project( *this, "" );
    break;
  }

  extension = obj->GetExtension();

  QString tmp;
  do
  {
      tmp.setNum( defNumber++ );
      tmp = tr( "Untitled-%1.%2" ).arg( tmp ).arg( extension );
  } while( itsObjectList->Contains( tmp ) );
  
  obj->SetName( tmp );

  if( !autoGenerate )
  {
      obj->Show();
  }

  itsObjects.append( obj );

  ProcessNewObject( obj );
  return obj;
}

void ProjectWnd::ObjectOpen( DlgObjectType type )
{
    QString str;
    
  //  str = QFileDialog::getOpenFileName( NULL, obj->GetExtension() );
    str = QFileDialog::getOpenFileName( QString::null, 
					"*." + DlgObject::GetExtension(type) );
    
    if( !str.isEmpty() )
    {
	ObjectLoad( str, NULL );
    }
}

DlgObject *ProjectWnd::ObjectLoad( const QString &fileName, Project *prj )
{
    DlgObject *obj = DlgObject::NewObject( *this, fileName );

    if( obj )
    {
	obj->SetName( fileName, FALSE );
	obj->Load();

	if( !autoGenerate )
	{
	    obj->Show();
	}

	if( prj )
	    obj->EnterProject( prj );
	else
	    itsObjects.append( obj );

	ProcessNewObject( obj );
    }
    return obj;
}

void ProjectWnd::ProcessNewObject( DlgObject *obj )
{
  itsObjectList->Append( obj );

  connect( obj, SIGNAL(Changed(DlgObject*)), 
           itsObjectList, SLOT(Update(DlgObject*)) );
  connect( obj, SIGNAL(Saved(DlgObject*)),
           itsObjectList, SLOT(Saved(DlgObject*)) );
  connect( obj, SIGNAL(Closed(DlgObject*)), SLOT(Closed(DlgObject*)) );
  connect( obj, SIGNAL(Deleted(DlgObject*)), SLOT(Delete(DlgObject*)) );
  connect( obj, SIGNAL(CPDataSet(DlgObject*)), SLOT(NewCPBuffer(DlgObject*)) );
  connect( this, SIGNAL(CPBufferAvailable(bool)), obj,
           SLOT(CPAvailable(bool)) );

  ClipBoardChanged();
    Update();
}


void ProjectWnd::ObjectSave()
{
    for( DlgObject *obj = itsObjects.first();
         obj;
         obj = itsObjects.next() )
    {
        QString name = obj->GetName();

        if( name.isEmpty() )
        {
            QString extension = "." + obj->GetExtension();

            name = QFileDialog::getSaveFileName( QString::null, 
                                                 "*" + extension );

            if( name.isEmpty() )
                continue;

            if( name.find( '.' ) == -1 )
            {
                name.append( extension );
            }

            obj->SetName( name );
        }      
    
        obj->Save();
    }
}

void ProjectWnd::ObjectGenerate()
{
  for( DlgObject *obj = itsObjects.first();
       obj;
       obj = itsObjects.next() )
  {
      obj->Generate( autoGenerate );
  }

}

void ProjectWnd::PixmapNew()
{
  CreateNewObject( DlgObject_Pixmap );
}

void ProjectWnd::PixmapOpen()
{
  ObjectOpen( DlgObject_Pixmap );
}

void ProjectWnd::DialogNew()
{
  CreateNewObject( DlgObject_Dialog );
}

void ProjectWnd::DialogOpen()
{
  ObjectOpen( DlgObject_Dialog );
}

void ProjectWnd::ProjectNew()
{
  CreateNewObject( DlgObject_Project );
}

void ProjectWnd::ProjectOpen()
{
  ObjectOpen( DlgObject_Project );
}


void ProjectWnd::EditPreferences()
{
    PreferencesDialog( this ).exec();
}

void ProjectWnd::HelpAbout()
{
    QMessageBox::about( this, 
                        "Qt Architect",
                        "<h1>Qt Architect</h1><i>Version " QTARCH_VERSION
                        "</i" );
}


void ProjectWnd::quit()
{
    // Scan for un-saved projects/dialogs - warn user and offer to save.
    
    DlgObject* obj;

    for( obj = itsObjects.first(); obj; obj = itsObjects.next() ) 
    {
        if( obj->notSaved() )
        {
            int s = 
                QMessageBox::warning(
                    this, 
                    "Qt Architect",
                    tr("Save modified file: %1").arg( obj->GetName() ),
                    tr("Yes"),
                    tr("No"),
                    tr("Cancel"), 
                    0, 
                    2 );

            switch (s) 
            {
                case 0:
                {
                    obj->Save();
                    break;
                }
                case 1:
                {
                    break;
                }
                case 2:
                {
                    return;
                }
            }
        }
    }

    qApp->exit( 0 );
}


void ProjectWnd::closeEvent( QCloseEvent*   event )
{
    event->ignore();
    quit();
}


void ProjectWnd::ProcessCommandArgs()
{
    autoGenerate = FALSE;
    
    int argc = qApp->argc();
    char** argv = qApp->argv();
    
    int i;
    for( i = 1; i < argc; ++i )
    {
        if( argv[i][0] == '-' )
        {
            switch( argv[i][1] )
            {
                case 'g':
                {
                    autoGenerate = TRUE;
                    break;
                }
                default:
                {
                    break;
                }
            }
        }
        else
        {
            break;
        }
    }

    if( !autoGenerate )
    {
        show();
    }
    
    for( ; i < argc; ++i )
    {
        ObjectLoad( argv[i] );
    }
    
    if( autoGenerate )
    {
        ObjectGenerate();
        
        quit();
    }
}


void ProjectWnd::Closed( DlgObject* obj )
{
  // shut up
  obj = obj;
    Update();
}


void ProjectWnd::Delete( DlgObject *obj )
{
  // Remove from ObjectList
  itsObjectList->Remove( obj );

  itsObjects.remove( obj );
}

void ProjectWnd::ResetContents( void )
{
}


void ProjectWnd::Update( void )
{
    itsNewProjectButton->setEnabled( TRUE );
    itsNewDialogButton->setEnabled( TRUE );

    bool b = itsObjectList->count() != 0;

    itsSaveAllButton->setEnabled( b );
    itsGenerateButton->setEnabled( b );
}

void ProjectWnd::NewCPBuffer( DlgObject *obj )
{
  // shut up
  obj = obj;

  QClipboard *cb = QApplication::clipboard();

  cb->clear();
  cb->setText( cpBuffer );

  emit CPBufferAvailable( TRUE );
}

void ProjectWnd::ClipBoardChanged()
{
  QClipboard *cb = QApplication::clipboard();

  const char *tst = cb->text();

  if( tst && ! strncmp( tst, "DlgEdit", 7 ) )
  {
    cpBuffer = tst;
    emit CPBufferAvailable( TRUE );
  } else
  {
    emit CPBufferAvailable( FALSE );
  }
}   

QString ProjectWnd::GetCPFileName()
{
  QFile outFile( tempFile );
  outFile.open( IO_WriteOnly );

  QTextStream current( &outFile );

  current << cpBuffer << endl;

  outFile.close();
  
  return tempFile;
}


QString ProjectWnd::GetSourceFileSuffix()
{
    return itsConfig.readEntry( "SourceFileSuffix", "cpp" );
}


void ProjectWnd::SetSourceFileSuffix( const QString& suffix )
{
    itsConfig.writeEntry( "SourceFileSuffix", suffix );
}


QString ProjectWnd::GetHeaderFileSuffix()
{
    return itsConfig.readEntry( "HeaderFileSuffix", "h" );
}


void ProjectWnd::SetHeaderFileSuffix( const QString& suffix )
{
    itsConfig.writeEntry( "HeaderFileSuffix", suffix );
}
