/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: SelectedWidgetBorder.h,v 2.3 2000/07/23 20:36:43 jeffharris Exp $

*******************************************************************************/


#ifndef SelectedWidgetBorder_included
#define SelectedWidgetBorder_included

#include <qframe.h>
#include <qlist.h>
#include <qobject.h>

class QWidget;
class WidgetFrame;

class SelectedWidgetBorder : public QObject
{
    Q_OBJECT

public: // Methods
    
    SelectedWidgetBorder(
        QWidget*	theSelectedWidget,
        WidgetFrame*	theWidgetFrame,
        bool		isMoveable
        );
    // Constructor

    ~SelectedWidgetBorder();
    // Destructor


    void setMoveable(
        bool	isMoveable
        );
    // Update the moveable state of the selected widget.


    bool eventFilter(
        QObject*	theSender,
        QEvent*		theEvent
        );
    // Event filter for receiving mouse events on the border elements
    // and the selected widget.

private: // Types

    enum BorderLocation
    {
        North,
        NorthEast,
        East,
        SouthEast,
        South,
        SouthWest,
        West,
        NorthWest
    };

private: // Methods

    void updateBorders();
    // Update the position of the border elements to the selected widget.

    void updateBorderType(
        BorderLocation	theLocation
        );
    // Update the color and cursor for the border part.

    //
    // Widget Rectangle updating methods
    //
    QRect calculateWidgetRect(
        const QRect&	theWidgetRect,
        const QPoint&	theDelta,
        BorderLocation	theLocation
        ) const;
    // Calculate the new widget rect and return it.

    static int round(
        int	theValue,
        int	theRoundingValue
        );
    // Round the value to a multiple of the rounding value.

    static int truncate(
        int	theValue,
        int	theTruncatingValue
        );
    // Truncate the value to a multiple of the truncating value.

    static int minimum(
        int	theVal1,
        int	theVal2
        );
    // Return the minimum of the two values.

private: // Attributes

    QWidget* mySelectedWidget;
    // The widget which is bordered

    WidgetFrame* myWidgetFrame;
    // The window on which the selected widget is edited

    QList<QFrame> myBorderFrames;
    // The frames representing the north, south, east, and west parts
    // of the border.

    bool isWidgetMoveable;
    // Is the widget's position maveable.

    QPoint myDraggingPoint;
    // The point where the user started dragging one of the border
    // elements.

    static const unsigned int myBorderWidth;
    // The width of the border

    static const unsigned int myBorderLineWidth;
    // The width of the border lines
};


inline int SelectedWidgetBorder::round(
    int	theValue,
    int	theRoundingValue
    )
{
    return ((theValue + (theRoundingValue >> 1)) / theRoundingValue) * theRoundingValue;
}


inline int SelectedWidgetBorder::truncate(
    int	theValue,
    int theTruncatingValue
    )
{
    return theValue - (theValue % theTruncatingValue);
}


inline int SelectedWidgetBorder::minimum(
    int	theVal1,
    int theVal2
    )
{
    return theVal1 < theVal2 ? theVal1 : theVal2;
}


#endif // SelectedWidgetBorder_included
