/**********************************************************************

	--- Qt Architect generated file ---

	File: TabOrderDialog.cpp
	Last generated: Sat Mar 3 21:21:16 2001

 *********************************************************************/

#include "TabOrderDialog.h"

#include <qpainter.h>
#include "WidgetFrame.h"

#define Inherited TabOrderDialogData

TabOrderEntry::TabOrderEntry( QListBox *lb, DlgWidget *wdgt )
    : QListBoxItem( lb ),   
  itsWidget( wdgt )
{
  // XXX To be changed
  hght = 16;
}
  
void TabOrderEntry::paint( QPainter *painter )
{
  int fh = painter->fontMetrics().lineSpacing();

  painter->drawText( 1, fh / 2 + hght / 2, itsWidget->GetWidgetName() );
}


TabOrderDialog::TabOrderDialog
(
	QWidget* parent,
  WidgetFrame *frame,
  QList<DlgWidget> *inList,
	const char* name
)
	:
    Inherited( parent, name ),
    widgetFrame( frame )
{
  QListIterator<DlgWidget> iter( *inList );
  DlgWidget *wdgt;

  frameOverlay = widgetFrame->SetTabMode( true, this );
  
  itsWidgetList->clear();
  while( (wdgt = iter()) )
  {
    new TabOrderEntry( itsWidgetList, wdgt );
  }

  itsUpButton->setEnabled( false );
  itsDownButton->setEnabled( false );

  DrawOrder();
  frameOverlay->repaint();
}

TabOrderDialog::~TabOrderDialog()
{
  widgetFrame->SetTabMode( false );
}

void TabOrderDialog::Down()
{
  unsigned int index = itsWidgetList->currentItem(); 
  if( index == itsWidgetList->count() - 1 )
    return;
  
  TabOrderEntry *above = static_cast<TabOrderEntry*>(itsWidgetList->item( index - 1 ));
  TabOrderEntry *below = static_cast<TabOrderEntry*>(itsWidgetList->item( index + 1 ));

  itsWidgetList->takeItem( below );
  
  itsWidgetList->insertItem( below, above );

  CheckButtons();
  DrawOrder();
}

void TabOrderDialog::Highlighted(QListBoxItem *lb)
{
  CheckButtons();
  
  crntEntry = static_cast<TabOrderEntry*>(lb);
}

void TabOrderDialog::Up()
{
  unsigned int index = itsWidgetList->currentItem(); 
  if( index == 0 )
    return;
  
  TabOrderEntry *above = static_cast<TabOrderEntry*>(
    itsWidgetList->item( index - 1 ) );

  itsWidgetList->takeItem( above );
  
  itsWidgetList->insertItem( above, crntEntry );

  CheckButtons();
  DrawOrder();
}

void TabOrderDialog::CheckButtons()
{
  int index = itsWidgetList->currentItem();
  int count = itsWidgetList->count();
  
  if( index < 0 )
  {
    itsUpButton->setEnabled( false );
    itsDownButton->setEnabled( false );
  } else if( index == 0 && count > 0 )
  {
    itsUpButton->setEnabled( false );
    itsDownButton->setEnabled( true );
  } else if( index == count - 1 && count > 0 )
  {
    itsUpButton->setEnabled( true );
    itsDownButton->setEnabled( false );
  } else
  {
    itsUpButton->setEnabled( true );
    itsDownButton->setEnabled( true );
  }
  
}

void TabOrderDialog::GetNewOrder( QList<DlgWidget> &list )
{
  list.clear();
  
  for( unsigned int i = 0; i < itsWidgetList->count(); i++ )
  {
    TabOrderEntry *item = static_cast<TabOrderEntry*>(itsWidgetList->item( i ));

    list.append( item->GetWidget() );
  }
}

void TabOrderDialog::DrawOrder()
{
  QPainter painter;
  QRect wRect, eRect;
  QPoint pos;
  QWidget *widget;
  QString number;

  painter.begin( frameOverlay, this );
  painter.setBrush( Qt::yellow );

  for( unsigned int i = 0; i < itsWidgetList->count(); i++ )
  {
    TabOrderEntry *item = static_cast<TabOrderEntry*>(
      itsWidgetList->item( i ));

    // Naming sometimes gets complicated...
    widget = item->GetWidget()->GetWidget();

    // Translate widgets zero point to WidgetFrame
    wRect = widget->geometry();
    wRect.moveTopLeft( widget->mapTo( widgetFrame, pos ));

    // the ellipse shouldn't cover the whole widget
    eRect = wRect;
    eRect.setHeight( eRect.height()/2 );
    eRect.setWidth( eRect.width()/2 );
    eRect.moveCenter( wRect.center() );
    
    number.setNum( i + 1 );

    painter.drawEllipse( eRect );
    painter.drawText( wRect, Qt::AlignCenter, number );
      
  }

  painter.end();
}
  
void TabOrderDialog::show()
{
  Inherited::show();
  DrawOrder();
}
