/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: WidgetFrame.cpp,v 2.4 2001/03/20 07:26:11 kebner Exp $

*******************************************************************************/


#include <qpainter.h>
#include <qfocusdata.h>

#include "WidgetFrame.h"
#include "WidgetLayout.h"
#include "TabOrderDialog.h"

Overlay::Overlay( QWidget *parent, TabOrderDialog *td )
    : QWidget( parent ),
      tabDialog( td )
{
}

void Overlay::paintEvent( QPaintEvent *theEvent )
{
  QWidget::paintEvent( theEvent );

  tabDialog->DrawOrder();
}

#define Inherited QFrame

WidgetFrame::WidgetFrame( QWidget* 	parent,
			  const char*	name )
    :
    Inherited( parent, name ),
    actGrid( 1 ),
    overlay( NULL )
{
}


WidgetFrame::~WidgetFrame()
{
}


void WidgetFrame::paintEvent( QPaintEvent *theEvent )
{
  QFrame::paintEvent( theEvent );

	// No grid, no action
  if( actGrid >= 2 )
  {
    QPainter painter;
    painter.begin( this );
    for( int x = actGrid; x < this->width(); x += actGrid )
	    for( int y = actGrid; y < this->height(); y += actGrid )
	    {
        painter.drawPoint( x, y );
	    }
    painter.end();
  }

}

void WidgetFrame::SetGrid( int grid )
{
	// Never accept < 1
	if( grid < 1 )
		grid = 1;
	
	actGrid = grid;

	repaint();
}

QWidget *WidgetFrame::SetTabMode( bool enable, TabOrderDialog *t )
{
  if( enable )
  {
    // Sanity check: overlay should be already destroyed
    if( overlay )
    {
      return overlay;
    }

    // Construct a overlayoing widget with no background
    overlay = new Overlay( this, t );
    overlay->setFixedSize( this->size() );
    overlay->setBackgroundPixmap( QPixmap::grabWidget( this ) );
    overlay->raise();
    overlay->show();

  } else
  {
    delete overlay;
    overlay = NULL;
  }

  return overlay;
}
    

