/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: BoxLayoutPropPage.cpp,v 2.3 2000/07/23 20:36:43 jeffharris Exp $

*******************************************************************************/


/**********************************************************************

	--- dlgedit generated file ---

	File: BoxLayoutPropPage.cpp
	Last generated: Fri May 2 14:17:25 1997

 *********************************************************************/

#include <qstring.h>
#include <qstrlist.h>

#include "layout/BoxLayoutPropPage.h"
#include "layout/BoxLayout.h"
#include "WidgetLayout.h"
#include "widget/DlgMenuBar.h"

#define Inherited BoxLayoutPropPageData

BoxLayoutPropPage::BoxLayoutPropPage
(
    BoxLayout*		layout,
    bool		topLevel,
    const WidgetLayout&	theWidgetLayout,
    QWidget*		parent,
    const char*		name
)
    :
    Inherited( parent, name ),
    itsLayout( layout ),
    myWidgetLayout( theWidgetLayout )
{
    switch( itsLayout->GetDirection() )
    {
	case QBoxLayout::LeftToRight:
	{
	    isLeftToRight->setChecked( TRUE );
	    break;
	}
	case QBoxLayout::RightToLeft:
	{
	    isRightToLeft->setChecked( TRUE );
	    break;
	}
	case QBoxLayout::TopToBottom:
	{
	    isTopToBottom->setChecked( TRUE );
	    break;
	}
	case QBoxLayout::BottomToTop:
	{
	    isBottomToTop->setChecked( TRUE );
	    break;
	}
    }
    
    QString str;

    itsBorder->setEnabled( topLevel );
    itsBorderLabel->setEnabled( topLevel );
    itsBorder->setText( str.setNum( itsLayout->GetBorder() ) );

    itsAutoBorder->setText( str.setNum( itsLayout->GetAutoBorder() ) );
    itsStrut->setText( str.setNum( itsLayout->GetStrut() ) );

    itsStretch->setEnabled( !topLevel );
    itsStretchLabel->setEnabled( !topLevel );
    itsStretch->setText( str.setNum( itsLayout->GetStretch() ) );

    itsName->setText( itsLayout->GetName() );

    // insert names of menu bars into combo box
    QStrList menus;
    myWidgetLayout.GetWidgetTypeNames( DlgWidget::DW_MenuBar, menus );

    itsMenuBars->insertItem( tr( "(None)" ) );
    itsMenuBars->insertStrList( &menus );

    QString menuName;
    if( itsLayout->GetMenuBarWidget() )
    {
	menuName = itsLayout->GetMenuBarWidget()->GetWidgetName();
    }
    
    int index = 0;

    if( !menuName.isEmpty() )
    {
	index = menus.find( menuName );
	if( index == -1 )
	{
	    index = 0;
	}
	else
	{
	    // account for the "(None)" entry
	    ++index;
	}
    }
    
    itsMenuBars->setCurrentItem( index );

    myChangedFlag = FALSE;
}


BoxLayoutPropPage::~BoxLayoutPropPage()
{
}


bool BoxLayoutPropPage::Apply( void )
{
    if( itsLayout )
    {
	QBoxLayout::Direction dir = QBoxLayout::LeftToRight;
    
	if( isRightToLeft->isChecked() )
	{
	    dir = QBoxLayout::RightToLeft;
	}
	else if( isTopToBottom->isChecked() )
	{
	    dir = QBoxLayout::TopToBottom;
	}
	else if( isBottomToTop->isChecked() )
	{
	    dir = QBoxLayout::BottomToTop;
	}
    
	itsLayout->SetDirection( dir );
    
	QString str;
    
	str = itsBorder->text();
	itsLayout->SetBorder( str.toInt() );
    
	str = itsAutoBorder->text();
	itsLayout->SetAutoBorder( str.toInt() );

	str = itsStrut->text();
	itsLayout->SetStrut( str.toInt() );

	str = itsStretch->text();
	itsLayout->SetStretch( str.toInt() );
    
	itsLayout->SetName( itsName->text() );

	str = "";
	if( itsMenuBars->currentItem() > 0 )
	{
	    str = itsMenuBars->currentText();
	}
	itsLayout->SetMenuBarWidget( 
	    (const DlgMenuBar*)myWidgetLayout.GetWidget(str) );
    }

    return myChangedFlag;
}

