/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: BoxSpacing.cpp,v 2.6 2000/07/23 20:36:43 jeffharris Exp $

*******************************************************************************/


#include "layout/BoxSpacing.h"
#include "layout/BoxSpacingPropPage.h"
#include "layout/LayoutHelper.h"

#include "xml/QtArchGenerator.h"

#define Inherited LayoutItem

BoxSpacing::BoxSpacing(
    LayoutItem*	theParent
    )
    :
    Inherited( LayoutItem::LI_BoxSpacing, 
	       theParent,
	       GetTreeLabel(), 
	       &PixmapFactory::GetPixmap( BoxSpacing_bmp ) ),
    mySpacing( 7 )
{
}


BoxSpacing::BoxSpacing(
    QListView*	theParent
    )
    :
    Inherited( LayoutItem::LI_BoxSpacing, 
	       theParent,
	       GetTreeLabel(), 
	       &PixmapFactory::GetPixmap( BoxSpacing_bmp ) ),
    mySpacing( 7 )
{
}


BoxSpacing::~BoxSpacing()
{
}


LayoutItemPropPage* BoxSpacing::GetPropertyPage( QWidget* theParent,
						 bool	/*isTopLevel*/,
						 const WidgetLayout&	/*theWidgetLayout*/ )
{
    return new BoxSpacingPropPage( this, theParent );
}


void BoxSpacing::AddToLayout( QBoxLayout*	theBoxLayout )
{
    theBoxLayout->addSpacing( mySpacing );
}


void BoxSpacing::SaveContents( QtArchGenerator &generator ) const
{
  generator.WriteStartBlock( Type2String() );

  generator.WriteXMLValue( "Spacing", mySpacing );

  generator.WriteEndBlock();
}


bool BoxSpacing::RestoreKeyValue( const QString&	theKey,
				  const QString&	theValue,
				  const DlgWidget*	/*theDlgWidget*/ )
{
    bool rc = TRUE;
    
    if( theKey == "Spacing" )
    {
	mySpacing = theValue.toInt();
    }
    else
    {
	rc = FALSE;
    }
    
    return rc;
}

void BoxSpacing::Restore( const XMLValue &value, LayoutHelper */*helper*/ )
{
  // Sanity check
  if( value.GetTypeString() != Type2String() )
    return;

  const XMLValueMap map = value.AsNested();

  mySpacing = map["Spacing"].AsInt();
}


void BoxSpacing::GenerateSourceInBox( QTextStream&	theStream,
				      const QString&	theLayoutVarName,
				      const QString&	/*theLayoutId*/,
				      const QString&	/*theLayoutParentWidget*/ ) const
{
    theStream << '\t' << theLayoutVarName << "->addSpacing( " <<
	mySpacing << " );\n";
}


QString BoxSpacing::GetTreeLabel()
{
    static QString treeLabel = QObject::tr( "Box Spacing" );
    return treeLabel;
}
