/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: GridRow.h,v 2.5 2000/07/23 20:36:43 jeffharris Exp $

*******************************************************************************/


#ifndef GridRow_included
#define GridRow_included

#include "layout/LayoutItem.h"

class QtArchGenerator;

class GridRow : public LayoutItem
{
public: // Methods

    GridRow(
	LayoutItem*	theParent
	);
    // Constructor

    GridRow(
	QListView*	theParent
	);
    // Constructor

    virtual ~GridRow();
    // Destructor

    virtual LayoutItemPropPage* GetPropertyPage( QWidget*	parent,
						 bool		topLevel,
						 const WidgetLayout&	theWidgetLayout );
    // Get the property page for the item.  All the information on the page 
    // should be updated.  The caller is responsible for freeing the
    // page when finished.

    //
    // Layout creation
    //

    void AddToLayout( QGridLayout*	theGridLayout,
		      int		theRowNum );
    // Add the contents of the row to the given grid layout.



    //
    // Loading and Saving
    //

    virtual void SaveContents( QtArchGenerator & ) const;
    // Save the contents of this layout item to theStream with theId as
    // its position identifier.
    
    virtual void Restore( const XMLValue &, LayoutHelper * );
    // Restore the item from the given XMLValue
    
    virtual bool RestoreKeyValue( const QString&	theKey,
				  const QString&	theValue,
				  const DlgWidget*	theDlgWidget );
    // Restore a key/value pair for the item.  theDlgWidget is the dialog
    // widget associated with theValue, if any.


    //
    // Code generation
    //

    virtual void GenerateSourceInGrid( QTextStream&	theStream,
				       const QString&	theLayoutVarName,
				       const QString&	theLayoutId,
				       unsigned int	theRowNum,
				       unsigned int	theColNum,
				       const QString&	theLayoutParentWidget ) const;
    // Generate the source code in a grid layout.


    //
    // Get/Set Properties
    //

    // Stretch
    int GetStretch( void ) const;
    void SetStretch( int theStretch );
    
    // Spacing
    int GetSpacing( void ) const;
    void SetSpacing( int theSpacing );

private: // Class Methods

    static QString GetTreeLabel();
    // Get the label for the item in the tree
    
private: // Attributes

    //
    // Properties
    //

    int		myStretch;
    int		mySpacing;
};


inline int GridRow::GetStretch( void ) const
{
    return myStretch;
}


inline void GridRow::SetStretch( int	theStretch )
{
    myStretch = theStretch;
}


inline int GridRow::GetSpacing( void ) const
{
    return mySpacing;
}


inline void GridRow::SetSpacing( int	theSpacing )
{
    mySpacing = theSpacing;
}


#endif // GridRow_included
