/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: LayoutPropFrame.cpp,v 2.3 2000/07/23 20:36:43 jeffharris Exp $

*******************************************************************************/


#include "layout/LayoutPropFrame.h"
#include "layout/LayoutItemPropPage.h"

#define Inherited QFrame

LayoutPropFrame::LayoutPropFrame( QWidget*	parent,
				  const char*	name )
    :
    Inherited( parent, name ),
    myEmptyPage( tr( "No Properties" ), this ),
    myCurrentPage( NULL ),
    myLayout( NULL ),
    myApplyButton( tr( "Apply" ), this )
{
    // Initialize empty page
    myEmptyPage.setAlignment( AlignCenter );
    myEmptyPage.setFrameStyle( QFrame::Box | QFrame::Raised );

    // Initialize apply button
    myApplyButton.setFixedSize( myApplyButton.sizeHint() );
    connect( &myApplyButton, SIGNAL(clicked()), SLOT(ApplyClicked()) );

    RecreateLayout();
}


LayoutPropFrame::~LayoutPropFrame()
{
    delete myLayout;
    delete myCurrentPage;
}


bool LayoutPropFrame::SetPropertyPage( LayoutItemPropPage*	page )
{
    bool currentPropsChanged = FALSE;

    setUpdatesEnabled( FALSE );

    if( myCurrentPage )
    {
	currentPropsChanged = myCurrentPage->Apply();
    }

    delete myLayout;
    delete myCurrentPage;

    myCurrentPage = page;
    RecreateLayout();

    setUpdatesEnabled( TRUE );
    repaint();

    return currentPropsChanged;
}


void LayoutPropFrame::RemovePropertyPage( void )
{
    setUpdatesEnabled( FALSE );
    
    delete myLayout;
    delete myCurrentPage;
    
    myCurrentPage = NULL;
    RecreateLayout();
    
    setUpdatesEnabled( TRUE );
    repaint();
}


void LayoutPropFrame::RecreateLayout( void )
{
    myLayout = new QBoxLayout( this, QBoxLayout::Down, 5, 5 );
    
    if( myCurrentPage )
    {
	myEmptyPage.hide();
	myCurrentPage->show();
	myLayout->addWidget( myCurrentPage, 1 );
    }
    else
    {
	myEmptyPage.show();
	myLayout->addWidget( &myEmptyPage, 1 );
    }

    QBoxLayout* buttonLayout = new QBoxLayout( QBoxLayout::LeftToRight );
    myLayout->addLayout( buttonLayout, 0 );

    // Initialize button layout
    buttonLayout->addStretch( 1 );
    buttonLayout->addWidget( &myApplyButton, 0 );
    
    myLayout->activate();
}


void LayoutPropFrame::ApplyClicked()
{
    if( myCurrentPage )
    {
	myCurrentPage->Apply();
	emit ApplyProperties();
    }
}
