/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: LayoutPropFrame.h,v 2.3 2000/07/23 20:36:43 jeffharris Exp $

*******************************************************************************/


#ifndef LayoutPropFrame_included
#define LayoutPropFrame_included

#include <qframe.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qpushbt.h>

class LayoutItemPropPage;

class LayoutPropFrame : public QFrame
{
    Q_OBJECT

public:

    LayoutPropFrame( QWidget*		parent = NULL,
		     const char*	name = NULL );
    // Constructor

    virtual ~LayoutPropFrame();
    // Destructor

    bool SetPropertyPage( LayoutItemPropPage*	page );
    // Make this property page current.  Return TRUE if the contents of the
    // current page were changed.

    LayoutItemPropPage* GetPropertyPage() const;
    // Get the current property page

    void RemovePropertyPage( void );
    // Remove the current property page

signals:

    void ApplyProperties();
    // Signal emitted when properties are applied.

private:

    void RecreateLayout( void );
    // Recreate the layout with the current page

private slots:

    void ApplyClicked();
    // Apply button was clicked

private:

    QLabel		myEmptyPage;
    // Page to show if there is none defined

    LayoutItemPropPage*	myCurrentPage;
    // Current property page

    QBoxLayout*		myLayout;
    // Layout for the frame

    QPushButton		myApplyButton;
    // Button to apply changes
};


inline LayoutItemPropPage* LayoutPropFrame::GetPropertyPage() const
{
    return myCurrentPage;
}


#endif // LayoutPropFrame_included
