/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: LayoutWidgetPropPage.cpp,v 2.5 2000/07/23 20:36:43 jeffharris Exp $

*******************************************************************************/


/**********************************************************************

	--- Qt Architect generated file ---

	File: LayoutWidgetPropPage.cpp
	Last generated: Sun Feb 1 17:48:01 1998

 *********************************************************************/

#include <qstring.h>
#include <qstrlist.h>

#include "layout/LayoutWidgetPropPage.h"
#include "layout/LayoutWidget.h"
#include "WidgetLayout.h"

#define Inherited LayoutWidgetPropPageData

LayoutWidgetPropPage::LayoutWidgetPropPage
(
    LayoutWidget*	theWidget,
    const WidgetLayout&	theWidgetLayout,
    QWidget* 		parent,
    const char*		name
)
    :
    Inherited( parent, name ),
    myWidget( theWidget ),
    myWidgetLayout( theWidgetLayout )
{
    // Widget names
    QStrList widgetNames;
    theWidgetLayout.GetLayoutWidgetNames( widgetNames, FALSE );
    myWidgetList->insertStrList( &widgetNames );

    bool newWidget = (widgetNames.count() > 0);
    QString widgetName;
    if( myWidget->GetDlgWidget() )
    {
	widgetName = myWidget->GetDlgWidget()->GetWidgetName();

	myWidgetList->insertItem( widgetName );
	myWidgetList->setCurrentItem( myWidgetList->count() - 1 );
	newWidget = FALSE;
    }

    myWidgetList->setEnabled( myWidgetList->count() > 0 );
    myWidgetListLabel->setEnabled( myWidgetList->count() > 0 );
    
    if( myWidget->IsLayoutWidgetChild() )
    {
        myBoxStretchLabel->setEnabled( FALSE );
        myBoxStretch->setEnabled( FALSE );
        myHorizontalAlignBG->setEnabled( FALSE );
        myVerticalAlignBG->setEnabled( FALSE );
        myAlignLeftRB->setEnabled( FALSE );
        myAlignHCenterRB->setEnabled( FALSE );
        myAlignRightRB->setEnabled( FALSE );
        myAlignTopRB->setEnabled( FALSE );
        myAlignVCenterRB->setEnabled( FALSE );
        myAlignBottomRB->setEnabled( FALSE );
        myAlignHNone->setEnabled( FALSE );
        myAlignVNone->setEnabled( FALSE );
    }
    else
    {
        int alignment = myWidget->GetAlignment();

        // Horizontal alignment
        if( alignment & AlignLeft )
        {
            myAlignLeftRB->setChecked( TRUE );
        }
        else if( alignment & AlignHCenter )
        {
            myAlignHCenterRB->setChecked( TRUE );
        }
        else if( alignment & AlignRight )
        {
            myAlignRightRB->setChecked( TRUE );
        }
        else
        {
            myAlignHNone->setChecked( TRUE );
        }
    
        // Vertical alignment
        if( alignment & AlignTop )
        {
            myAlignTopRB->setChecked( TRUE );
        }
        else if( alignment & AlignVCenter )
        {
            myAlignVCenterRB->setChecked( TRUE );
        }
        else if( alignment & AlignBottom )
        {
            myAlignBottomRB->setChecked( TRUE );
        }
        else
        {
            myAlignVNone->setChecked( TRUE );
        }

        // Box Stretch
        QString str;
        myBoxStretch->setText( str.setNum( myWidget->GetBoxStretch() ) );

        bool parentIsBox = 
            (myWidget->GetParentType() == LayoutItem::LI_BoxLayout);
        myBoxStretch->setEnabled( parentIsBox );
        myBoxStretchLabel->setEnabled( parentIsBox );
    }
    
    myChangedFlag = newWidget;
}


LayoutWidgetPropPage::~LayoutWidgetPropPage()
{
}


bool LayoutWidgetPropPage::Apply()
{
    if( myWidget )
    {
        // The widget
        if( myWidgetList->isEnabled() )
        {
            myWidget->SetDlgWidget( 
                myWidgetLayout.GetWidget( myWidgetList->currentText() ) );
        }

        if( !myWidget->IsLayoutWidgetChild() )
        {
            // Alignment
            int alignment = 0;
	
            if( myAlignLeftRB->isChecked() )
            {
                alignment |= AlignLeft;
            }
            else if( myAlignHCenterRB->isChecked() )
            {
                alignment |= AlignHCenter;
            }
            else if( myAlignRightRB->isChecked() )
            {
                alignment |= AlignRight;
            }
	
            if( myAlignTopRB->isChecked() )
            {
                alignment |= AlignTop;
            }
            else if( myAlignVCenterRB->isChecked() )
            {
                alignment |= AlignVCenter;
            }
            else if( myAlignBottomRB->isChecked() )
            {
                alignment |= AlignBottom;
            }
	
            myWidget->SetAlignment( alignment );
	
            // Box Stretch
            QString str = myBoxStretch->text();
            myWidget->SetBoxStretch( str.toInt() );
        }
    }

    return myChangedFlag;
}


void LayoutWidgetPropPage::WidgetChanged( const QString& oldName, 
                                          const DlgWidget* widget )
{
    int i;
    for( i = 0; i < myWidgetList->count(); ++i )
    {
        if( myWidgetList->text( i ) == oldName )
        {
            myWidgetList->changeItem( widget->GetWidgetName(), i );
            break;
        }
    }
}


