/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: WidgetModule.h,v 2.2 2000/07/23 20:37:44 jeffharris Exp $

*******************************************************************************/


#ifndef WidgetModule_Included
#define WidgetModule_Included

#include <qstring.h>
#include <qstrlist.h>

class DlgWidget;
class QApplication;
class QPixmap;
class WidgetModule;


extern "C"
{
    void GetModuleVersion(
	int&	theMajorVersion,
	int&	theMinorVersion
	);

    WidgetModule* GetWidgetModule();
    
    QApplication* GetQApplication(
	int&	theArgc,
	char**	theArgv
	);
}


class WidgetModule
{
public:

    /**
     *	Return the name of the widget module
     *
     *	@return	the name of the widget module
     */
    virtual QString getName() = 0;


    /**
     *	Should a toolbar be shown for adding the widgets
     */
    virtual bool useToolbar() = 0;
    

    /**
     *	Return the list of the names of widgets created by this module
     *
     *	@return	the list of names of the widgets
     */
    virtual QStrList getWidgetNames() = 0;


    /**
     *	Return a pixmap representing the widget specified
     *
     *	@param	theWidgetName	the name of the widget for the pixmap
     *
     *	@return	a valid pixmap if the widget has one, or a null pixmap if it
     *		doesn't
     */
    virtual QPixmap getWidgetPixmap(
	const QString&	theWidgetName
	) = 0;
    

    /**
     *	Return a new widget based on the number specified
     *
     *	@param	theWidgetNum	the number of the widget to create
     *
     *	@return	a new widget based on the number specified
     */
    virtual DlgWidget* newWidget(
	int	theWidgetNum
	) = 0;


    /**
     *	Return a new widget based on the name specified
     *
     *	@param	theWidgetName	the name of the widget to create
     *
     *	@return	a new widget based on the name specified
     */
    virtual DlgWidget* newWidget(
	const QString&	theWidgetName
	) = 0;
};


#endif // WidgetModule_Included
