/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgKColorButton.cpp,v 2.3 2000/07/23 20:37:44 jeffharris Exp $

*******************************************************************************/


#include "DlgKColorButton.h"

#ifndef MODULE_BOOTSTRAP
#include "KColorButtonPropPage.h"
#endif // MODULE_BOOTSTRAP

#include "GuiTools.h"

#define Inherited DlgPushButton

QStrList DlgKColorButton::itsSignals;
int DlgKColorButton::itsNextWidgetNum = 1;

DlgKColorButton::DlgKColorButton() :
    Inherited( DlgWidget::DW_PushButton )
{
}


DlgKColorButton::DlgKColorButton(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgKColorButton::~DlgKColorButton()
{
}


void DlgKColorButton::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

#ifndef MODULE_BOOTSTRAP
    pages.append( new KColorButtonPropPage( this, parent, "KColorButton" ) );

    labels.append( "KColorButton" );
#endif // MODULE_BOOTSTRAP
}


void DlgKColorButton::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
    if( topLevel )
    {
	stream << "KDE::KColorButton {" << endl;
    }
  
    KColorButton* widget = GetKColorButton();

    stream << saveIndent << "Color {" << widget->color().rgb() << "}" << endl;

    Inherited::SaveContents( stream, FALSE );

    if( topLevel )
    {
	stream << "}" << endl;
    }
}


bool DlgKColorButton::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    KColorButton* widget = GetKColorButton();

    // Compare to known keys
    if( key == "Color" )
    {
	widget->setColor( QColor( value.toUInt() ) );
    } 
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}


void DlgKColorButton::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const QList<DlgWidget>&	zOrdering
    )
{
    Inherited::GenerateSource( stream, varName, zOrdering );

    const KColorButton* widget = GetKColorButton();

    stream << varName << "setColor( QColor( QRgb( " << widget->color().rgb()
	   << " ) ) );" << endl;
}


void DlgKColorButton::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgKColorButton::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
	itsSignals.append( "changed(const QColor&)" );
    }

    return itsSignals;
}


void DlgKColorButton::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    KColorButton* widget = 
	new KColorButton( theParent,
			  CreateName( "KColorButton", 
				      theName, 
				      itsNextWidgetNum ) );

    SetQtWidget( widget, theParent );
}
