/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgKDatePicker.cpp,v 2.3 2000/07/23 20:37:44 jeffharris Exp $

*******************************************************************************/


#include "DlgKDatePicker.h"

#ifndef MODULE_BOOTSTRAP
#include "KDatePickerPropPage.h"
#endif // MODULE_BOOTSTRAP

#include "GuiTools.h"

#define Inherited DlgFrame

QStrList DlgKDatePicker::itsSignals = QStrList();
int DlgKDatePicker::itsNextWidgetNum = 1;

DlgKDatePicker::DlgKDatePicker() :
    Inherited( DlgWidget::DW_Frame )
{
}


DlgKDatePicker::DlgKDatePicker(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgKDatePicker::~DlgKDatePicker()
{
}


void DlgKDatePicker::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

#ifndef MODULE_BOOTSTRAP
    pages.append( new KDatePickerPropPage( this, parent, "KDatePicker" ) );

    labels.append( "KDatePicker" );
#endif // MODULE_BOOTSTRAP
}


void DlgKDatePicker::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
    if( topLevel )
    {
	stream << "KDE::KDatePicker {" << endl;
    }
  
    stream << saveIndent << "InitialDate {";
    if( myInitialDate.isValid() )
    {
	stream << myInitialDate.year() << ' ' 
	       << myInitialDate.month() << ' '
	       << myInitialDate.day();
    }
    stream << "}" << endl;

    Inherited::SaveContents( stream, FALSE );

    if( topLevel )
    {
	stream << "}" << endl;
    }
}


bool DlgKDatePicker::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    // Compare to known keys
    if( key == "InitialDate" )
    {
	if( !value.isEmpty() )
	{
	    int year, month, day;
	    sscanf( value, "%d %d %d", &year, &month, &day );
	    
	    myInitialDate.setYMD( year, month, day );
	}
    } 
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}


void DlgKDatePicker::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const QList<DlgWidget>&	zOrdering
    )
{
    Inherited::GenerateSource( stream, varName, zOrdering );

    //const KDatePicker* widget = GetKDatePicker();

    // Insert source generation code here.
}


void DlgKDatePicker::GenerateCtor(
    QTextStream&	stream,
    const QString&	varName,
    const QString&	parentName
    )
{
    QString name;
    const char* cp = GetWidget()->name();
    if( cp )
    {
	name = "\"";
	name += cp;
	name += "\"";
    }
    else
    {
	name = "NULL";
    }

    stream << '\t' << varName << " = new " << GetWidgetClass() << "( "
	   << parentName << ", ";

    if( myInitialDate.isValid() )
    {
	stream << "QDate( " 
	       << myInitialDate.year() << ", " 
	       << myInitialDate.month() << ", "
	       << myInitialDate.day() << " )";
    }
    else
    {
	stream << "QDate::currentDate()";
    }

    stream << ", " << name << " );\n";
}


void DlgKDatePicker::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgKDatePicker::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
	itsSignals.append( "dateSelected(QDate)" );
    }

    return itsSignals;
}


void DlgKDatePicker::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    KDatePicker* widget = 
	new KDatePicker( theParent,
			 QDate::currentDate(),
			 CreateName( "KDatePicker", 
				     theName, 
				     itsNextWidgetNum ) );

    SetQtWidget( widget, theParent );
}
