/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgKGradientSelector.cpp,v 2.3 2000/07/23 20:37:44 jeffharris Exp $

*******************************************************************************/


#include "DlgKGradientSelector.h"

#ifndef MODULE_BOOTSTRAP
#include "KGradientSelectorPropPage.h"
#endif // MODULE_BOOTSTRAP

#include "GuiTools.h"

#define Inherited DlgKSelector

QStrList DlgKGradientSelector::itsSignals;
int DlgKGradientSelector::itsNextWidgetNum = 1;

DlgKGradientSelector::DlgKGradientSelector() :
    Inherited( DlgWidget::DW_User )
{
}


DlgKGradientSelector::DlgKGradientSelector(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgKGradientSelector::~DlgKGradientSelector()
{
}


void DlgKGradientSelector::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

#ifndef MODULE_BOOTSTRAP
    pages.append( new KGradientSelectorPropPage( this, parent, "KGradientSelector" ) );

    labels.append( "KGradientSelector" );
#endif // MODULE_BOOTSTRAP
}


void DlgKGradientSelector::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
    if( topLevel )
    {
	stream << "KDE::KGradientSelector {" << endl;
    }
  
    stream << saveIndent << "Color1 {" << myColor1.rgb() << "}" << endl;
    stream << saveIndent << "Color2 {" << myColor2.rgb() << "}" << endl;
    stream << saveIndent << "Text1 {" << myText1 << "}" << endl;
    stream << saveIndent << "Text2 {" << myText2 << "}" << endl;

    Inherited::SaveContents( stream, FALSE );

    if( topLevel )
    {
	stream << "}" << endl;
    }
}


bool DlgKGradientSelector::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    // Compare to known keys
    if( key == "Color1" )
    {
	setColor1( QColor( value.toUInt() ) );
    } 
    else if( key == "Color2" )
    {
	setColor2( QColor( value.toUInt() ) );
    } 
    else if( key == "Text1" )
    {
	setText1( value );
    }
    else if( key == "Text2" )
    {
	setText2( value );
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}


void DlgKGradientSelector::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const QList<DlgWidget>&	zOrdering
    )
{
    Inherited::GenerateSource( stream, varName, zOrdering );

    stream << varName << "setColors( "
	   << "QColor( QRgb( " << myColor1.rgb() << " ) ), " 
	   << "QColor( QRgb( " << myColor2.rgb() << " ) ) );" << endl;
    
    stream << varName << "setText( "
	   << "\"" << myText1 << "\", \"" << myText2 << "\" );" << endl;
}


void DlgKGradientSelector::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgKGradientSelector::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
    }

    return itsSignals;
}


void DlgKGradientSelector::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    KGradientSelector* widget = 
	new KGradientSelector( KSelector::Horizontal,
			       theParent,
			       CreateName( "KGradientSelector", 
					   theName, 
					   itsNextWidgetNum ) );

    SetQtWidget( widget, theParent );

    setColor1( white );
    setColor2( darkBlue );
}
