/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgKGradientSelector.h,v 2.3 2000/07/23 20:37:44 jeffharris Exp $

*******************************************************************************/


#ifndef DlgKGradientSelector_included
#define DlgKGradientSelector_included

#include <kselect.h>
#include <qstring.h>
#include <qstrlist.h>

#include "DlgKSelector.h"

class DlgKGradientSelector : public DlgKSelector
{

public:

    DlgKGradientSelector();
    // Constructor

    virtual ~DlgKGradientSelector();
    // Destructor
    

    KGradientSelector* GetKGradientSelector();
    // Get the Qt Widget as KGradientSelector.


    const QColor& getColor1() const;

    void setColor1(
	const QColor&	theColor
	);
    
    const QColor& getColor2() const;
    
    void setColor2(
	const QColor&	theColor
	);
    
    const QString& getText1() const;
    
    void setText1(
	const QString&	theText
	);
    
    const QString& getText2() const;
    
    void setText2(
	const QString&	theText
	);


    //
    // Saving and Loading Methods
    //

    virtual void SaveContents(
	QTextStream&	stream,
	bool		topLevel = TRUE
	);
    // save the contents of the widget

    virtual bool RestoreKeyValue( 
	const QString& 	key, 
	const QString& 	value,
	bool		topLevel = TRUE
	);
    // Restore a single key/value pair


    //
    // Code Generation Methods
    //

    virtual QString GetHeaderFilename() const;
    // return the header filename for the widget

    virtual QString GetWidgetClass() const;
    // return the class name for the widget

    virtual void GenerateSource(
	QTextStream&		stream,
	const QString&		varName,
	const QList<DlgWidget>&	zOrdering
	);
    // Generate the source code needed to initialize the widget.
    // varName already has a leading tab and the correct dereferencer.


    //
    // Property Editing Methods
    //

    virtual void GetPropertyPages(
	QWidget*		parent,
	QList<QWidget>&		pages,
	QStrList&		labels
	);
    // Get the property pages and their labels for the widget


    //
    // Connected Signals List
    //

    virtual void AddSignalsToList( 
	QStrList&	signalList
	) const;
    // add signals specific to a particular widget type to the list

    static const QStrList& GetSignalList();
    // method to return the list of signals


protected: // Methods

    DlgKGradientSelector(
	DlgWidget::Type	theType
	);
    // Constructor.  Used by a derived class' constructor.

    virtual void CreateQtWidget(
	QWidget*	theParent,
	const char*	theName
	);
    // Create and return the correct Qt widget for the specific type of
    // DlgWidget.

private: // data

    QColor 	myColor1;
    QColor	myColor2;
    QString	myText1;
    QString	myText2;

private: // static data

    static QStrList	itsSignals;
    // list of the signals that can be connected to this widget

    static int itsNextWidgetNum;
    // number of the next widget

};


inline QString DlgKGradientSelector::GetHeaderFilename() const
{
    return "<kselect.h>";
}

inline QString DlgKGradientSelector::GetWidgetClass() const
{
    return "KGradientSelector";
}


inline KGradientSelector* DlgKGradientSelector::GetKGradientSelector()
{
    return static_cast<KGradientSelector*> (GetWidget());
}


inline const QColor& DlgKGradientSelector::getColor1() const
{
    return myColor1;
}


inline void DlgKGradientSelector::setColor1(
    const QColor&	theColor
    )
{
    myColor1 = theColor;
    GetKGradientSelector()->setColors( myColor1, myColor2 );
}


inline const QColor& DlgKGradientSelector::getColor2() const
{
    return myColor2;
}


inline void DlgKGradientSelector::setColor2(
    const QColor&	theColor
    )
{
    myColor2 = theColor;
    GetKGradientSelector()->setColors( myColor1, myColor2 );
}


inline const QString& DlgKGradientSelector::getText1() const
{
    return myText1;
}


inline void DlgKGradientSelector::setText1(
    const QString&	theText
    )
{
    myText1 = theText;
    GetKGradientSelector()->setText( myText1, myText2 );
}


inline const QString& DlgKGradientSelector::getText2() const
{
    return myText2;
}


inline void DlgKGradientSelector::setText2(
    const QString&	theText
    )
{
    myText2 = theText;
    GetKGradientSelector()->setText( myText1, myText2 );
}



#endif // DlgKGradientSelector_included
