/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgKIntegerLine.cpp,v 2.3 2000/07/23 20:37:44 jeffharris Exp $

*******************************************************************************/


#include "DlgKIntegerLine.h"

#ifndef MODULE_BOOTSTRAP
#include "KIntegerLinePropPage.h"
#endif // MODULE_BOOTSTRAP

#include "GuiTools.h"

#define Inherited DlgKRestrictedLine

QStrList DlgKIntegerLine::itsSignals;
int DlgKIntegerLine::itsNextWidgetNum = 1;

DlgKIntegerLine::DlgKIntegerLine() :
    Inherited( DlgWidget::DW_LineEdit ),
    myLineType( KEditTypeDec )
{
    SetValidCharsAreGenerated( FALSE );
}


DlgKIntegerLine::DlgKIntegerLine(
    DlgWidget::Type	theType
    ) :
    Inherited( theType ),
    myLineType( KEditTypeDec )
{
    SetValidCharsAreGenerated( FALSE );
}


DlgKIntegerLine::~DlgKIntegerLine()
{
}


void DlgKIntegerLine::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

#ifndef MODULE_BOOTSTRAP
    pages.append( new KIntegerLinePropPage( this, parent, "KIntegerLine" ) );

    labels.append( "KIntegerLine" );
#endif // MODULE_BOOTSTRAP
}


void DlgKIntegerLine::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
    if( topLevel )
    {
	stream << "KDE::KIntegerLine {" << endl;
    }
  
    KIntegerLine* widget = GetKIntegerLine();

    stream << saveIndent << "LineType {";
    switch( myLineType )
    {
	case KEditTypeOct:
	{
	    stream << "KEditTypeOct";
	    break;
	}
	case KEditTypeDec:
	{
	    stream << "KEditTypeDec";
	    break;
	}
	case KEditTypeHex:
	{
	    stream << "KEditTypeHex";
	    break;
	}
    }
    stream << "}" << endl;
    
    stream << saveIndent << "Value {" << widget->value() << "}" << endl;

    Inherited::SaveContents( stream, FALSE );

    if( topLevel )
    {
	stream << "}" << endl;
    }
}


bool DlgKIntegerLine::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    KIntegerLine* widget = GetKIntegerLine();

    // Compare to known keys
    if( key == "LineType" )
    {
	if( value == "KEditTypeOct" )
	{
	    myLineType = KEditTypeOct;
	}
	else if( value == "KEditTypeDec" )
	{
	    myLineType = KEditTypeDec;
	}
	else if( value == "KEditTypeHex" )
	{
	    myLineType = KEditTypeHex;
	}
    } 
    else if( key == "Value" )
    {
	widget->setValue( value.toInt() );
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}


void DlgKIntegerLine::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const QList<DlgWidget>&	zOrdering
    )
{
    Inherited::GenerateSource( stream, varName, zOrdering );

    KIntegerLine* widget = GetKIntegerLine();
    
    stream << varName << "setValue( " << widget->value() << " );" << endl;
}


void DlgKIntegerLine::GenerateCtor(
    QTextStream&	stream,
    const QString&	varName,
    const QString&	parentName
    )
{
    QString name;
    const char* cp = GetWidget()->name();
    if( cp )
    {
	name = "\"";
	name += cp;
	name += "\"";
    }
    else
    {
	name = "NULL";
    }

    stream << '\t' << varName << " = new " << GetWidgetClass() << "( "
	   << parentName << ", " << name << ", ";
    
    switch( myLineType )
    {
	case KEditTypeOct:
	{
	    stream << "KEditTypeOct";
	    break;
	}
	case KEditTypeDec:
	{
	    stream << "KEditTypeDec";
	    break;
	}
	case KEditTypeHex:
	{
	    stream << "KEditTypeHex";
	    break;
	}
    }
    
    stream << " );" << endl;
}


void DlgKIntegerLine::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgKIntegerLine::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
	itsSignals.append( "valueChanged(int)" );
    }

    return itsSignals;
}


void DlgKIntegerLine::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    KIntegerLine* widget = 
	new KIntegerLine( theParent,
			  CreateName( "KIntegerLine", 
				      theName, 
				      itsNextWidgetNum ) );

    SetQtWidget( widget, theParent );
}
