/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgKLedLamp.cpp,v 2.3 2000/07/23 20:37:44 jeffharris Exp $

*******************************************************************************/


#include "DlgKLedLamp.h"

#include "KLedLampPropPage.h"

#include "GuiTools.h"

#define Inherited DlgFrame

QStrList DlgKLedLamp::itsSignals;
int DlgKLedLamp::itsNextWidgetNum = 1;

DlgKLedLamp::DlgKLedLamp() :
    Inherited( DlgWidget::DW_Frame )
{
}


DlgKLedLamp::DlgKLedLamp(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgKLedLamp::~DlgKLedLamp()
{
}


void DlgKLedLamp::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new KLedLampPropPage( this, parent, "KLedLamp" ) );

    labels.append( "KLedLamp" );
}


void DlgKLedLamp::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
    if( topLevel )
    {
	stream << "KDE::KLedLamp {" << endl;
    }
  
    KLedLamp* widget = GetKLedLamp();

    stream << saveIndent << "State {";
    switch( widget->state() )
    {
	case KLedLamp::On:
	{
	    stream << "On";
	    break;
	}
	case KLedLamp::Off:
	{
	    stream << "Off";
	    break;
	}
    }
    stream << "}" << endl;

    Inherited::SaveContents( stream, FALSE );

    if( topLevel )
    {
	stream << "}" << endl;
    }
}


bool DlgKLedLamp::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    KLedLamp* widget = GetKLedLamp();

    // Compare to known keys
    if( key == "State" )
    {
	KLedLamp::State state = KLedLamp::Off;
	if( value == "On" )
	{
	    state = KLedLamp::On;
	}
	else if( value == "Off" )
	{
	    state = KLedLamp::Off;
	}
	
	widget->setState( state );
    } 
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}


void DlgKLedLamp::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const QList<DlgWidget>&	zOrdering
    )
{
    Inherited::GenerateSource( stream, varName, zOrdering );

    const KLedLamp* widget = GetKLedLamp();

    stream << varName << "setState( KLedLamp::";
    switch( widget->state() )
    {
	case KLedLamp::On:
	{
	    stream << "On";
	    break;
	}
	case KLedLamp::Off:
	{
	    stream << "Off";
	    break;
	}
    }
    stream << " );" << endl;
}


void DlgKLedLamp::GenerateCtor(
    QTextStream&	stream,
    const QString&	varName,
    const QString&	parentName
    )
{
    QString name;
    const char* cp = GetWidget()->name();
    if( cp )
    {
	name = "\"";
	name += cp;
	name += "\"";
    }
    else
    {
	name = "NULL";
    }

    stream << '\t' << varName << " = new " << GetWidgetClass() << "( "
	   << parentName << " );" << endl;

    stream << '\t' << varName << "->setName( " << name << " );" << endl;
}


void DlgKLedLamp::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgKLedLamp::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
    }

    return itsSignals;
}


void DlgKLedLamp::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    KLedLamp* widget = 
	new KLedLamp( theParent );

    widget->setName( CreateName( "KLedLamp", 
				 theName, 
				 itsNextWidgetNum ) );

    SetQtWidget( widget, theParent );
}
