/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgKNumCheckButton.h,v 2.3 2000/07/23 20:37:44 jeffharris Exp $

*******************************************************************************/


#ifndef DlgKNumCheckButton_included
#define DlgKNumCheckButton_included

#include <ktablistbox.h>
#include <qstring.h>
#include <qstrlist.h>

#include "widget/DlgWidget.h"

class DlgKNumCheckButton : public DlgWidget
{

public:

    DlgKNumCheckButton();
    // Constructor

    virtual ~DlgKNumCheckButton();
    // Destructor
    

    KNumCheckButton* GetKNumCheckButton();
    // Get the Qt Widget as KNumCheckButton.

    const QString& GetText() const;
    
    void SetText(
	const QString&	theText
	);

    //
    // Saving and Loading Methods
    //

    virtual void SaveContents(
	QTextStream&	stream,
	bool		topLevel = TRUE
	);
    // save the contents of the widget

    virtual bool RestoreKeyValue( 
	const QString& 	key, 
	const QString& 	value,
	bool		topLevel = TRUE
	);
    // Restore a single key/value pair


    //
    // Code Generation Methods
    //

    virtual QString GetHeaderFilename() const;
    // return the header filename for the widget

    virtual QString GetWidgetClass() const;
    // return the class name for the widget

    virtual void GenerateSource(
	QTextStream&		stream,
	const QString&		varName,
	const QList<DlgWidget>&	zOrdering
	);
    // Generate the source code needed to initialize the widget.
    // varName already has a leading tab and the correct dereferencer.


    //
    // Property Editing Methods
    //

    virtual void GetPropertyPages(
	QWidget*		parent,
	QList<QWidget>&		pages,
	QStrList&		labels
	);
    // Get the property pages and their labels for the widget


    //
    // Connected Signals List
    //

    virtual void AddSignalsToList( 
	QStrList&	signalList
	) const;
    // add signals specific to a particular widget type to the list

    static const QStrList& GetSignalList();
    // method to return the list of signals


protected: // Methods

    DlgKNumCheckButton(
	DlgWidget::Type	theType
	);
    // Constructor.  Used by a derived class' constructor.

    virtual void CreateQtWidget(
	QWidget*	theParent,
	const char*	theName
	);
    // Create and return the correct Qt widget for the specific type of
    // DlgWidget.


private:

    QString	myText;
    // The text of the check button


private: // static data

    static QStrList	itsSignals;
    // list of the signals that can be connected to this widget

    static int itsNextWidgetNum;
    // number of the next widget

};


inline QString DlgKNumCheckButton::GetHeaderFilename() const
{
    return "<ktablistbox.h>";
}

inline QString DlgKNumCheckButton::GetWidgetClass() const
{
    return "KNumCheckButton";
}


inline KNumCheckButton* DlgKNumCheckButton::GetKNumCheckButton()
{
    return static_cast<KNumCheckButton*> (GetWidget());
}


inline const QString& DlgKNumCheckButton::GetText() const
{
    return myText;
}


inline void DlgKNumCheckButton::SetText(
    const QString&	theText
    )
{
    myText = theText;
    GetKNumCheckButton()->setText( myText );
}


#endif // DlgKNumCheckButton_included
