/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgKNumericSpinBox.cpp,v 2.3 2000/07/23 20:37:44 jeffharris Exp $

*******************************************************************************/


#include "DlgKNumericSpinBox.h"

#include "KNumericSpinBoxPropPage.h"

#include "GuiTools.h"

#define Inherited DlgKSpinBox

QStrList DlgKNumericSpinBox::itsSignals;
int DlgKNumericSpinBox::itsNextWidgetNum = 1;

DlgKNumericSpinBox::DlgKNumericSpinBox() :
    Inherited( DlgWidget::DW_User )
{
}


DlgKNumericSpinBox::DlgKNumericSpinBox(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgKNumericSpinBox::~DlgKNumericSpinBox()
{
}


void DlgKNumericSpinBox::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new KNumericSpinBoxPropPage( this, parent, "KNumericSpinBox" ) );

    labels.append( "KNumericSpinBox" );
}


void DlgKNumericSpinBox::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
    if( topLevel )
    {
	stream << "KDE::KNumericSpinBox {" << endl;
    }
  
    KNumericSpinBox* widget = GetKNumericSpinBox();

    stream << saveIndent << "NumValue {" << widget->getValue() << "}" << endl;
    
    stream << saveIndent << "Step {" << widget->getStep() << "}" << endl;
    
    int min, max;
    widget->getRange( min, max );    
    stream << saveIndent << "Min {" << min << "}" << endl;
    stream << saveIndent << "Max {" << max << "}" << endl;

    Inherited::SaveContents( stream, FALSE );

    if( topLevel )
    {
	stream << "}" << endl;
    }
}


bool DlgKNumericSpinBox::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    KNumericSpinBox* widget = GetKNumericSpinBox();

    // Compare to known keys
    if( key == "NumValue" )
    {
	widget->setValue( value.toInt() );
    }
    else if( key == "Step" )
    {
	widget->setStep( value.toInt() );
    }
    else if( key == "Min" )
    {
	int min, max;
	widget->getRange( min, max );
	widget->setRange( value.toInt(), max );
    }
    else if( key == "Max" )
    {
	int min, max;
	widget->getRange( min, max );
	widget->setRange( min, value.toInt() );
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}


void DlgKNumericSpinBox::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const QList<DlgWidget>&	zOrdering
    )
{
    Inherited::GenerateSource( stream, varName, zOrdering );

    KNumericSpinBox* widget = GetKNumericSpinBox();

    stream << varName << "setValue( " << widget->getValue() << " );" << endl;
    stream << varName << "setStep( " << widget->getStep() << " );" << endl;
    
    int min, max;
    widget->getRange( min, max );
    stream << varName << "setRange( " << min << ", " << max << " );" << endl;
}


void DlgKNumericSpinBox::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgKNumericSpinBox::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
    }

    return itsSignals;
}


void DlgKNumericSpinBox::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    KNumericSpinBox* widget = 
	new KNumericSpinBox( theParent,
			     CreateName( "KNumericSpinBox", 
					 theName, 
					 itsNextWidgetNum ) );

    SetQtWidget( widget, theParent );
}
