/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgKRestrictedLine.cpp,v 2.3 2000/07/23 20:37:44 jeffharris Exp $

*******************************************************************************/


#include "DlgKRestrictedLine.h"

#include "KRestrictedLinePropPage.h"

#include "GuiTools.h"

#define Inherited DlgLineEdit

QStrList DlgKRestrictedLine::itsSignals;
int DlgKRestrictedLine::itsNextWidgetNum = 1;

DlgKRestrictedLine::DlgKRestrictedLine() :
    Inherited( DlgWidget::DW_LineEdit ),
    areValidCharsGenerated( TRUE )
{
}


DlgKRestrictedLine::DlgKRestrictedLine(
    DlgWidget::Type	theType
    ) :
    Inherited( theType ),
    areValidCharsGenerated( TRUE )
{
}


DlgKRestrictedLine::~DlgKRestrictedLine()
{
}


void DlgKRestrictedLine::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new KRestrictedLinePropPage( this, parent, "KRestrictedLine" ) );

    labels.append( "KRestrictedLine" );
}


void DlgKRestrictedLine::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
    if( topLevel )
    {
	stream << "KDE::KRestrictedLine {" << endl;
    }

    stream << saveIndent << "ValidChars {" << myValidChars << "}" << endl;
  
    Inherited::SaveContents( stream, FALSE );

    if( topLevel )
    {
	stream << "}" << endl;
    }
}


bool DlgKRestrictedLine::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    // Compare to known keys
    if( key == "ValidChars" )
    {
	myValidChars = value;
    } 
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}


void DlgKRestrictedLine::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const QList<DlgWidget>&	zOrdering
    )
{
    Inherited::GenerateSource( stream, varName, zOrdering );

    if( areValidCharsGenerated )
    {
	stream << varName << "setValidChars( \"" << myValidChars << "\" );" 
	       << endl;
    }
}


void DlgKRestrictedLine::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgKRestrictedLine::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
	itsSignals.append( "invalidChar(int)" );
    }

    return itsSignals;
}


void DlgKRestrictedLine::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    KRestrictedLine* widget = 
	new KRestrictedLine( theParent,
			     CreateName( "KRestrictedLine", 
					 theName, 
					 itsNextWidgetNum ) );

    SetQtWidget( widget, theParent );
}
