/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgKRuler.cpp,v 2.3 2000/07/23 20:37:44 jeffharris Exp $

*******************************************************************************/


#include "DlgKRuler.h"

#include "KRulerPropPage.h"

#include "GuiTools.h"

#define Inherited DlgFrame

QStrList DlgKRuler::itsSignals;
int DlgKRuler::itsNextWidgetNum = 1;

DlgKRuler::DlgKRuler() :
    Inherited( DlgWidget::DW_Frame )
{
}


DlgKRuler::DlgKRuler(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgKRuler::~DlgKRuler()
{
}


void DlgKRuler::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new KRulerPropPage( this, parent, "KRuler" ) );

    labels.append( "KRuler" );
}


void DlgKRuler::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
    if( topLevel )
    {
	stream << "KDE::KRuler {" << endl;
    }
  
    KRuler* widget = GetKRuler();

    Inherited::SaveContents( stream, FALSE );

    if( topLevel )
    {
	stream << "}" << endl;
    }
}


bool DlgKRuler::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    KRuler* widget = GetKRuler();

    // Compare to known keys
    if( key == "Color" )
    {
    } 
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}


void DlgKRuler::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const QList<DlgWidget>&	zOrdering
    )
{
    Inherited::GenerateSource( stream, varName, zOrdering );

    const KRuler* widget = GetKRuler();
}


void DlgKRuler::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgKRuler::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
	itsSignals.append( "changed(const QColor&)" );
    }

    return itsSignals;
}


void DlgKRuler::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    KRuler* widget = 
	new KRuler( KRuler::horizontal,
		    theParent,
		    CreateName( "KRuler", 
				theName, 
				itsNextWidgetNum ) );

    SetQtWidget( widget, theParent );
}
