/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgKSeparator.cpp,v 2.3 2000/07/23 20:37:44 jeffharris Exp $

*******************************************************************************/


#include "DlgKSeparator.h"

#include "KSeparatorPropPage.h"

#include "GuiTools.h"

#define Inherited DlgFrame

QStrList DlgKSeparator::itsSignals;
int DlgKSeparator::itsNextWidgetNum = 1;

DlgKSeparator::DlgKSeparator() :
    Inherited( DlgWidget::DW_Frame )
{
}


DlgKSeparator::DlgKSeparator(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgKSeparator::~DlgKSeparator()
{
}


void DlgKSeparator::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    DlgWidget::GetPropertyPages( parent, pages, labels );

    pages.append( new KSeparatorPropPage( this, parent, "KSeparator" ) );

    labels.append( "KSeparator" );
}


void DlgKSeparator::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
    if( topLevel )
    {
	stream << "KDE::KSeparator {" << endl;
    }
  
    Inherited::SaveContents( stream, FALSE );

    if( topLevel )
    {
	stream << "}" << endl;
    }
}


bool DlgKSeparator::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    retVal = Inherited::RestoreKeyValue( key, value, FALSE );

    return retVal;
}


void DlgKSeparator::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const QList<DlgWidget>&	zOrdering
    )
{
    Inherited::GenerateSource( stream, varName, zOrdering );
}


void DlgKSeparator::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgKSeparator::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
    }

    return itsSignals;
}


void DlgKSeparator::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    KSeparator* widget = 
	new KSeparator( theParent,
			CreateName( "KSeparator", 
				    theName, 
				    itsNextWidgetNum ) );

    SetQtWidget( widget, theParent );
}
