/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgKTabListBox.cpp,v 2.3 2000/07/23 20:37:44 jeffharris Exp $

*******************************************************************************/


#include "DlgKTabListBox.h"

#ifndef MODULE_BOOTSTRAP
#include "KTabListBoxPropPage.h"
#endif // MODULE_BOOTSTRAP

#include "GuiTools.h"

#define Inherited DlgWidget

QStrList DlgKTabListBox::itsSignals;
int DlgKTabListBox::itsNextWidgetNum = 1;

DlgKTabListBox::DlgKTabListBox() :
    Inherited( DlgWidget::DW_User )
{
}


DlgKTabListBox::DlgKTabListBox(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgKTabListBox::~DlgKTabListBox()
{
}


void DlgKTabListBox::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

#ifndef MODULE_BOOTSTRAP
    pages.append( new KTabListBoxPropPage( this, parent, "KTabListBox" ) );

    labels.append( "KTabListBox" );
#endif // MODULE_BOOTSTRAP
}


void DlgKTabListBox::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
    if( topLevel )
    {
	stream << "KDE::KTabListBox {" << endl;
    }
  
    KTabListBox* widget = GetKTabListBox();

    stream << saveIndent << "NumColumns {" << widget->numCols() << "}" << endl;
    stream << saveIndent << "NumRows {" << widget->numRows() << "}" << endl;
    
    stream << saveIndent << "TableFlags {" 
	   << widget->tableFlags() << "}" << endl;
    
    stream << saveIndent << "AutoUpdate {";
    SaveBool( stream, widget->autoUpdate() );
    stream << "}" << endl;

    Inherited::SaveContents( stream, FALSE );

    if( topLevel )
    {
	stream << "}" << endl;
    }
}


bool DlgKTabListBox::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    KTabListBox* widget = GetKTabListBox();

    // Compare to known keys
    if( key == "NumColumns" )
    {
	widget->setNumCols( value.toInt() );
    } 
    else if( key == "NumRows" )
    {
	widget->setNumRows( value.toInt() );
    }
    else if( key == "TableFlags" )
    {
	widget->setTableFlags( value.toInt() );
    }
    else if( key == "AutoUpdate" )
    {
	widget->setAutoUpdate( (value == "TRUE") );
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}


void DlgKTabListBox::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const QList<DlgWidget>&	zOrdering
    )
{
    Inherited::GenerateSource( stream, varName, zOrdering );

    KTabListBox* widget = GetKTabListBox();

    stream << varName << "setNumCols( " << widget->numCols() << " );" << endl;
    stream << varName << "setNumRows( " << widget->numRows() << " );" << endl;

    stream << varName << "setTableFlags( " 
	   << widget->tableFlags() << " );" << endl;
    
    stream << varName << "setAutoUpdate( "
	   << (widget->autoUpdate() ? "TRUE" : "FALSE") << " );" << endl;
}


void DlgKTabListBox::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgKTabListBox::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
	itsSignals.append( "highlighted(int,int)" );
	itsSignals.append( "selected(int,int)" );
	itsSignals.append( "popupMenu(int,int)" );
	itsSignals.append( "midClick(int,int)" );
	itsSignals.append( "headerClicked(int)" );
    }

    return itsSignals;
}


void DlgKTabListBox::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    KTabListBox* widget = 
	new KTabListBox( theParent,
			 CreateName( "KTabListBox", 
				     theName, 
				     itsNextWidgetNum ) );

    SetQtWidget( widget, theParent );
}
