/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: KDEModule.cpp,v 2.2 2000/07/23 20:37:44 jeffharris Exp $

*******************************************************************************/


#include <kapp.h>

#include <qpixmap.h>

#include "module/WidgetModule.h"

#include "DlgKColorButton.h"
#include "DlgKDatePicker.h"
#include "DlgKDirectionButton.h"
#include "DlgKGradientSelector.h"
//#include "DlgKIconLoaderButton.h"
#include "DlgKIntLineEdit.h"
#include "DlgKIntegerLine.h"
#include "DlgKKeyButton.h"
#include "DlgKLed.h"
#include "DlgKLedLamp.h"
#include "DlgKLined.h"
#include "DlgKListSpinBox.h"
#include "DlgKNumCheckButton.h"
#include "DlgKNumericSpinBox.h"
#include "DlgKProgress.h"
#include "DlgKRestrictedLine.h"
#include "DlgKSeparator.h"
#include "DlgKSlider.h"
#include "DlgKSpinBox.h"
#include "DlgKTabListBox.h"
#include "DlgKTreeList.h"
#include "DlgKURLLabel.h"


class KDEWidgetModule : public WidgetModule
{
public:

    KDEWidgetModule();
    
    virtual ~KDEWidgetModule();

    virtual QString getName();

    virtual bool useToolbar();

    virtual QStrList getWidgetNames();

    virtual QPixmap getWidgetPixmap(
	const QString&	theWidgetName
	);
    
    virtual DlgWidget* newWidget(
	int	theWidgetNum
	);

    virtual DlgWidget* newWidget(
	const QString&	theWidgetName
	);

private:

    QStrList	myWidgetNames;
    // The names of all the widgets generated by this module
};


KDEWidgetModule::KDEWidgetModule()
{
    myWidgetNames.append( "KColorButton" );
    myWidgetNames.append( "KDatePicker" );
    myWidgetNames.append( "KDirectionButton" );
    myWidgetNames.append( "KGradientSelector" );
//    myWidgetNames.append( "KIconLoaderButton" );
    myWidgetNames.append( "KIntLineEdit" );
    myWidgetNames.append( "KIntegerLine" );
    myWidgetNames.append( "KKeyButton" );
    myWidgetNames.append( "KLed" );
    myWidgetNames.append( "KLedLamp" );
    myWidgetNames.append( "KLined" );
    myWidgetNames.append( "KListSpinBox" );
    myWidgetNames.append( "KNumCheckButton" );
    myWidgetNames.append( "KNumericSpinBox" );
    myWidgetNames.append( "KProgress" );
    myWidgetNames.append( "KRestrictedLine" );
    myWidgetNames.append( "KSeparator" );
    myWidgetNames.append( "KSlider" );
    myWidgetNames.append( "KSpinBox" );
    myWidgetNames.append( "KTabListBox" );
    myWidgetNames.append( "KTreeList" );
    myWidgetNames.append( "KURLLabel" );
}


KDEWidgetModule::~KDEWidgetModule()
{
}


QString KDEWidgetModule::getName()
{
    return "KDE";
}


bool KDEWidgetModule::useToolbar()
{
    return FALSE;
}


QStrList KDEWidgetModule::getWidgetNames()
{
    return myWidgetNames;
}


QPixmap KDEWidgetModule::getWidgetPixmap(
    const QString&	/*theWidgetName*/
    )
{
    return QPixmap();
}


DlgWidget* KDEWidgetModule::newWidget(
    int	theWidgetNum
    )
{
    return newWidget( myWidgetNames.at( theWidgetNum ) );
}


DlgWidget* KDEWidgetModule::newWidget(
    const QString&	theModuleName
    )
{
    DlgWidget* dlgWidget = 0;
    
#define FIRST_NEW_WIDGET( widget ) \
    if( theModuleName == #widget ) \
        dlgWidget = new Dlg##widget();

#define NEW_WIDGET( widget ) \
    else if( theModuleName == #widget ) \
	dlgWidget = new Dlg##widget(); \


    FIRST_NEW_WIDGET( KColorButton )
	NEW_WIDGET( KDatePicker )
	NEW_WIDGET( KDirectionButton )
	NEW_WIDGET( KGradientSelector )
//	NEW_WIDGET( KIconLoaderButton )
	NEW_WIDGET( KIntLineEdit )
	NEW_WIDGET( KIntegerLine )
	NEW_WIDGET( KKeyButton )
	NEW_WIDGET( KLed )
	NEW_WIDGET( KLedLamp )
	NEW_WIDGET( KLined )
	NEW_WIDGET( KListSpinBox )
	NEW_WIDGET( KNumCheckButton )
	NEW_WIDGET( KNumericSpinBox )
	NEW_WIDGET( KProgress )
	NEW_WIDGET( KRestrictedLine )
	NEW_WIDGET( KSeparator )
	NEW_WIDGET( KSlider )
	NEW_WIDGET( KSpinBox )
	NEW_WIDGET( KTabListBox )
	NEW_WIDGET( KTreeList )
	NEW_WIDGET( KURLLabel )
    
    return dlgWidget;
}


void GetModuleVersion(
    int&	theMajorVersion,
    int&	theMinorVersion
    )
{
    theMajorVersion = 1;
    theMinorVersion = 0;
}


WidgetModule* GetWidgetModule()
{
    return new KDEWidgetModule();
}


QApplication* GetQApplication( 
    int&	theArgc,
    char**	theArgv
    )
{
    return new KApplication( theArgc, theArgv, "QtArchitect" );
}


