/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: TestPushButton.cpp,v 2.4 2000/07/23 20:37:44 jeffharris Exp $

*******************************************************************************/


#include "module/test/TestPushButton.h"

#include "prop/PushButtonPage.h"

#include "GuiTools.h"

#include "xml/QtArchExpat.h"
#include "xml/QtArchGenerator.h"

#define Inherited DlgButton

QStrList TestPushButton::itsSignals;
int TestPushButton::itsNextWidgetNum = 1;

TestPushButton::TestPushButton() :
    Inherited( DlgWidget::DW_PushButton ),
    isDefault( FALSE ),
    isAutoDefault( FALSE )
{
}


TestPushButton::TestPushButton(
    DlgWidget::Type	theType
    ) :
    Inherited( theType ),
    isDefault( FALSE ),
    isAutoDefault( FALSE )
{
}


TestPushButton::~TestPushButton()
{
}


void TestPushButton::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    DlgWidget::GetPropertyPages( parent, pages, labels );

    pages.append( new PushButtonPage( this, parent, "Test PushButton" ) );

    labels.append( tr( "Test Push Button" ) );
}

void TestPushButton::Save( QtArchGenerator &generator )
{
  generator.WriteStartBlock( "TEST::TestPushButton" );
  QPushButton* widget = GetPushButton();

  generator.WriteXMLValue( "ToggleButton", widget->isToggleButton() );
  generator.WriteXMLValue( "Default", isDefault );
  generator.WriteXMLValue( "AutoDefault", isAutoDefault );

  Inherited::Save( generator );

  generator.WriteEndBlock();
}


bool TestPushButton::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
        if( key == "Name" )
        {
            // Update the widget number so that no duplicate names are created
            UpdateNextWidgetNumber( value, itsNextWidgetNum );
        }
    }

    QPushButton* widget = GetPushButton();

    // Compare to known keys
    if( key == "ToggleButton" )
    {
        widget->setToggleButton( (value == "TRUE") );
    } 
    else if( key == "Default" )
    {
        SetIsDefault( (value == "TRUE" ) );
    }
    else if( key == "AutoDefault" )
    {
        SetIsAutoDefault( (value == "TRUE") );
    }
    else if( key == "IsMenuButton" )
    {
        // Deprecated
    }
    else
    {
        retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}

void TestPushButton::RestoreXML( const XMLValue &value )
{
  // Sanity check
  if( value.GetTypeString() != "TEST::TestPushButton" )
    return;

  const XMLValueMap map = value.AsNested();

  // Create if necessary
  if( ! GetPushButton() )
  {
    CreateQtWidget( 0 );
  }
  
  QPushButton* widget = GetPushButton();

  widget->setToggleButton( map["ToggleButton"].AsBool() );
  SetIsDefault( map["Default"].AsBool() );
  SetIsAutoDefault( map["AutoDefault"].AsBool() );

  // We inherit from 
	Inherited::RestoreXML( map["Button"] );

}


void TestPushButton::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    const QPushButton* widget = GetPushButton();

    stream << varName << "setToggleButton( " << 
	(widget->isToggleButton() ? "TRUE" : "FALSE") << " );" << endl;

    stream << varName << "setDefault( " << 
	(isDefault ? "TRUE" : "FALSE") << " );" << endl;

    stream << varName << "setAutoDefault( " << 
	(isAutoDefault ? "TRUE" : "FALSE") << " );" << endl;
}


void TestPushButton::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& TestPushButton::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
    }

    return itsSignals;
}


void TestPushButton::CreateQtWidget(
    const char*	theName
    )
{
    QPushButton* widget = 
	new QPushButton( GetParent(),
			 CreateName( "TestPushButton", 
				     theName, 
				     itsNextWidgetNum ) );

    SetQtWidget( widget );
}
