/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: TestPushButton.h,v 2.4 2000/07/23 20:37:44 jeffharris Exp $

*******************************************************************************/


#ifndef TestPushButton_included
#define TestPushButton_included

#include <qpushbutton.h>
#include <qstring.h>
#include <qstrlist.h>

#include "widget/DlgButton.h"

class TestPushButton : public DlgButton
{

public:

    TestPushButton();
    // Constructor

    virtual ~TestPushButton();
    // Destructor
    

    //
    // Saving and Loading Methods
    //

    virtual void Save( QtArchGenerator & );
    // save the contents of the widget

    virtual bool RestoreKeyValue( 
	const QString& 	key, 
	const QString& 	value,
	bool		topLevel = TRUE
	);
    // Restore a single key/value pair

    virtual void RestoreXML( const  XMLValue & );
    // Restore a XMLValue


    //
    // Code Generation Methods
    //

    virtual QString GetHeaderFilename() const;
    // return the header filename for the widget

    virtual QString GetWidgetClass() const;
    // return the class name for the widget

    virtual void GenerateSource(
	QTextStream&		stream,
	const QString&		varName,
        const WidgetLayout&     widgetLayout
	);
    // Generate the source code needed to initialize the widget.
    // varName already has a leading tab and the correct dereferencer.


    //
    // Property Editing Methods
    //

    virtual void GetPropertyPages(
	QWidget*		parent,
	QList<QWidget>&		pages,
	QStrList&		labels
	);
    // Get the property pages and their labels for the widget

    
    //
    // Properties
    //

    bool IsDefault() const;
    void SetIsDefault( bool isDefaultButton );
    // Default button

    bool IsAutoDefault() const;
    void SetIsAutoDefault( bool autoDefault );
    // Auto Default button


    //
    // Connected Signals List
    //

    virtual void AddSignalsToList( 
	QStrList&	signalList
	) const;
    // add signals specific to a particular widget type to the list

    static const QStrList& GetSignalList();
    // method to return the list of signals


protected: // Methods

    TestPushButton(
	DlgWidget::Type	theType
	);
    // Constructor.  Used by a derived class' constructor.

    virtual void CreateQtWidget(
	const char*	theName
	);
    // Create and return the correct Qt widget for the specific type of
    // DlgWidget.


private: // Methods

    QPushButton* GetPushButton();
    // Get the Qt Widget as QPushButton.

private: // Properties

    bool    isDefault;
    
    bool    isAutoDefault;

private: // static data

    static QStrList	itsSignals;
    // list of the signals that can be connected to this widget

    static int itsNextWidgetNum;
    // number of the next widget

};


inline QString TestPushButton::GetHeaderFilename() const
{
    return "<qpushbutton.h>";
}


inline QString TestPushButton::GetWidgetClass() const
{
    return "QPushButton";
}


inline bool TestPushButton::IsDefault() const
{
    return isDefault;
}


inline void TestPushButton::SetIsDefault( bool isDefaultButton )
{
    isDefault = isDefaultButton;
}


inline bool TestPushButton::IsAutoDefault() const
{
    return isAutoDefault;
}


inline void TestPushButton::SetIsAutoDefault( bool autoDefault )
{
    isAutoDefault = autoDefault;
}


inline QPushButton* TestPushButton::GetPushButton()
{
    return static_cast<QPushButton*> (GetWidget());
}


#endif // TestPushButton_included
