/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: ColorFrame.h,v 2.2 2000/07/23 20:37:45 jeffharris Exp $

*******************************************************************************/


#ifndef ColorFrame_included
#define ColorFrame_included

#include <qframe.h>
#include <qpushbt.h>
#include <qlabel.h>

const int PredefinedColors = 19;

class ColorButton : public QPushButton
{
  Q_OBJECT
public:
  ColorButton( QWidget *, const QColor & );

private:
  virtual void mousePressEvent( QMouseEvent * );

signals:
  void ColorClicked( int, QWidget * ); 

};

class EditButton : public QPushButton
{
  Q_OBJECT
public:
  EditButton( const QString &, QWidget * );

private:
  virtual void mousePressEvent( QMouseEvent * );

signals:
  void Clicked( int ); 

};

class ColorFrame : public QFrame
{
  Q_OBJECT
public:
  ColorFrame( QWidget *parent );
  
  void SetLeftColor( const QColor & );
  void SetRightColor( const QColor & );

signals:
  void ColorChanged( int, const QColor & );

private slots:
  void ColorClicked( int, QWidget * );
  void EditColor( int );

private: // Attributes
  QPushButton *editButton;
  QWidget *lastColorClicked;
  QLabel *leftColor, *rightColor;
};

inline void ColorFrame::SetLeftColor( const QColor &c )
{
  leftColor->setBackgroundColor( c );
}

inline void ColorFrame::SetRightColor( const QColor &c )
{
  rightColor->setBackgroundColor( c );
}

#endif
