/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DialogBasePage.cpp,v 2.3 2000/07/23 20:37:45 jeffharris Exp $

*******************************************************************************/


#include "DialogBasePage.h"

#include "Dialog.h"
#include "DialogWnd.h"

#define Inherited DialogBasePageData

DialogBasePage::DialogBasePage
(
 Dialog* dialog,
 QWidget* parent,
 const char* name
 )
  :
  Inherited( parent, name )
{
  switch( dialog->GetWindowType() )
  {
	case Dialog::WT_Dialog:
    {
	    isQDialog->setChecked( TRUE );
	    break;
    }
	case Dialog::WT_TabDialog:
    {
	    isQTabDialog->setChecked( TRUE );
	    break;
    }
	case Dialog::WT_Widget:
    {
	    isQWidget->setChecked( TRUE );
	    break;
    }
	case Dialog::WT_Frame:
    {
	    isQFrame->setChecked( TRUE );
	    break;
    }
  case Dialog::WT_Custom:
    {
	    isCustom->setChecked( TRUE );
	    break;
    }
  }
    
  isModalDialog->setChecked( dialog->IsDialogModal() );
  QString cl, clHdr;

  dialog->GetCustomBase( cl, clHdr );
    
  customClass->setText( cl );
  customClassHeader->setText( clHdr );
    
  if( isCustom->isChecked() )
  {
    customClass->setEnabled( TRUE );
    customClassHeader->setEnabled( TRUE );
  } else
  {
    customClass->setEnabled( FALSE );
    customClassHeader->setEnabled( FALSE );
  }
}


DialogBasePage::~DialogBasePage()
{
}

void DialogBasePage::UseCustom( bool useIt )
{
  customClass->setEnabled( useIt );
  customClassHeader->setEnabled( useIt );
}

void DialogBasePage::Apply( Dialog* dialog )
{
    Dialog::WindowType type = Dialog::WT_Dialog;
    
    if( isQTabDialog->isChecked() )
    {
	type = Dialog::WT_TabDialog;
    }
    else if( isQWidget->isChecked() )
    {
	type = Dialog::WT_Widget;
    }
    else if( isQFrame->isChecked() )
    {
	type = Dialog::WT_Frame;
    }
    else if( isCustom->isChecked() )
    {
	type = Dialog::WT_Custom;
    }
    
    dialog->SetWindowType( type );
    
    dialog->SetIsDialogModal( isModalDialog->isChecked() );

    dialog->SetCustomBase( customClass->text(), customClassHeader->text() );
}

