/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DialogGenPage.cpp,v 2.6 2000/07/23 20:37:45 jeffharris Exp $

*******************************************************************************/


#include "prop/DialogGenPage.h"

#include "Dialog.h"
#include "ProjectWnd.h"

#define Inherited DialogGenPageData

DialogGenPage::DialogGenPage
(
    Dialog* dialog,
    QWidget* parent,
    const char* name
)
    :
    Inherited( parent, name ),
    canModClassFiles( TRUE ),
    canModDataName( TRUE ),
    canModDataFiles( TRUE )
{
    itsClassName->setText( dialog->GetClassName() );
    itsClassHeader->setText( dialog->GetClassHeader() );
    itsClassSource->setText( dialog->GetClassSource() );
    itsDataName->setText( dialog->GetDataName() );
    itsDataHeader->setText( dialog->GetDataHeader() );
    itsDataSource->setText( dialog->GetDataSource() );
    isTextTranslated->setChecked( dialog->IsTextTranslated() );
    
    connect( itsClassName, SIGNAL(textChanged(const QString&)),
	     SLOT(ClassNameChanged(const QString&)) );
    connect( itsClassHeader, SIGNAL(textChanged(const QString&)),
	     SLOT(ClassFilesKeyPressed(const QString&)) );
    connect( itsClassSource, SIGNAL(textChanged(const QString&)),
	     SLOT(ClassFilesKeyPressed(const QString&)) );
    connect( itsDataName, SIGNAL(textChanged(const QString&)),
	     SLOT(DataNameChanged(const QString&)) );
    connect( itsDataHeader, SIGNAL(textChanged(const QString&)),
	     SLOT(DataFilesKeyPressed(const QString&)) );
    connect( itsDataSource, SIGNAL(textChanged(const QString&)),
	     SLOT(DataFilesKeyPressed(const QString&)) );

}


DialogGenPage::~DialogGenPage()
{
}


void DialogGenPage::Apply( Dialog* dialog )
{
    dialog->SetClassName( itsClassName->text() );
    dialog->SetClassHeader( itsClassHeader->text() );
    dialog->SetClassSource( itsClassSource->text() );
    dialog->SetDataName( itsDataName->text() );
    dialog->SetDataHeader( itsDataHeader->text() );
    dialog->SetDataSource( itsDataSource->text() );
    dialog->SetTextTranslated( isTextTranslated->isChecked() );
}


void DialogGenPage::ClassNameChanged( const QString& text )
{
    QString str = text;
    
    if( canModClassFiles )
    {
	itsClassHeader->setText( str + "." + ProjectWnd::GetHeaderFileSuffix() );
	itsClassSource->setText( str + "." + ProjectWnd::GetSourceFileSuffix() );
    }
    
    if( canModDataName )
    {
	itsDataName->setText( tr( "%1Data" ).arg( str ) );
    }
}


void DialogGenPage::ClassFilesKeyPressed( const QString& )
{
    QString header = itsClassName->text();
    header += "." + ProjectWnd::GetHeaderFileSuffix();
    
    QString source = itsClassName->text();
    source += "." + ProjectWnd::GetSourceFileSuffix();
    
    canModClassFiles = ( (header == itsClassHeader->text()) &&
			 (source == itsClassSource->text()) );
}


void DialogGenPage::DataNameChanged( const QString& text )
{
    QString str = tr( "%1Data" ).arg( itsClassName->text() );
    
    canModDataName = (str == text);

    if( canModDataFiles )
    {
	str = text;
	
	itsDataHeader->setText( str + "." + ProjectWnd::GetHeaderFileSuffix() );
	itsDataSource->setText( str + "." + ProjectWnd::GetSourceFileSuffix() );
    }
}


void DialogGenPage::DataFilesKeyPressed( const QString& )
{
    QString header = itsDataName->text();
    header += "." + ProjectWnd::GetHeaderFileSuffix();
    
    QString source = itsDataName->text();
    source += "." + ProjectWnd::GetSourceFileSuffix();
    
    canModDataFiles = ( (header == itsDataHeader->text()) &&
			(source == itsDataSource->text()) );
}

