/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: FramePage.cpp,v 2.4 2000/07/23 20:37:45 jeffharris Exp $

*******************************************************************************/


#include <qframe.h>
#include <qstring.h>

#include "FramePage.h"

#include "widget/DlgWidget.h"

#define Inherited FramePageData

FramePage::FramePage
(
    DlgWidget* widget,
    QWidget* parent,
    const char* name
)
    :
    Inherited( parent, name )
{
  connect( itsShapes, SIGNAL(activated(int)), 
           SLOT(ShapeChanged(int)) );
  connect( itsShadows, SIGNAL(activated(int)),
           SLOT(ShadowChanged(int)) );

  NewWidget( widget );
}


FramePage::~FramePage()
{
}


void FramePage::Apply( DlgWidget*	widget )
{
    QFrame* frame = (QFrame*)widget->GetWidget();
    
    frame->setFrameStyle( itsShapes->currentItem() | 
			  (itsShadows->currentItem() << 4) );
    
    QString str = itsLineWidth->text();
    frame->setLineWidth( str.toInt() );
    str = itsMidLineWidth->text();
    frame->setMidLineWidth( str.toInt() );
    str = itsMargin->text();
    frame->setMargin( str.toInt() );
}


void FramePage::ShapeChanged( int index )
{
    if( index == 0 )
    {
	itsShadows->setCurrentItem( 0 );
    }
    // only NoFrame can have None for shadow
    else if( itsShadows->currentItem() == 0 )
    {
	itsShadows->setCurrentItem( 1 );
    }
}


void FramePage::ShadowChanged( int index )
{
    if( (itsShapes->currentItem() == 0) && (index != 0) )
    {
	itsShadows->setCurrentItem( 0 );
    }
    else if( (itsShapes->currentItem() != 0) && (index == 0) )
    {
	itsShadows->setCurrentItem( 1 );
    }
}
void FramePage::NewWidget( DlgWidget *widget )
{
   QFrame* frame = (QFrame*)widget->GetWidget();

    itsShapes->setCurrentItem( frame->frameShape() );
    itsShadows->setCurrentItem( frame->frameShadow() >> 4 );
    QString str;
    
    str.setNum( frame->lineWidth() );
    itsLineWidth->setText( str );
    str.setNum( frame->midLineWidth() );
    itsMidLineWidth->setText( str );
    str.setNum( frame->margin() );
    itsMargin->setText( str );
}
