/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: ListBoxPage.cpp,v 2.7 2000/07/23 20:37:45 jeffharris Exp $

*******************************************************************************/


#include <qlistbox.h>

#include "ListBoxPage.h"

#include "widget/DlgWidget.h"

#include "GuiTools.h"

#define Inherited ListBoxPageData

ListBoxPage::ListBoxPage
(
    DlgWidget* widget,
    QWidget* parent,
    const char* name
)
    :
    Inherited( parent, name )
{
    connect( itsAddButton, SIGNAL(clicked()), SLOT(AddClicked()) );
    connect( itsChangeButton, SIGNAL(clicked()), SLOT(ChangeClicked()) );
    connect( itsDeleteButton, SIGNAL(clicked()), SLOT(DeleteClicked()) );
    connect( itsList, SIGNAL(highlighted(int)), 
	     SLOT(ListSelChanged(int)) );
    connect( itsNewItem, SIGNAL(textChanged(const QString&)),
	     SLOT(NewItemChanged(const QString&)) );

    NewWidget( widget );
}


ListBoxPage::~ListBoxPage()
{
}


void ListBoxPage::Apply( DlgWidget*	widget )
{
    QListBox* list = (QListBox*)widget->GetWidget();
    
    GuiTools::TransferListBox( itsList, list );

    QListBox::SelectionMode selMode = QListBox::Single;
    if( isSelectionModeSingle->isChecked() )
    {
        selMode = QListBox::Single;
    }
    else if( isSelectionModeMultiple->isChecked() )
    {
        selMode = QListBox::Multi;
    }
    else if( isSelectionModeExtended->isChecked() )
    {
        selMode = QListBox::Extended;
    }   
    else if( isSelectionModeNoSelection->isChecked() )
    {
        selMode = QListBox::NoSelection;
    }

    list->setSelectionMode( selMode );

    
    if( isLayoutModeVariable->isChecked() )
    {
        list->setColumnMode( QListBox::Variable );
        list->setRowMode( QListBox::Variable );
    }
    else if( isLayoutModeFitToWidth->isChecked() )
    {
        list->setColumnMode( QListBox::FitToWidth );
    }
    else if( isLayoutModeFitToHeight->isChecked() )
    {
        list->setRowMode( QListBox::FitToHeight );
    }
    else if( isLayoutModeFixedColumns->isChecked() )
    {
        list->setColumnMode( itsFixedColumns->text().toInt() );
    }
    else if( isLayoutModeFixedRows->isChecked() )
    {
        list->setRowMode( itsFixedRows->text().toInt() );
    }
}


void ListBoxPage::LayoutModeClicked( int )
{
    itsFixedRows->setEnabled( isLayoutModeFixedRows->isChecked() );
    itsFixedColumns->setEnabled( isLayoutModeFixedColumns->isChecked() );
}


void ListBoxPage::AddClicked()
{
    const char* text = itsNewItem->text();
    if( text && text[0] )
    {
	itsList->insertItem( text );
    }
}


void ListBoxPage::ChangeClicked()
{
    const char* text = itsNewItem->text();
    if( text && text[0] )
    {
	itsList->changeItem( text, itsList->currentItem() );
    }
}


void ListBoxPage::DeleteClicked()
{
    int item = itsList->currentItem();
    if( item >= 0 )
    {
	itsList->removeItem( item );
	Update();
    }
}


void ListBoxPage::ListSelChanged( int item )
{
   itsNewItem->setText( itsList->text( item ) );
    Update();
}


void ListBoxPage::NewItemChanged( const QString& )
{
    Update();
}


void ListBoxPage::Update()
{
    const char* text = itsNewItem->text();
    bool newText = ( text && text[0] );
    bool itemSelected = (itsList->currentItem() != -1);

    itsAddButton->setEnabled( newText );
    itsChangeButton->setEnabled( newText & itemSelected );
    itsDeleteButton->setEnabled( itemSelected );
}


void ListBoxPage::NewWidget( DlgWidget *widget )
{
    const QListBox* list = (QListBox*)widget->GetWidget();
    
    GuiTools::TransferListBox( list, itsList );

    switch( list->selectionMode() )
    {
        case QListBox::Single:
        {
            isSelectionModeSingle->setChecked( TRUE );
            break;
        }
        case QListBox::Multi:
        {
            isSelectionModeMultiple->setChecked( TRUE );
            break;
        }
        case QListBox::Extended:
        {
            isSelectionModeExtended->setChecked( TRUE );
            break;
        }
        case QListBox::NoSelection:
        {
            isSelectionModeNoSelection->setChecked( TRUE );
            break;
        }
    }

    if( (list->columnMode() == QListBox::Variable) &&
        (list->rowMode() == QListBox::Variable) )
    {
        isLayoutModeVariable->setChecked( TRUE );
    }
    else if( list->columnMode() == QListBox::FitToWidth )
    {
        isLayoutModeFitToWidth->setChecked( TRUE );
    }
    else if( list->rowMode() == QListBox::FitToHeight )
    {
        isLayoutModeFitToHeight->setChecked( TRUE );
    }
    else if( list->columnMode() == QListBox::FixedNumber )
    {
        isLayoutModeFixedColumns->setChecked( TRUE );
        itsFixedColumns->setText( QString::number( list->numColumns() ) );
    }
    else if( list->rowMode() == QListBox::FixedNumber )
    {
        isLayoutModeFixedRows->setChecked( TRUE );
        itsFixedRows->setText( QString::number( list->numRows() ) );
    }
    
    Update();
    LayoutModeClicked( 0 );
}
