/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: MultiLineEditPage.cpp,v 2.4 2000/07/23 20:37:45 jeffharris Exp $

*******************************************************************************/


#include <qmlined.h>

#include "MultiLineEditPage.h"

#include "widget/DlgWidget.h"

#define Inherited MultiLineEditPageData

MultiLineEditPage::MultiLineEditPage
(
    DlgWidget* widget,
    QWidget* parent,
    const char* name
)
    :
    Inherited( parent, name )
{
  NewWidget( widget );
}


MultiLineEditPage::~MultiLineEditPage()
{
}


void MultiLineEditPage::Apply( DlgWidget*	widget )
{
    QMultiLineEdit* edit = (QMultiLineEdit*)widget->GetWidget();
    
    edit->setText( itsText->text() );

    int alignment = AlignLeft;
    if( isAlignmentLeft->isChecked() )
    {
        alignment = AlignLeft;
    }
    else if( isAlignmentCenter->isChecked() )
    {
        alignment = AlignHCenter;
    }
    else if( isAlignmentRight->isChecked() )
    {
        alignment = AlignRight;
    }
    edit->setAlignment( alignment );
    
    QMultiLineEdit::EchoMode echoMode = QMultiLineEdit::Normal;
    if( isEchoModeNormal->isChecked() )
    {
        echoMode = QMultiLineEdit::Normal;
    }
    else if( isEchoModeNoEcho->isChecked() )
    {
        echoMode = QMultiLineEdit::NoEcho;
    }
    else if( isEchoModePassword->isChecked() )
    {
        echoMode = QMultiLineEdit::Password;
    }
    edit->setEchoMode( echoMode );
    
    QMultiLineEdit::WordWrap wordWrap = QMultiLineEdit::NoWrap;
    if( isWordWrapNoWrap->isChecked() )
    {
        wordWrap = QMultiLineEdit::NoWrap;
    }
    else if( isWordWrapWidgetWidth->isChecked() )
    {
        wordWrap = QMultiLineEdit::WidgetWidth;
    }
    else if( isWordWrapFixedPixelWidth->isChecked() )
    {
        wordWrap = QMultiLineEdit::FixedPixelWidth;
    }
    else if( isWordWrapFixedColumnWidth->isChecked() )
    {
        wordWrap = QMultiLineEdit::FixedColumnWidth;
    }
    edit->setWordWrap( wordWrap );
    
    QMultiLineEdit::WrapPolicy wrapPolicy = QMultiLineEdit::AtWhiteSpace;
    if( isWrapPolicyAtWhiteSpace->isChecked() )
    {
        wrapPolicy = QMultiLineEdit::AtWhiteSpace;
    }
    else if( isWrapPolicyAnywhere->isChecked() )
    {
        wrapPolicy = QMultiLineEdit::Anywhere;
    }
    edit->setWrapPolicy( wrapPolicy );
    
    edit->setWrapColumnOrWidth( itsWrapColumnOrWidth->text().toInt() );
    
    edit->setUndoEnabled( isUndoEnabled->isChecked() );
    edit->setUndoDepth( itsUndoDepth->text().toInt() );
    
    edit->setMaxLength( itsMaxLength->text().toInt() );
    edit->setMaxLineLength( itsMaxLineLength->text().toInt() );
    edit->setMaxLines( itsMaxLines->text().toInt() );
    edit->setHMargin( itsHorizontalMargin->text().toInt() );

    edit->setReadOnly( isReadOnly->isChecked() );
    edit->setOverwriteMode( isOverwrite->isChecked() );
    edit->setAutoUpdate( isAutoUpdate->isChecked() );
}


void MultiLineEditPage::UndoToggled( bool )
{
    UpdateScreen();
}


void MultiLineEditPage::WordWrapClicked( int )
{
    UpdateScreen();
}


void MultiLineEditPage::UpdateScreen()
{
    bool undoEnabled = isUndoEnabled->isChecked();
    itsUndoDepthLabel->setEnabled( undoEnabled );
    itsUndoDepth->setEnabled( undoEnabled );
    
    bool wrapColumnOrWidthEnabled = 
        isWordWrapFixedPixelWidth->isChecked() || 
        isWordWrapFixedColumnWidth->isChecked();
    
    itsWrapColumnOrWidthLabel->setEnabled( wrapColumnOrWidthEnabled );
    itsWrapColumnOrWidth->setEnabled( wrapColumnOrWidthEnabled );
}

void MultiLineEditPage::NewWidget( DlgWidget *widget )
{
    QMultiLineEdit* edit = (QMultiLineEdit*)widget->GetWidget();
    
    itsText->setText( edit->text() );

    switch( edit->alignment() & (AlignLeft | AlignHCenter | AlignRight) )
    {
	case AlignLeft:
	default:
	{
	    isAlignmentLeft->setChecked( TRUE );
	    break;
	}
	case AlignHCenter:
	{
	    isAlignmentCenter->setChecked( TRUE );
	    break;
	}
	case AlignRight:
	{
	    isAlignmentRight->setChecked( TRUE );
	    break;
	}
    }

    switch( edit->echoMode() )
    {
	case QMultiLineEdit::Normal:
	{
	    isEchoModeNormal->setChecked( TRUE );
	    break;
	}
	case QLineEdit::NoEcho:
	{
	    isEchoModeNoEcho->setChecked( TRUE );
	    break;
	}
	case QLineEdit::Password:
	{
	    isEchoModePassword->setChecked( TRUE );
	    break;
	}
    }

    switch( edit->wordWrap() )
    {
        case QMultiLineEdit::NoWrap:
        {
            isWordWrapNoWrap->setChecked( TRUE );
            break;
        }
        case QMultiLineEdit::WidgetWidth:
        {
            isWordWrapWidgetWidth->setChecked( TRUE );
            break;
        }
        case QMultiLineEdit::FixedPixelWidth:
        {
            isWordWrapFixedPixelWidth->setChecked( TRUE );
            break;
        }
        case QMultiLineEdit::FixedColumnWidth:
        {
            isWordWrapFixedColumnWidth->setChecked( TRUE );
            break;
        }
    }

    switch( edit->wrapPolicy() )
    {
        case QMultiLineEdit::AtWhiteSpace:
        {
            isWrapPolicyAtWhiteSpace->setChecked( TRUE );
            break;
        }
        case QMultiLineEdit::Anywhere:
        {
            isWrapPolicyAnywhere->setChecked( TRUE );
            break;
        }
    }

    itsWrapColumnOrWidth->setText( 
        QString::number( edit->wrapColumnOrWidth() ) );
    
    isUndoEnabled->setChecked( edit->isUndoEnabled() );
    itsUndoDepth->setText( QString::number( edit->undoDepth() ) );
    
    itsMaxLength->setText( QString::number( edit->maxLength() ) );
    itsMaxLineLength->setText( QString::number( edit->maxLineLength() ) );
    itsMaxLines->setText( QString::number( edit->maxLines() ) );
    itsHorizontalMargin->setText( QString::number( edit->hMargin() ) );

    isReadOnly->setChecked( edit->isReadOnly() );
    isOverwrite->setChecked( edit->isOverwriteMode() );
    isAutoUpdate->setChecked( edit->autoUpdate() );

    UpdateScreen();
}
