/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: ScrollBarPage.cpp,v 2.3 2000/07/23 20:37:45 jeffharris Exp $

*******************************************************************************/


#include <qscrbar.h>
#include <qstring.h>

#include "ScrollBarPage.h"

#include "widget/DlgWidget.h"

#define Inherited ScrollBarPageData

ScrollBarPage::ScrollBarPage
(
    DlgWidget* widget,
    QWidget* parent,
    const char* name
)
    :
    Inherited( parent, name )
{
  NewWidget( widget );
}


ScrollBarPage::~ScrollBarPage()
{
}


void ScrollBarPage::Apply( DlgWidget*	widget )
{
    QScrollBar* scroll = (QScrollBar*)widget->GetWidget();
    
    QScrollBar::Orientation orient = QScrollBar::Vertical;
    
    if( isHoriz->isChecked() )
    {
	orient = QScrollBar::Horizontal;
    }
    
    scroll->setOrientation( orient );
    
    QString str = itsMinValue->text();
    int min = str.toInt();
    str = itsMaxValue->text();
    int max = str.toInt();
    
    scroll->setRange( min, max );
    
    str = itsLineStep->text();
    int line = str.toInt();
    str = itsPageStep->text();
    int page = str.toInt();
    
    scroll->setSteps( line, page );
    
    str = itsInitialValue->text();
    scroll->setValue( str.toInt() );

    scroll->setTracking( isTracking->isChecked() );
}

void ScrollBarPage::NewWidget( DlgWidget *widget )
{
    QScrollBar* scroll = (QScrollBar*)widget->GetWidget();
    
    switch( scroll->orientation() )
    {
	case QScrollBar::Horizontal:
	{
	    isHoriz->setChecked( TRUE );
	    break;
	}
	case QScrollBar::Vertical:
	{
	    isVert->setChecked( TRUE );
	    break;
	}
    }
    
    QString str;
    itsMinValue->setText( str.setNum( scroll->minValue() ) );
    itsMaxValue->setText( str.setNum( scroll->maxValue() ) );
    itsLineStep->setText( str.setNum( scroll->lineStep() ) );
    itsPageStep->setText( str.setNum( scroll->pageStep() ) );
    itsInitialValue->setText( str.setNum( scroll->value() ) );
    isTracking->setChecked( scroll->tracking() );
}
