/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: SignalSelectDialog.cpp,v 2.5 2000/07/23 20:37:45 jeffharris Exp $

*******************************************************************************/


#include <qmsgbox.h>

#include "widget/SignalConnection.h"

#include "prop/SignalSelectDialog.h"

#define Inherited SignalSelectDialogData

SignalSelectDialog::SignalSelectDialog
(
    const QStrList& signalList,
    QWidget* parent,
    const char* name
)
    :
    Inherited( parent, name )
{
    setCaption( tr( "Signal / Slot Connection" ) );

    itsSignals->insertStrList( &signalList );

    connect( itsSignals, SIGNAL(highlighted(int)), SLOT(SignalChanged(int)) );
    connect( itsSlot, SIGNAL(textChanged(const QString&)), 
	     SLOT(SlotChanged(const QString&)) );
    connect( itsOkButton, SIGNAL(clicked()), SLOT(accept()) );
    connect( itsCancelButton, SIGNAL(clicked()), SLOT(reject()) );
    
    Update();
}


SignalSelectDialog::~SignalSelectDialog()
{
}


void SignalSelectDialog::SignalChanged( int )
{
    Update();
}


void SignalSelectDialog::SlotChanged( const QString& )
{
    Update();
}


void SignalSelectDialog::Update()
{
    bool enable = (itsSignals->currentItem() != -1);
    
    itsAccessBox->setEnabled( enable );
    isBaseClass->setEnabled( enable );
    isProtected->setEnabled( enable );
    isPublic->setEnabled( enable );
    itsSlotLabel->setEnabled( enable );
    itsSlot->setEnabled( enable );

    QString str = itsSlot->text();
    itsOkButton->setEnabled( enable && !str.isEmpty() );
}


void SignalSelectDialog::SetData( const SignalConnection&	sig )
{
    unsigned int i;
    bool found = FALSE;
    QString sigStr = sig.GetSignal();
    
    for( i = 0; (i < itsSignals->count()) && !found; i++ )
    {
	if( sigStr == itsSignals->text( i ) )
	{
	    found = TRUE;
	}
    }
    
    if( found )
    {
	itsSignals->setCurrentItem( i-1 );
    }
    else
    {
	QString str = tr( "Signal %1 not found for this widget" ).arg( sigStr );
	QMessageBox::message( tr( "Warning" ), str );
    }

    itsSlot->setText( sig.GetSlotName() );
    
    isProtected->setChecked( FALSE );
    isPublic->setChecked( FALSE );

    switch( sig.GetScope() )
    {
	case SignalConnection::BaseClass:
	{
	    isBaseClass->setChecked( TRUE );
	    break;
	}
	case SignalConnection::Protected:
	{
	    isProtected->setChecked( TRUE );
	    break;
	}
	case SignalConnection::Public:
	{
	    isPublic->setChecked( TRUE );
	    break;
	}
    }

    Update();
}


void SignalSelectDialog::GetData( SignalConnection&	sig ) const
{
    QString sigstr = itsSignals->text( itsSignals->currentItem() );

    sig.SetSignal( sigstr );
    sig.SetSlotName( itsSlot->text() );
    
    SignalConnection::Scope scope = SignalConnection::Protected;
    if( isPublic->isChecked() )
    {
	scope = SignalConnection::Public;
    } else if( isBaseClass->isChecked() )
    {
	scope = SignalConnection::BaseClass;
    }
    
    sig.SetScope( scope );
}
