/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: TextViewPage.cpp,v 2.5 2000/07/23 20:37:45 jeffharris Exp $

*******************************************************************************/


#include <qtextview.h>
#include <qstring.h>

#include "TextViewPage.h"

#include "widget/DlgWidget.h"

#define Inherited TextViewPageData

TextViewPage::TextViewPage
(
    DlgWidget* widget,
    QWidget* parent,
    const char* name
)
    :
    Inherited( parent, name )
{
  NewWidget( widget );
}


TextViewPage::~TextViewPage()
{
}


void TextViewPage::Apply( DlgWidget*	widget )
{
    QTextView* tview = (QTextView*)widget->GetWidget();
    
    // Set the format before the text
    Qt::TextFormat format = Qt::PlainText;
    if( isTextFormatPlain->isChecked() )
    {
        format = Qt::PlainText;
    }
    else if( isTextFormatRich->isChecked() )
    {
        format = Qt::RichText;
    }
    else if( isTextFormatAuto->isChecked() )
    {
        format = Qt::AutoText;
    }
    tview->setTextFormat( format );

    // Set underlining of links before the text
    tview->setLinkUnderline( isLinkUnderline->isChecked() );

    tview->setText( itsText->text(), itsContext->text() );
}

void TextViewPage::NewWidget( DlgWidget *widget )
{    QTextView* tview = (QTextView*)widget->GetWidget();
    
    itsText->setText( tview->text() );
    itsContext->setText( tview->context() );
    
    switch( tview->textFormat() )
    {
        case Qt::PlainText:
        {
            isTextFormatPlain->setChecked( TRUE );
            break;
        }
        case Qt::RichText:
        {
            isTextFormatRich->setChecked( TRUE );
            break;
        }
        case Qt::AutoText:
        {
            isTextFormatAuto->setChecked( TRUE );
            break;
        }
    }
 
    isLinkUnderline->setChecked( tview->linkUnderline() );

}
