/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: WidgetVarMapPage.cpp,v 2.5 2001/03/13 20:34:15 kebner Exp $

*******************************************************************************/


#include "WidgetVarMapPage.h"

#include "prop/SignalSelectDialog.h"

#define Inherited WidgetVarMapPageData

WidgetVarMapPage::WidgetVarMapPage
(
    DlgWidget* widget,
    QWidget* parent,
    const char* name
)
    :
    Inherited( parent, name )
{
    connect( itsMapCB, SIGNAL(clicked()), SLOT(Update()) );

    NewWidget( widget );

    connect( itsSignalList, SIGNAL(highlighted(int)), 
	     SLOT(SignalChanged(int)) );
    connect( itsSignalList, SIGNAL(selected(int)), SLOT(SignalSelected(int)) );

    connect( itsAddSignal, SIGNAL(clicked()), SLOT(AddSignal()) );
    connect( itsModifySignal, SIGNAL(clicked()), SLOT(ModifySignal()) );
    connect( itsDeleteSignal, SIGNAL(clicked()), SLOT(DeleteSignal()) );

    Update();
}


WidgetVarMapPage::~WidgetVarMapPage()
{
}


void WidgetVarMapPage::AddSignal()
{
    SignalSelectDialog dlg( itsSignals );
    if( dlg.exec() == QDialog::Accepted )
    {
	SignalConnection sig;
	dlg.GetData( sig );
    
	itsSignalList->inSort( sig.ToString() );
    }
}


void WidgetVarMapPage::ModifySignal()
{
    SignalSelectDialog dlg( itsSignals );
    SignalConnection sig;
    sig.FromString( itsSignalList->text( itsSignalList->currentItem() ) );
    dlg.SetData( sig );
    
    if( dlg.exec() == QDialog::Accepted )
    {
	dlg.GetData( sig );
    
	itsSignalList->removeItem( itsSignalList->currentItem() );
	itsSignalList->inSort( sig.ToString() );
    }
}


void WidgetVarMapPage::DeleteSignal()
{
    itsSignalList->removeItem( itsSignalList->currentItem() );

    Update();
}


void WidgetVarMapPage::Update()
{
    bool enable = itsMapCB->isChecked();

    itsVarName->setEnabled( enable );
    itsVarLabel->setEnabled( enable );

    enable = (itsSignalList->currentItem() != -1);
    
    itsModifySignal->setEnabled( enable );
    itsDeleteSignal->setEnabled( enable );
}


void WidgetVarMapPage::SignalChanged( int )
{
    Update();
}


void WidgetVarMapPage::SignalSelected( int item )
{
    if( item >= 0 )
    {
	ModifySignal();
    }
}

void WidgetVarMapPage::NewWidget( DlgWidget *widget )
{
  widget->GetSignalList( itsSignals );

  itsVarName->setText( widget->GetVariableName() );
  itsMapCB->setChecked( widget->IsVariableMapped() );

  QListIterator<SignalConnection> iter( widget->GetConnectedSignalList() );
  SignalConnection* conn;

  itsSignalList->clear();
  while( (conn = iter()) )
  {
    itsSignalList->inSort( conn->ToString() );
  }

  Update();
}

void WidgetVarMapPage::Apply( DlgWidget*	widget )
{
    widget->SetVariableMapping( itsMapCB->isChecked() ?
				itsVarName->text() : QString() );

    
    QList<SignalConnection>& list = widget->GetConnectedSignalList();
    SignalConnection* sig;
    unsigned int i;
    
    list.clear();
    
    for( i = 0; i < itsSignalList->count(); i++ )
    {
	sig = new SignalConnection;
	sig->FromString( itsSignalList->text( i ) );
	list.append( sig );
    }
}

    










