/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgCheckBox.cpp,v 2.9 2000/07/23 20:38:01 jeffharris Exp $

*******************************************************************************/


#include "widget/DlgCheckBox.h"

#include "prop/CheckBoxPage.h"

#include "GuiTools.h"

#include "xml/QtArchExpat.h"
#include "xml/QtArchGenerator.h"

#define Inherited DlgButton

QStrList DlgCheckBox::itsSignals;
int DlgCheckBox::itsNextWidgetNum = 1;

DlgCheckBox::DlgCheckBox() :
    Inherited( DlgWidget::DW_CheckBox )
{
}


DlgCheckBox::DlgCheckBox(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgCheckBox::~DlgCheckBox()
{
}


void DlgCheckBox::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    DlgWidget::GetPropertyPages( parent, pages, labels );

    pages.append( new CheckBoxPage( this, parent, "CheckBox" ) );

    labels.append( tr( "Check Box" ) );
}

void DlgCheckBox::Save( QtArchGenerator &generator )
{
  generator.WriteStartBlock( "CheckBox" );
  QCheckBox* widget = GetCheckBox();

  generator.WriteXMLValue( "Checked", XMLValue( widget->isChecked() ));
  generator.WriteXMLValue( 
      "Tristate", 
      XMLValue( widget->toggleType() == QButton::Tristate ) );

  Inherited::Save( generator );

  generator.WriteEndBlock();
}


bool DlgCheckBox::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    QCheckBox* widget = GetCheckBox();

    // Compare to known keys
    if( key == "Checked" )
    {
	widget->setChecked( (value == "TRUE") );
    } 
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}

void DlgCheckBox::RestoreXML( const XMLValue &value )
{
  // Sanity check
  if( value.GetTypeString() != "CheckBox" )
    return;

  const XMLValueMap map = value.AsNested();

  // Create if necessary
  if( ! GetCheckBox() )
  {
    CreateQtWidget( 0 );
  }
  
  QCheckBox* widget = GetCheckBox();

  widget->setChecked( map["Checked"].AsBool() );

  widget->setTristate( map["Tristate"].AsBool() );
  
  // We inherit from 
	Inherited::RestoreXML( map["Button"] );

}


void DlgCheckBox::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    const QCheckBox* widget = GetCheckBox();

    stream << varName << "setChecked( " <<
	(widget->isChecked() ? "TRUE" : "FALSE") << " );\n";

    stream << varName << "setTristate( " 
           << ((widget->toggleType() == QButton::Tristate) ? "TRUE" : "FALSE")
           << " );" << endl;
}


void DlgCheckBox::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgCheckBox::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
    }

    return itsSignals;
}


void DlgCheckBox::CreateQtWidget(
    const char*	theName
    )
{
    QCheckBox* widget = 
	new QCheckBox( GetParent(),
		       CreateName( "CheckBox", 
				   theName, 
				   itsNextWidgetNum ) );

    SetQtWidget( widget );
}
