/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgDial.cpp,v 2.3 2000/07/23 20:38:01 jeffharris Exp $

*******************************************************************************/


#include "widget/DlgDial.h"

#include "prop/DialPage.h"

#include "GuiTools.h"

#include "xml/QtArchExpat.h"
#include "xml/QtArchGenerator.h"

#define Inherited DlgWidget

QStrList DlgDial::itsSignals;
int DlgDial::itsNextWidgetNum = 1;

DlgDial::DlgDial() :
    Inherited( DlgWidget::DW_Dial )
{
}


DlgDial::DlgDial(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgDial::~DlgDial()
{
}


void DlgDial::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new DialPage( this, parent, "Dial" ) );

    labels.append( tr( "Dial" ) );
}

void DlgDial::Save( QtArchGenerator &generator )
{
    generator.WriteStartBlock( "Dial" );
  
    QDial* widget = GetDial();

    generator.WriteXMLValue( "MinValue", widget->minValue() );
    generator.WriteXMLValue( "MaxValue", widget->maxValue() );
    generator.WriteXMLValue( "Initial", widget->value() );
    generator.WriteXMLValue( "LineStep", widget->lineStep() );
    generator.WriteXMLValue( "PageStep", widget->pageStep() );

    generator.WriteXMLValue( "Tracking", widget->tracking() );
    generator.WriteXMLValue( "Wrapping", widget->wrapping() );
    generator.WriteXMLValue( "NotchTarget", widget->notchTarget() );
    generator.WriteXMLValue( "ShowNotches", widget->showNotches() );

    Inherited::Save( generator );

    generator.WriteEndBlock();
}


void DlgDial::RestoreXML( const XMLValue &value )
{
    // Sanity check
    if( value.GetTypeString() != "Dial" )
        return;

    const XMLValueMap map = value.AsNested();

    // Create if necessary
    if( ! GetDial() )
    {
        CreateQtWidget( 0 );
    }
  
    QDial* widget = GetDial();

    widget->setRange( map["MinValue"].AsInt(), map["MaxValue"].AsInt() );
    widget->setValue( map["Initial"].AsInt() );
    widget->setSteps( map["LineStep"].AsInt(), map["PageStep"].AsInt() );

    widget->setTracking( map["Tracking"].AsBool() );
    widget->setWrapping( map["Wrapping"].AsBool() );
    widget->setNotchTarget( map["NotchTarget"].AsDouble() );
    widget->setShowNotches( map["ShowNotches"].AsBool() );

    // We inherit from 
    Inherited::RestoreXML( map["DlgWidget"] );

}


void DlgDial::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    const QDial* widget = GetDial();

    stream << varName << "setRange( " <<
	widget->minValue() << ", " << widget->maxValue() << " );" << endl;
    
    stream << varName << "setSteps( " <<
	widget->lineStep() << ", " << widget->pageStep() << " );" << endl;
    
    stream << varName << 
	"setValue( " << widget->value() << " );" << endl;
    
    stream << varName << "setTracking( ";
    SaveBool( stream, widget->tracking() );
    stream << " );" << endl;
    
    stream << varName << "setWrapping( ";
    SaveBool( stream, widget->wrapping() );
    stream << " );" << endl;
    
    stream << varName << "setNotchTarget( " 
           << widget->notchTarget() << " );" << endl;

    stream << varName << "setShowNotches( ";
    SaveBool( stream, ((QDial*)widget)->showNotches() );
    stream << " );" << endl;
}


void DlgDial::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgDial::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
	itsSignals.append( "valueChanged(int)" );
	itsSignals.append( "dialPressed()" );
	itsSignals.append( "dialMoved(int)" );
	itsSignals.append( "dialReleased()" );
    }

    return itsSignals;
}


void DlgDial::CreateQtWidget(
    const char*	theName
    )
{
    QDial* widget = 
	new QDial( GetParent(),
                   CreateName( "Dial", 
                               theName, 
                               itsNextWidgetNum ) );

    SetQtWidget( widget );
}
