/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgHBox.cpp,v 2.8 2000/07/23 20:38:01 jeffharris Exp $

*******************************************************************************/


#include "widget/DlgHBox.h"

#include "prop/HBoxPage.h"

#include "GuiTools.h"

#include "xml/QtArchExpat.h"
#include "xml/QtArchGenerator.h"

#define Inherited DlgFrame

QStrList DlgHBox::itsSignals;
int DlgHBox::itsNextWidgetNum = 1;

DlgHBox::DlgHBox() :
    Inherited( DlgWidget::DW_HBox ),
    mySpacing( 5 )
{
}


DlgHBox::DlgHBox(
    DlgWidget::Type	theType
    ) :
    Inherited( theType ),
    mySpacing( 5 )
{
}


DlgHBox::~DlgHBox()
{
}


void DlgHBox::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new HBoxPage( this, parent, "HBox" ) );

    labels.append( tr( "Box" ) );
}

void DlgHBox::Save( QtArchGenerator &generator )
{
  generator.WriteStartBlock( "HBox" );

  generator.WriteXMLValue( "Spacing", mySpacing );

  Inherited::Save( generator );

  generator.WriteEndBlock();
}


bool DlgHBox::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    // Compare to known keys
    if( key == "Spacing" )
    {
	SetSpacing( value.toInt() );
    } 
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}

void DlgHBox::RestoreXML( const XMLValue &value )
{
  // Sanity check
  if( value.GetTypeString() != "HBox" )
    return;

  const XMLValueMap map = value.AsNested();

  // Create if necessary
  if( ! GetHBox() )
  {
    CreateQtWidget( 0 );
  }
  
  SetSpacing( map["Spacing"].AsInt() );

  // We inherit from 
	Inherited::RestoreXML( map["Frame"] );

}


void DlgHBox::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    stream << varName << "setSpacing( " << mySpacing << " );" << endl;
}


void DlgHBox::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgHBox::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
    }

    return itsSignals;
}


void DlgHBox::CreateQtWidget(
    const char*	theName
    )
{
    QHBox* widget = 
	new QHBox( GetParent(),
                   CreateName( "HBox", 
                               theName, 
                               itsNextWidgetNum ) );
    
    SetQtWidget( widget );

    SetSpacing( 5 );
}
