/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgIconView.h,v 2.4 2000/07/23 20:38:01 jeffharris Exp $

*******************************************************************************/


#ifndef DlgIconView_included
#define DlgIconView_included

#include <qiconview.h>
#include <qstring.h>
#include <qstrlist.h>

#include "widget/DlgScrollView.h"

class QtArchGenerator;
class XMLValue;

class DlgIconView : public DlgScrollView
{

public:

    DlgIconView();
    // Constructor

    virtual ~DlgIconView();
    // Destructor
    

    void AddDemoItems();
    // Add demonstration items


    //
    // Saving and Loading Methods
    //

    virtual void Save( QtArchGenerator& generator );
    // save the contents of the widget

    virtual void RestoreXML( const XMLValue&  value );
    // Restore a XMLValue


    //
    // Code Generation Methods
    //

    virtual QString GetHeaderFilename() const;
    // return the header filename for the widget

    virtual QString GetWidgetClass() const;
    // return the class name for the widget

    virtual void GenerateSource(
	QTextStream&		stream,
	const QString&		varName,
        const WidgetLayout&     widgetLayout
	);
    // Generate the source code needed to initialize the widget.
    // varName already has a leading tab and the correct dereferencer.


    //
    // Property Editing Methods
    //

    virtual void GetPropertyPages(
	QWidget*		parent,
	QList<QWidget>&		pages,
	QStrList&		labels
	);
    // Get the property pages and their labels for the widget


    //
    // Connected Signals List
    //

    virtual void AddSignalsToList( 
	QStrList&	signalList
	) const;
    // add signals specific to a particular widget type to the list

    static const QStrList& GetSignalList();
    // method to return the list of signals


    //
    // Utility methods
    //

    static QString SelectionModeToStr( QIconView::SelectionMode mode );
    static QIconView::SelectionMode StrToSelectionMode( const QString& str );
    // SelectionMode enum

    static QString ArrangementToStr( QIconView::Arrangement arrangement );
    static QIconView::Arrangement StrToArrangement( const QString& str );
    // Arrangement enum

    static QString ResizeModeToStr( QIconView::ResizeMode  mode );
    static QIconView::ResizeMode StrToResizeMode( const QString&      str );
    // ResizeMode enum

    static QString ItemTextPosToStr( QIconView::ItemTextPos  pos );
    static QIconView::ItemTextPos StrToItemTextPos( const QString& str );
    // ItemTextPos enum
    

protected: // Methods

    DlgIconView(
	DlgWidget::Type	theType
	);
    // Constructor.  Used by a derived class' constructor.

    virtual void CreateQtWidget(
	const char*	theName
	);
    // Create and return the correct Qt widget for the specific type of
    // DlgWidget.


private: // Methods

    QIconView* GetIconView();
    // Get the Qt Widget as QIconView.

private: // static data

    static QStrList	itsSignals;
    // list of the signals that can be connected to this widget

    static int itsNextWidgetNum;
    // number of the next widget

};


inline QString DlgIconView::GetHeaderFilename() const
{
    return "<qiconview.h>";
}

inline QString DlgIconView::GetWidgetClass() const
{
    return "QIconView";
}


inline QIconView* DlgIconView::GetIconView()
{
    return static_cast<QIconView*> (GetWidget());
}


#endif // DlgIconView_included
