/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgLCDNumber.cpp,v 2.9 2000/07/23 20:38:01 jeffharris Exp $

*******************************************************************************/


#include "widget/DlgLCDNumber.h"

#include "prop/LCDNumberPage.h"

#include "GuiTools.h"

#include "xml/QtArchExpat.h"
#include "xml/QtArchGenerator.h"

#define Inherited DlgFrame

QStrList DlgLCDNumber::itsSignals;
int DlgLCDNumber::itsNextWidgetNum = 1;

DlgLCDNumber::DlgLCDNumber() :
    Inherited( DlgWidget::DW_LCDNumber ),
    itsValue( "0" )
{
}


DlgLCDNumber::DlgLCDNumber(
    DlgWidget::Type	theType
    ) :
    Inherited( theType ),
    itsValue( "0" )
{
}


DlgLCDNumber::~DlgLCDNumber()
{
}


void DlgLCDNumber::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new LCDNumberPage( this, parent, "LCDNumber" ) );

    labels.append( tr( "LCD Number" ) );
}

void DlgLCDNumber::Save( QtArchGenerator &generator )
{
  generator.WriteStartBlock( "LCDNumber" );
  
  QLCDNumber* widget = GetLCDNumber();

  generator.WriteXMLValue( "Value", GetValue() );
  generator.WriteXMLValue( "Digits", widget->numDigits() );
  generator.WriteXMLValue( "SmallDecimalPoint", widget->smallDecimalPoint() );
    
  QString mode;
  switch( widget->mode() )
  {
    case QLCDNumber::HEX:
    {
	    mode = "Hex";
	    break;
    }
    case QLCDNumber::DEC:
    {
	    mode = "Dec";
	    break;
    }
    case QLCDNumber::OCT:
    {
	    mode = "Oct";
	    break;
    }
    case QLCDNumber::BIN:
    {
	    mode = "Bin";
	    break;
    }
  }
  generator.WriteSingleLine( "Mode", mode );

  generator.WriteSingleLine( "SegmentStyle",
                             SegmentStyleToStr( widget->segmentStyle() ) );

  Inherited::Save( generator );

  generator.WriteEndBlock();

}


bool DlgLCDNumber::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    QLCDNumber* widget = GetLCDNumber();

    // Compare to known keys
    if( key == "Digits" )
    {
	widget->setNumDigits( value.toInt() );
    } 
    else if( key == "SmallDecimalPoint" )
    {
	widget->setSmallDecimalPoint( (value == "TRUE") );
    }
    else if( key == "Mode" )
    {
	if( value == "Hex" )
	{
	    widget->setHexMode();
	}
	else if( value == "Dec" )
	{
	    widget->setDecMode();
	}
	else if( value == "Oct" )
	{
	    widget->setOctMode();
	}
	else if( value == "Bin" )
	{
	    widget->setBinMode();
	}
    }
    else if( key == "SegmentStyle" )
    {
	widget->setSegmentStyle( StrToSegmentStyle( value ) );
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}

void DlgLCDNumber::RestoreXML( const XMLValue &value )
{
    // Sanity check
    if( value.GetTypeString() != "LCDNumber" )
        return;

    const XMLValueMap map = value.AsNested();

    // Create if necessary
    if( ! GetLCDNumber() )
    {
        CreateQtWidget( 0 );
    }
  
    QLCDNumber* widget = GetLCDNumber();

    SetValue( map["Value"].AsQString() );
    widget->setNumDigits( map["Digits"].AsInt() );
    widget->setSmallDecimalPoint( map["SmallDecimalPoint"].AsBool() );

    QString mode = map["Mode"].AsQString();
    if( mode == "Hex" )
        widget->setHexMode();
    else if( mode == "Dec" )
        widget->setDecMode();
    else if( mode == "Oct" )
        widget->setOctMode();
    else if( mode == "Bin" )
        widget->setBinMode();

    widget->setSegmentStyle( 
        StrToSegmentStyle( map["SegmentStyle"].AsQString() ) );

    // We inherit from 
    Inherited::RestoreXML( map["Frame"] );

}


void DlgLCDNumber::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    const QLCDNumber* widget = GetLCDNumber();

    stream << varName << "display( " << itsValue << " );" << endl;

    stream << varName << "setSmallDecimalPoint( " <<
	(widget->smallDecimalPoint() ? "TRUE" : "FALSE") << " );" << endl;
    
    stream << varName <<
	"setNumDigits( " << widget->numDigits() << " );" << endl;
    
    stream << varName << "setMode( QLCDNumber::";
    switch( widget->mode() )
    {
	case QLCDNumber::HEX:
	{
	    stream << "HEX";
	    break;
	}
	case QLCDNumber::OCT:
	{
	    stream << "OCT";
	    break;
	}
	case QLCDNumber::BIN:
	{
	    stream << "BIN";
	    break;
	}
	case QLCDNumber::DEC:
	{
	    stream << "DEC";
	    break;
	}
    }
    stream << " );" << endl;
    
    stream << varName << "setSegmentStyle( QLCDNumber::" <<
	SegmentStyleToStr( widget->segmentStyle() ) << 
	" );" << endl;
}


void DlgLCDNumber::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgLCDNumber::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
	itsSignals.append( "overflow()" );
    }

    return itsSignals;
}


QString DlgLCDNumber::SegmentStyleToStr( QLCDNumber::SegmentStyle style )
{
    QString str = "Outline";
    
    switch( style )
    {
	case QLCDNumber::Outline:
	default:
	{
	    // already set to Outline
	    break;
	}
	case QLCDNumber::Filled:
	{
	    str = "Filled";
	    break;
	}
	case QLCDNumber::Flat:
	{
	    str = "Flat";
	    break;
	}
    }
    
    return str;
}


QLCDNumber::SegmentStyle DlgLCDNumber::StrToSegmentStyle( const QString& str )
{
    QLCDNumber::SegmentStyle style = QLCDNumber::Outline;
    
    if( str == "Filled" )
    {
	style = QLCDNumber::Filled;
    }
    else if( str == "Flat" )
    {
	style = QLCDNumber::Flat;
    }
    
    return style;
}


void DlgLCDNumber::CreateQtWidget(
    const char*	theName
    )
{
    QLCDNumber* widget = 
	new QLCDNumber( GetParent(),
			CreateName( "LCDNumber", 
				    theName, 
				    itsNextWidgetNum ) );

    SetQtWidget( widget );
}
