/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgListView.h,v 2.7 2000/07/23 20:38:01 jeffharris Exp $

*******************************************************************************/


#ifndef DlgListView_included
#define DlgListView_included

#include <qlistview.h>
#include <qstring.h>
#include <qstrlist.h>

#include "widget/DlgScrollView.h"

class DlgListView : public DlgScrollView
{

public:

    DlgListView();
    // Constructor

    virtual ~DlgListView();
    // Destructor
    

    //
    // Saving and Loading Methods
    //

    virtual void Save( QtArchGenerator & );
    // save the contents of the widget

    virtual bool RestoreKeyValue( 
	const QString& 	key, 
	const QString& 	value,
	bool		topLevel = TRUE
	);
    // Restore a single key/value pair

    virtual void RestoreXML( const XMLValue & );
    // Restore a XMLValue


    //
    // Code Generation Methods
    //

    virtual QString GetHeaderFilename() const;
    // return the header filename for the widget

    virtual QString GetWidgetClass() const;
    // return the class name for the widget

    virtual void GenerateSource(
	QTextStream&		stream,
	const QString&		varName,
        const WidgetLayout&     widgetLayout
	);
    // Generate the source code needed to initialize the widget.
    // varName already has a leading tab and the correct dereferencer.


    //
    // Property Editing Methods
    //

    virtual void GetPropertyPages(
	QWidget*		parent,
	QList<QWidget>&		pages,
	QStrList&		labels
	);
    // Get the property pages and their labels for the widget


    //
    // Connected Signals List
    //

    virtual void AddSignalsToList( 
	QStrList&	signalList
	) const;
    // add signals specific to a particular widget type to the list

    static const QStrList& GetSignalList();
    // method to return the list of signals


    //
    // Properties
    //

    unsigned int GetNumHeaders() const;
    void SetNumHeaders(
	unsigned int	theNumHeaders
	);
    // Get/Set the number of headers for this list view

    //
    // Utility methods
    //

    static QString WidthModeToStr( QListView::WidthMode theMode );
    // Return a string for the width mode enum

    static QListView::WidthMode StrToWidthMode( const QString& theWidthMode );
    // Return an enum for the policy, defaulting to Maximum

protected: // Methods

    DlgListView(
	DlgWidget::Type	theType
	);
    // Constructor.  Used by a derived class' constructor.

    virtual void CreateQtWidget(
	const char*	theName
	);
    // Create and return the correct Qt widget for the specific type of
    // DlgWidget.


private: // Methods

    QListView* GetListView();
    // Get the Qt Widget as QListView.

private: // static data

    static QStrList	itsSignals;
    // list of the signals that can be connected to this widget

    static int itsNextWidgetNum;
    // number of the next widget

private:

    unsigned int itsNumHeaders;
    // The number of headers for this list view

    bool isFirstHeaderLoaded;
    // Has the first header been loaded from a file
};


inline QString DlgListView::GetHeaderFilename() const
{
    return "<qlistview.h>";
}

inline QString DlgListView::GetWidgetClass() const
{
    return "QListView";
}


inline unsigned int DlgListView::GetNumHeaders() const
{
    return itsNumHeaders;
}


inline void DlgListView::SetNumHeaders(
    unsigned int	theNumHeaders
    )
{
    itsNumHeaders = theNumHeaders;
}


inline QListView* DlgListView::GetListView()
{
    return static_cast<QListView*> (GetWidget());
}


#endif // DlgListView_included
