/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgSplitter.cpp,v 2.9 2000/07/23 20:38:01 jeffharris Exp $

*******************************************************************************/


#include "widget/DlgSplitter.h"

#include "prop/SplitterPage.h"

#include "GuiTools.h"

#include "xml/QtArchExpat.h"
#include "xml/QtArchGenerator.h"

#define Inherited DlgFrame

QStrList DlgSplitter::itsSignals;
int DlgSplitter::itsNextWidgetNum = 1;

DlgSplitter::DlgSplitter() :
    Inherited( DlgWidget::DW_Splitter ),
    mySpacing( 5 )
{
}


DlgSplitter::DlgSplitter(
    DlgWidget::Type	theType
    ) :
    Inherited( theType ),
    mySpacing( 5 )
{
}


DlgSplitter::~DlgSplitter()
{
}


void DlgSplitter::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new SplitterPage( this, parent, "Splitter" ) );

    labels.append( tr( "Splitter" ) );
}

void DlgSplitter::Save( QtArchGenerator &generator )
{
  generator.WriteStartBlock( "Splitter" );

  QSplitter *splitter = GetSplitter();
  Qt::Orientation o = splitter->orientation();
  QString orientation;

  if( o == Qt::Horizontal )
    orientation = "Horizontal";
  else
    orientation =  "Vertical";
  generator.WriteSingleLine( "Orientation", orientation );

  generator.WriteXMLValue( "OpaqueResize", splitter->opaqueResize() );

  Inherited::Save( generator );

  generator.WriteEndBlock();
}


bool DlgSplitter::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    QSplitter *splitter = GetSplitter();

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    // Compare to known keys
    if( key == "Orientation" )
    {
      Qt::Orientation o;
      if( value == "Horizontal" )
	o = Qt::Horizontal;
      else
	o = Qt::Vertical;
      splitter->setOrientation( o );
    } 
    else if( key == "OpaqueResize" )
    {
      splitter->setOpaqueResize( value == "TRUE" );
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}

void DlgSplitter::RestoreXML( const XMLValue &value )
{
  // Sanity check
  if( value.GetTypeString() != "Splitter" )
    return;

  const XMLValueMap map = value.AsNested();

  // Create if necessary
  if( ! GetSplitter() )
  {
    CreateQtWidget( 0 );
  }
  
  QSplitter* widget = GetSplitter();

  Qt::Orientation o;
  if( map["Orientation"].AsQString() == "Horizontal" )
    o = Qt::Horizontal;
  else
    o = Qt::Vertical;
  widget->setOrientation( o );

  widget->setOpaqueResize( map["OpaqueResize"].AsBool() );
  

  // We inherit from 
	Inherited::RestoreXML( map["Frame"] );

}


void DlgSplitter::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    QSplitter *splitter = GetSplitter();
    Qt::Orientation o = splitter->orientation();

    stream << varName << "setOrientation( ";
    if( o == Qt::Horizontal )
      stream << "Horizontal";
    else
      stream << "Vertical";
    stream << " );" << endl;

    stream << varName << "setOpaqueResize( " <<
	(splitter->opaqueResize() ? "TRUE" : "FALSE") << " );" << endl;

}


void DlgSplitter::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgSplitter::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
    }

    return itsSignals;
}


void DlgSplitter::CreateQtWidget(
    const char*	theName
    )
{
    QSplitter* widget = 
	new QSplitter( GetParent(),
                   CreateName( "Splitter", 
                               theName, 
                               itsNextWidgetNum ) );
    
    SetQtWidget( widget );
}
