/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgTextBrowser.cpp,v 2.10 2000/07/23 20:38:01 jeffharris Exp $

*******************************************************************************/


#include "widget/DlgTextBrowser.h"

#include "GuiTools.h"

#include "xml/QtArchExpat.h"
#include "xml/QtArchGenerator.h"

#define Inherited DlgTextView

QStrList DlgTextBrowser::itsSignals;
int DlgTextBrowser::itsNextWidgetNum = 1;

DlgTextBrowser::DlgTextBrowser() :
    Inherited( DlgWidget::DW_TextBrowser )
{
}


DlgTextBrowser::DlgTextBrowser(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgTextBrowser::~DlgTextBrowser()
{
}


void DlgTextBrowser::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );
}

void DlgTextBrowser::Save( QtArchGenerator &generator )
{
  generator.WriteStartBlock( "TextBrowser" );

  //QTextBrowser* widget = GetTextBrowser();

  Inherited::Save( generator );

  generator.WriteEndBlock();
}


bool DlgTextBrowser::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
  bool retVal = TRUE;

  if( topLevel )
  {
    if( key == "Name" )
    {
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
    }
  }

  //QTextBrowser* widget = GetTextBrowser();

  // Compare to known keys
  retVal = Inherited::RestoreKeyValue( key, value, FALSE );

  return retVal;
}

void DlgTextBrowser::RestoreXML( const XMLValue &value )
{
  // Sanity check
  if( value.GetTypeString() != "TextBrowser" )
    return;

  const XMLValueMap map = value.AsNested();

  // Create if necessary
  if( ! GetTextBrowser() )
  {
    CreateQtWidget( 0 );
  }
  
  //QTextBrowser* widget = GetTextBrowser();

  
  // We inherit from 
	Inherited::RestoreXML( map["TextView"] );

}


void DlgTextBrowser::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    //const QTextBrowser* widget = GetTextBrowser();
}


void DlgTextBrowser::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgTextBrowser::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
      itsSignals.append( "backwardAvailable(bool)" );
      itsSignals.append( "forwardAvailable(bool)" );
      itsSignals.append( "highlighted(const QString&)" );
      itsSignals.append( "textChanged()" );
    }

    return itsSignals;
}


void DlgTextBrowser::CreateQtWidget(
    const char*	theName
    )
{
    QTextBrowser* widget = 
	new QTextBrowser( GetParent(),
		       CreateName( "TextBrowser", 
				   theName, 
				   itsNextWidgetNum ) );

    SetQtWidget( widget );
}
