/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgTextView.cpp,v 2.10 2000/07/23 20:38:01 jeffharris Exp $

*******************************************************************************/


#include "widget/DlgTextView.h"

#include "prop/TextViewPage.h"

#include "GuiTools.h"

#include "xml/QtArchExpat.h"
#include "xml/QtArchGenerator.h"

#define Inherited DlgScrollView

QStrList DlgTextView::itsSignals;
int DlgTextView::itsNextWidgetNum = 1;

DlgTextView::DlgTextView() :
    Inherited( DlgWidget::DW_TextView )
{
}


DlgTextView::DlgTextView(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgTextView::~DlgTextView()
{
}


void DlgTextView::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new TextViewPage( this, parent, "TextView" ) );

    labels.append( tr( "Text View" ) );
}

void DlgTextView::Save( QtArchGenerator &generator )
{
  generator.WriteStartBlock( "TextView" );

  QTextView* widget = GetTextView();

  generator.WriteSingleLine( "Text", widget->text() );
  generator.WriteSingleLine( "Context", widget->context() );
  
  generator.WriteXMLValue( "TextFormat",
                           TextFormatToStr( widget->textFormat() ) );
  
  generator.WriteXMLValue( "LinkUnderline", widget->linkUnderline() );

  Inherited::Save( generator );

  generator.WriteEndBlock();
}


bool DlgTextView::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    QTextView* widget = GetTextView();

    // Compare to known keys
    if( key == "Text" )
    {
	widget->setText( value );
    } 
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}

void DlgTextView::RestoreXML( const XMLValue &value )
{
    // Sanity check
    if( value.GetTypeString() != "TextView" )
        return;

    const XMLValueMap map = value.AsNested();

    // Create if necessary
    if( ! GetTextView() )
    {
        CreateQtWidget( 0 );
    }
  
    QTextView* widget = GetTextView();

    // Must set the text format before setting the text
    widget->setTextFormat( StrToTextFormat( map["TextFormat"].AsQString() ) );
    
    // Must set whether links are underlined before the text
    widget->setLinkUnderline( map["LinkUnderline"].AsBool() );

    widget->setText( map["Text"].AsQString(), map["Context"].AsQString() );

    // We inherit from 
    Inherited::RestoreXML( map["ScrollView"] );

}


void DlgTextView::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    const QTextView* widget = GetTextView();

    // Must set the text format before setting the text
    stream << varName << "setTextFormat( Qt::"
           << TextFormatToStr( widget->textFormat() ) << " );" << endl;

    // Must set whether links are underlined before the text
    stream << varName << "setLinkUnderline( "
           << (widget->linkUnderline() ? "TRUE" : "FALSE") << " );" << endl;

    stream << varName << "setText( ";
    GenerateText( widget->text(), widgetLayout, stream );
    stream << ", ";
    GenerateText( widget->context(), widgetLayout, stream, TRUE );
    stream << " );" << endl;
}


void DlgTextView::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgTextView::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
    }

    return itsSignals;
}


void DlgTextView::CreateQtWidget(
    const char*	theName
    )
{
    QTextView* widget = 
	new QTextView( GetParent(),
		       CreateName( "TextView", 
				   theName, 
				   itsNextWidgetNum ) );

    SetQtWidget( widget );
}
