/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgVBox.cpp,v 2.8 2000/07/23 20:38:01 jeffharris Exp $

*******************************************************************************/


#include "widget/DlgVBox.h"

#include "GuiTools.h"

#include "xml/QtArchExpat.h"
#include "xml/QtArchGenerator.h"

#define Inherited DlgHBox

QStrList DlgVBox::itsSignals;
int DlgVBox::itsNextWidgetNum = 1;

DlgVBox::DlgVBox() :
    Inherited( DlgWidget::DW_VBox )
{
}


DlgVBox::DlgVBox(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgVBox::~DlgVBox()
{
}


void DlgVBox::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );
}

void DlgVBox::Save( QtArchGenerator &generator )
{
  generator.WriteStartBlock( "VBox" );

  Inherited::Save( generator );

  generator.WriteEndBlock();
}


bool DlgVBox::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    retVal = Inherited::RestoreKeyValue( key, value, FALSE );

    return retVal;
}

void DlgVBox::RestoreXML( const XMLValue &value )
{
  // Sanity check
  if( value.GetTypeString() != "VBox" )
    return;

  const XMLValueMap map = value.AsNested();

  // Create if necessary
  if( ! GetVBox() )
  {
    CreateQtWidget( 0 );
  }
  
  // We inherit from 
	Inherited::RestoreXML( map["HBox"] );

}


void DlgVBox::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );
}


void DlgVBox::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgVBox::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
    }

    return itsSignals;
}


void DlgVBox::CreateQtWidget(
    const char*	theName
    )
{
    QVBox* widget = 
	new QVBox( GetParent(),
                   CreateName( "VBox", 
                               theName, 
                               itsNextWidgetNum ) );

    SetQtWidget( widget );
}
