/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgWidget.cpp,v 2.24 2000/08/20 19:19:19 kebner Exp $

*******************************************************************************/


#include <iostream.h>

#include <qapp.h>
#include <qpalette.h>
#include <qregexp.h>
#include <qobjcoll.h>

#include "WidgetLayout.h"

#include "widget/DlgWidget.h"
#include "widget/DlgButton.h"
#include "widget/DlgButtonGroup.h"
#include "widget/DlgCheckBox.h"
#include "widget/DlgComboBox.h"
#include "widget/DlgFrame.h"
#include "widget/DlgGrid.h"
#include "widget/DlgGroupBox.h"
#include "widget/DlgHBox.h"
#include "widget/DlgHButtonGroup.h"
#include "widget/DlgHGroupBox.h"
#include "widget/DlgIconView.h"
#include "widget/DlgLCDNumber.h"
#include "widget/DlgLabel.h"
#include "widget/DlgLineEdit.h"
#include "widget/DlgListBox.h"
#include "widget/DlgListView.h"
#include "widget/DlgMenuBar.h"
#include "widget/DlgMultiLineEdit.h"
#include "widget/DlgProgressBar.h"
#include "widget/DlgPushButton.h"
#include "widget/DlgRadioButton.h"
#include "widget/DlgScrollBar.h"
#include "widget/DlgScrollView.h"
#include "widget/DlgSlider.h"
#include "widget/DlgSpinBox.h"
#include "widget/DlgSplitter.h"
#include "widget/DlgTabBar.h"
#include "widget/DlgTextView.h"
#include "widget/DlgTextBrowser.h"
#include "widget/DlgUser.h"
#include "widget/DlgVBox.h"
#include "widget/DlgVButtonGroup.h"
#include "widget/DlgVGroupBox.h"
#include "pixmap/PixmapDraw.h"

#include "prop/WidgetBitmapPage.h"
#include "prop/WidgetPropDialog.h"

#include "bmp/PixmapFactory.h"

#include "xml/QtArchExpat.h"
#include "xml/QtArchGenerator.h"
#include "xml/XMLValue.h"


QString DlgWidget::srcIndent = "    ";
WidgetPropDialog *DlgWidget::propDialog = 0;

DlgWidget::DlgWidget(
    Type	type
    ) :
    useBackgroundPixmap( FALSE ),
    itsWidget( 0 ),
    itsType( type ),
    isInLayout( FALSE ),
    itsOriginalParent( 0 ),
    adjustSize( FALSE ),
    addPixmapToProject( FALSE ),
    readPixmapFromData( FALSE),
    isEnabled( TRUE ),
    itsParentForSource(NULL)
{
    itsConnectedSignalList.setAutoDelete( TRUE );
}


DlgWidget::~DlgWidget()
{
    itsConnectedSignalList.clear();
}


DlgWidget *DlgWidget::NewDlgWidget( 
    QWidget *parent,
    DlgWidget::Type type,
    const QString *name 
    )
{
    DlgWidget *newWidget = NULL;

    const char *theName = NULL;

    if( name )
	theName = *name;

#define NEW_DLG_WIDGET_BY_TYPE(type) \
    case DlgWidget::DW_##type:				\
    {							\
	newWidget = new Dlg##type;	\
	break;						\
    }
    
    switch( type )
    {
	NEW_DLG_WIDGET_BY_TYPE( Button )
	NEW_DLG_WIDGET_BY_TYPE( ButtonGroup )
	NEW_DLG_WIDGET_BY_TYPE( CheckBox )
	NEW_DLG_WIDGET_BY_TYPE( ComboBox )
	NEW_DLG_WIDGET_BY_TYPE( Frame )
	NEW_DLG_WIDGET_BY_TYPE( Grid )
	NEW_DLG_WIDGET_BY_TYPE( GroupBox )
	NEW_DLG_WIDGET_BY_TYPE( HBox )
	NEW_DLG_WIDGET_BY_TYPE( HButtonGroup )
	NEW_DLG_WIDGET_BY_TYPE( HGroupBox )
	NEW_DLG_WIDGET_BY_TYPE( IconView )
	NEW_DLG_WIDGET_BY_TYPE( LCDNumber )
	NEW_DLG_WIDGET_BY_TYPE( Label )
	NEW_DLG_WIDGET_BY_TYPE( LineEdit )
	NEW_DLG_WIDGET_BY_TYPE( ListBox )
	NEW_DLG_WIDGET_BY_TYPE( ListView )
	NEW_DLG_WIDGET_BY_TYPE( MenuBar )
	NEW_DLG_WIDGET_BY_TYPE( MultiLineEdit )
	NEW_DLG_WIDGET_BY_TYPE( ProgressBar )
	NEW_DLG_WIDGET_BY_TYPE( PushButton )
	NEW_DLG_WIDGET_BY_TYPE( RadioButton )
	NEW_DLG_WIDGET_BY_TYPE( ScrollBar )
	NEW_DLG_WIDGET_BY_TYPE( ScrollView )
	NEW_DLG_WIDGET_BY_TYPE( Slider )
	NEW_DLG_WIDGET_BY_TYPE( SpinBox )
	NEW_DLG_WIDGET_BY_TYPE( Splitter )
	NEW_DLG_WIDGET_BY_TYPE( TabBar )
	NEW_DLG_WIDGET_BY_TYPE( TextView )
	NEW_DLG_WIDGET_BY_TYPE( TextBrowser )
	NEW_DLG_WIDGET_BY_TYPE( User )
	NEW_DLG_WIDGET_BY_TYPE( VBox )
	NEW_DLG_WIDGET_BY_TYPE( VButtonGroup )
	NEW_DLG_WIDGET_BY_TYPE( VGroupBox )

	case DlgWidget::DW_None:
	{
	    break;
	}
    }
    
    newWidget->SetParent( parent );  
    newWidget->CreateQtWidget( theName );

    return newWidget;
}


DlgWidget *DlgWidget::NewDlgWidget( 
    QWidget *parent, 
    const QString &type, 
    const QString *name 
    )
{
    DlgWidget *newWidget = NULL;

    const char *theName = NULL;

    if( name )
	theName = *name;

#define ELSE_IF_TYPE( x )		\
    else if( type == #x )		\
	newWidget = new Dlg##x;	

    if( type == "Button" )
	newWidget = new DlgButton;
    ELSE_IF_TYPE( ButtonGroup )
    ELSE_IF_TYPE( CheckBox )
    ELSE_IF_TYPE( ComboBox )
    ELSE_IF_TYPE( Frame )
    ELSE_IF_TYPE( Grid )
    ELSE_IF_TYPE( GroupBox )
    ELSE_IF_TYPE( HBox )
    ELSE_IF_TYPE( HButtonGroup )
    ELSE_IF_TYPE( HGroupBox )
    ELSE_IF_TYPE( IconView )
    ELSE_IF_TYPE( LCDNumber )
    ELSE_IF_TYPE( Label )
    ELSE_IF_TYPE( LineEdit )
    ELSE_IF_TYPE( ListBox )
    ELSE_IF_TYPE( ListView )
    ELSE_IF_TYPE( MenuBar )
    ELSE_IF_TYPE( MultiLineEdit )
    ELSE_IF_TYPE( ProgressBar )
    ELSE_IF_TYPE( PushButton )
    ELSE_IF_TYPE( RadioButton )
    ELSE_IF_TYPE( ScrollBar )
    ELSE_IF_TYPE( ScrollView )
    ELSE_IF_TYPE( Slider )
    ELSE_IF_TYPE( SpinBox )
    ELSE_IF_TYPE( Splitter )
    ELSE_IF_TYPE( TabBar )
    ELSE_IF_TYPE( TextView )
    ELSE_IF_TYPE( TextBrowser )
    ELSE_IF_TYPE( User )
    ELSE_IF_TYPE( VBox )
    ELSE_IF_TYPE( VButtonGroup )
    ELSE_IF_TYPE( VGroupBox )
#undef ELSE_IF_TYPE
      
    newWidget->SetParent( parent );  
    newWidget->CreateQtWidget( theName );

    return newWidget;
}

DlgWidget *DlgWidget::NewDlgWidget( 
    QWidget *parent, 
    const XMLValue &value
    )
{
    DlgWidget *newWidget = NULL;

    QString type = value.GetTypeString();

#define ELSE_IF_TYPE( x )		\
    else if( type == #x )		\
	newWidget = new Dlg##x;	

    if( value.GetTypeString() == "Button" )
	newWidget = new DlgButton;
    ELSE_IF_TYPE( ButtonGroup )
    ELSE_IF_TYPE( CheckBox )
    ELSE_IF_TYPE( ComboBox )
    ELSE_IF_TYPE( Frame )
    ELSE_IF_TYPE( Grid )
    ELSE_IF_TYPE( GroupBox )
    ELSE_IF_TYPE( HBox )
    ELSE_IF_TYPE( HButtonGroup )
    ELSE_IF_TYPE( HGroupBox )
    ELSE_IF_TYPE( IconView )
    ELSE_IF_TYPE( LCDNumber )
    ELSE_IF_TYPE( Label )
    ELSE_IF_TYPE( LineEdit )
    ELSE_IF_TYPE( ListBox )
    ELSE_IF_TYPE( ListView )
    ELSE_IF_TYPE( MenuBar )
    ELSE_IF_TYPE( MultiLineEdit )
    ELSE_IF_TYPE( ProgressBar )
    ELSE_IF_TYPE( PushButton )
    ELSE_IF_TYPE( RadioButton )
    ELSE_IF_TYPE( ScrollBar )
    ELSE_IF_TYPE( ScrollView )
    ELSE_IF_TYPE( Slider )
    ELSE_IF_TYPE( SpinBox )
    ELSE_IF_TYPE( Splitter )
    ELSE_IF_TYPE( TabBar )
    ELSE_IF_TYPE( TextView )
    ELSE_IF_TYPE( TextBrowser )
    ELSE_IF_TYPE( User )
    ELSE_IF_TYPE( VBox )
    ELSE_IF_TYPE( VButtonGroup )
    ELSE_IF_TYPE( VGroupBox )
#undef ELSE_IF_TYPE

      // Sets the parent and a dummy widget  
      if( newWidget )
      {
        newWidget->SetParent( parent );  
        newWidget->RestoreXML( value );
      }

    return newWidget;
}


QPoint DlgWidget::GetPos() const
{
    QWidget* widget = GetWidget();
    return itsOriginalParent->mapFromGlobal(widget->mapToGlobal(QPoint(0,0)));
}

void DlgWidget::UpdateProperty()
{
  if( propDialog &&
      propDialog->isVisible() )
    propDialog->ChangeWidget( this );
}

void DlgWidget::DoPropertyDialog()
{
  if( ! propDialog )
  {
    propDialog = new WidgetPropDialog( this,
                                       NULL /*GetWidget()*/,
                                       "Properties" );
  } else {
    propDialog->ChangeWidget( this );
    
  }
  propDialog->show();
    
//  GetWidget()->update();
    
}

void DlgWidget::Changed()
{
  emit PropertiesChanged( this );
}  

void DlgWidget::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    pages.append( new WidgetBitmapPage( this, parent, "Bitmap" ) );

    labels.append( tr( "Bitmap" ) );
}


QString DlgWidget::GetGeneratedVarName() const
{
    QString str = "qtarch_";
    
    if( IsVariableMapped() )
    {
	str = GetVariableName();
    }
    else
    {
	str += GetWidgetName();
    }

    return str;
}


void DlgWidget::SetInLayout(
    QWidget*	newParent 
    )
{
    isInLayout = TRUE;

    QWidget::FocusPolicy focusPolicy = GetWidget()->focusPolicy();
    
    // If a new parent is given, set the widget's parent to it.  If not given,
    // set the parent to the widget's original parent.
    if( newParent )
    {
	QWidget* widget = GetWidget();
	QPoint pos = newParent->mapFromGlobal(
	    widget->mapToGlobal( QPoint(0,0) ) );
	
	widget->recreate( newParent, 0, pos, TRUE );
    }
    else
    {
	// Only set if the widget's parent is currently not the original 
	// parent.
	if( GetWidget()->parent() != itsOriginalParent )
	{
	    QWidget* widget = GetWidget();
	    QPoint pos = itsOriginalParent->mapFromGlobal( 
		widget->mapToGlobal( QPoint(0,0) ) );
		
	    widget->recreate( itsOriginalParent, 0, pos, TRUE );
	}
    }

    GetWidget()->setFocusPolicy( focusPolicy );

    emit LayoutStatusChanged( this );
}


void DlgWidget::RemoveFromLayout()
{
    isInLayout = FALSE;

    // Don't recreate if the app is going down
    if( !QApplication::closingDown() )
    {
	QWidget* widget = GetWidget();

	QWidget::FocusPolicy focusPolicy = widget->focusPolicy();

	QPoint pos = 
	    itsOriginalParent->mapFromGlobal(widget->mapToGlobal(QPoint(0,0)));
    
	widget->recreate( itsOriginalParent, 0, pos, TRUE );

	widget->setMinimumSize( 0,0 );
	widget->setMaximumSize( QWIDGETSIZE_MAX,QWIDGETSIZE_MAX );
	widget->setFocusPolicy( focusPolicy );

	emit LayoutStatusChanged( this );
    }
}


void DlgWidget::Save( QtArchGenerator&	generator )
{
  generator.WriteStartBlock( "DlgWidget" );
  
  generator.WriteXMLValue( "Rect", MapGeometryToOriginalParent( itsWidget ) );
  generator.WriteSingleLine( "Name", itsWidget->name() );

    if( IsVariableMapped() )
      generator.WriteSingleLine( "Variable", itsVariableName );

    // user changed palette
    const QPalette palette = itsWidget->palette();
    if( palette != QApplication::palette() )
      generator.WriteXMLValue( "Palette", palette );

    // user changed font
    const QFont font = itsWidget->font();
    if( font != QApplication::font() )
      generator.WriteXMLValue( "Font", font );

    QListIterator<SignalConnection> iter( itsConnectedSignalList );
    SignalConnection* sig;

    generator.WriteStartList( "SignalConnection" );
    while( (sig = iter()) )
    {
      generator.WriteSingleLine( "Signal", sig->ToString() );
    }
    generator.WriteEndBlock();

    generator.WriteXMLValue( "MinimumSize", itsWidget->minimumSize() );
    generator.WriteXMLValue( "MaximumSize", itsWidget->maximumSize() );
    generator.WriteSingleLine( "BackgroundPixmap", backgroundPixmapsName );
    generator.WriteXMLValue( "UseBackgroundPixmap", useBackgroundPixmap );
    generator.WriteXMLValue( "AdjustSize", adjustSize );
    generator.WriteXMLValue( "ReadPixmapFromData", readPixmapFromData );
    generator.WriteXMLValue( "Enabled", isEnabled );
    generator.WriteSingleLine( "DataVarName", dataVarName );
    generator.WriteSingleLine( "DataLenName", dataLenName );
    generator.WriteSingleLine( 
        "FocusPolicy",
        FocusPolicyToStr( itsWidget->focusPolicy() )  );
    generator.WriteSingleLine( 
        "BackgroundMode",
        BackgroundModeToStr( itsWidget->backgroundMode() ) );

    generator.WriteSingleLine( 
        "BackgroundOrigin",
        BackgroundOriginToStr( itsWidget->backgroundOrigin() ) );

    generator.WriteEndBlock();
}


bool DlgWidget::RestoreKeyValue( 
    const QString&	key, 
    const QString&	value,
    bool		/*topLevel*/
    )
{
    bool rc = TRUE;

    QString val = value;
    QTextStream read( val, IO_ReadOnly );

    if( key == "Name" )
    {
	itsWidget->setName( value );
    } 
    else if( key == "Rect" )
    {
	QRect aRect;
	read >> aRect;
	itsWidget->setGeometry( aRect );
    } 
    else if( key == "Variable" )
    {
	itsVariableName = value;
    }
    else if( key == "Palette" )
    {
	QPalette palette;
	read >> palette;
	itsWidget->setPalette( palette );
    }
    else if( key == "Font" )
    {
	QFont font;
	read >> font;
	itsWidget->setFont( font );
    }
    else if( key == "Signal" )
    {
	SignalConnection* sig = new SignalConnection;
	sig->FromString( value );
	
	itsConnectedSignalList.append( sig );
    }
    else if( key == "MinimumSize" )
    {
	QSize size;
	read >> size;
	itsWidget->setMinimumSize( size );
    }
    else if( key == "MaximumSize" )
    {
	QSize size;
	read >> size;
	itsWidget->setMaximumSize( size );
    }
    else if( key == "BackgroundPixmap" )
    {
	backgroundPixmapsName = value;
    }
    else if( key == "UseBackgroundPixmap" )
    {
	RestoreBool( read, useBackgroundPixmap );
	if( ! backgroundPixmapsName.isEmpty() && useBackgroundPixmap )
	{
	    QPixmap pixmap( backgroundPixmapsName );
	    if( itsWidget->inherits( "QButton" ) )
		((QButton *)itsWidget)->setPixmap( pixmap );
	    else
		itsWidget->setBackgroundPixmap( pixmap );
	}  
    }
    else if( key == "AdjustSize" )
    {
	RestoreBool( read, adjustSize );
	if( adjustSize && useBackgroundPixmap )
	{
	    const QPixmap *pixmap = itsWidget->backgroundPixmap();
	    if( pixmap )
		itsWidget->setFixedSize( pixmap->size() );
	}
    }
    else if( key == "ReadPixmapFromData" )
    {
	RestoreBool( read, readPixmapFromData );
    }
    else if( key == "DataVarName" )
    {
	dataVarName = value;
    }
    else if( key == "DataLenName" )
    {
	dataLenName = value;
    }
    else if( key == "FocusPolicy" )
    {
	itsWidget->setFocusPolicy( StrToFocusPolicy( value ) );
    }
    else if( key == "BackgroundMode" )
    {
	QWidget::BackgroundMode mode = StrToBackgroundMode( value );
	if( (mode != QWidget::FixedPixmap) && (mode != QWidget::FixedColor) )
	{
	    itsWidget->setBackgroundMode( mode );
	}
    }
    else if( key == "FontPropagation" )
    {
        // Deprecated
    }
    else if( key == "PalettePropagation" )
    {
        // Deprecated
    }
    else if( key == "Enabled" )
    {
        isEnabled = (value == "TRUE");
    }
    else
    {
	// Key unknown
	rc = FALSE;
    }
    
    return rc;
}

void DlgWidget::RestoreXML( const XMLValue &value )
{
    // Sanity check
    if( value.GetTypeString() != "DlgWidget" )
        return;

    const XMLValueMap map = value.AsNested();

    QWidget* widget = GetWidget();

    widget->setGeometry( map["Rect"].AsQRect() );

    {
      QString tmp = map["Name"].AsQString();
      QObjectList *list = GetWidget()->parent()->queryList( 0, tmp );
      if( list->isEmpty() )
      {
        widget->setName( tmp );
      }
      delete list;
    }

    itsVariableName = map["Variable"].AsQString();
    if( map.contains( "Palette" ) )
        widget->setPalette( map["Palette"].AsQPalette() );
    if( map.contains( "Font" ) )
        widget->setFont( map["Font"].AsQFont() );

    XMLValueList list = map["SignalConnection"].AsList();
    XMLValueList::Iterator it( list );
    for( ; it.current(); ++it )
    {
        SignalConnection* sig = new SignalConnection;
        sig->FromString( it.current()->AsQString() );
	
        itsConnectedSignalList.append( sig );
    }

    widget->setMinimumSize( map["MinimumSize"].AsQSize() );
    widget->setMaximumSize( map["MaximumSize"].AsQSize() );
    backgroundPixmapsName = map["BackgroundPixmap"].AsQString();
    useBackgroundPixmap = map["UseBackgroundPixmap"].AsBool();
    adjustSize = map["AdjustSize"].AsBool();
    readPixmapFromData = map["ReadPixmapFromData"].AsBool();
    isEnabled = map["Enabled"].AsBool();
    dataVarName = map["DataVarName"].AsQString();
    dataLenName = map["DataLenName"].AsQString();
  
    itsWidget->setFocusPolicy( 
        StrToFocusPolicy( map["FocusPolicy"].AsQString() ) );

    QWidget::BackgroundMode mode = 
        StrToBackgroundMode( map["BackgroundMode"].AsQString() );
    if( (mode != QWidget::FixedPixmap) && (mode != QWidget::FixedColor) )
    {
        itsWidget->setBackgroundMode( mode );
    }

    itsWidget->setBackgroundOrigin(
        StrToBackgroundOrigin( map["BackgroundOrigin"].AsQString() ) );
}


void DlgWidget::GenerateCtor(
    QTextStream&	stream,
    const QString&	parentName
    )
{
    QString name;
    const char* cp = GetWidget()->name();
    if( cp )
    {
	name = "\"";
	name += cp;
	name += "\"";
    }
    else
    {
	name = "NULL";
    }

    stream << parentName << ", " << name;
}


void DlgWidget::GenerateSource(
    QTextStream&	stream,
    const QString&	varName,
    const WidgetLayout& /*widgetLayout*/
    )
{
    stream << varName << "setGeometry(";
    if(itsParentForSource!=NULL)
        stream << itsWidget->x()-itsParentForSource->GetWidget()->x() << ", " << itsWidget->y()-itsParentForSource->GetWidget()->y();
    else
        stream << itsWidget->x() << ", " << itsWidget->y();
    stream << ", " << itsWidget->width() << ", " << itsWidget->height() << ");\n";

    stream << varName << "setMinimumSize(" << itsWidget->minimumSize().width() << ", " <<  itsWidget->minimumSize().height() << ");\n"
           << varName << "setMaximumSize(" << itsWidget->maximumSize().width() << ", " << itsWidget->maximumSize().height() << ");\n";

    const QPalette palette = itsWidget->palette();
    if( palette != QApplication::palette() )
    {
	stream << srcIndent<< "{\n"

               << srcIndent << srcIndent <<"QColorGroup normal;" << endl;
        stream << srcIndent << srcIndent 
               << "normal.setColor( QColorGroup::Foreground, "
               << "QColor( QRgb( " 
               << palette.normal().foreground().rgb() << "U ) ) );" << endl;
        stream << srcIndent << srcIndent 
               << "normal.setColor( QColorGroup::Button, "
               << "QColor( QRgb( " 
               << palette.normal().button().rgb() << "U ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "normal.setColor( QColorGroup::Light, "
               << "QColor( QRgb( " 
               << palette.normal().light().rgb() << "U ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "normal.setColor( QColorGroup::Midlight, "
               << "QColor( QRgb( " 
               << palette.normal().midlight().rgb() << "U ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "normal.setColor( QColorGroup::Dark, "
               << "QColor( QRgb( " 
               << palette.normal().dark().rgb() << "U ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "normal.setColor( QColorGroup::Mid, "
               << "QColor( QRgb( " 
               << palette.normal().mid().rgb() << "U ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "normal.setColor( QColorGroup::Text, "
               << "QColor( QRgb( " 
               << palette.normal().text().rgb() << "U ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "normal.setColor( QColorGroup::BrightText, "
               << "QColor( QRgb( " 
               << palette.normal().brightText().rgb() << "U ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "normal.setColor( QColorGroup::ButtonText, "
               << "QColor( QRgb( " 
               << palette.normal().buttonText().rgb() << "U ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "normal.setColor( QColorGroup::Base, "
               << "QColor( QRgb( " 
               << palette.normal().base().rgb() << "U ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "normal.setColor( QColorGroup::Background, "
               << "QColor( QRgb( " 
               << palette.normal().background().rgb() << "U ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "normal.setColor( QColorGroup::Shadow, "
               << "QColor( QRgb( " 
               << palette.normal().shadow().rgb() << "U ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "normal.setColor( QColorGroup::Highlight, "
               << "QColor( QRgb( " 
               << palette.normal().highlight().rgb() << "U ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "normal.setColor( QColorGroup::HighlightedText, "
               << "QColor( QRgb( " 
               << palette.normal().highlightedText().rgb() << "U ) ) );" << endl;

        stream << srcIndent << srcIndent << "QColorGroup disabled;" << endl;
        stream << srcIndent << srcIndent
               << "disabled.setColor( QColorGroup::Foreground, "
               << "QColor( QRgb( " 
               << palette.disabled().foreground().rgb() << "U ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "disabled.setColor( QColorGroup::Button, "
               << "QColor( QRgb( " 
               << palette.disabled().button().rgb() << "U ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "disabled.setColor( QColorGroup::Light, "
               << "QColor( QRgb( " 
               << palette.disabled().light().rgb() << "U ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "disabled.setColor( QColorGroup::Midlight, "
               << "QColor( QRgb( " 
               << palette.disabled().midlight().rgb() << "U ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "disabled.setColor( QColorGroup::Dark, "
               << "QColor( QRgb( " 
               << palette.disabled().dark().rgb() << "U ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "disabled.setColor( QColorGroup::Mid, "
               << "QColor( QRgb( " 
               << palette.disabled().mid().rgb() << "U ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "disabled.setColor( QColorGroup::Text, "
               << "QColor( QRgb( " 
               << palette.disabled().text().rgb() << "U ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "disabled.setColor( QColorGroup::BrightText, "
               << "QColor( QRgb( " 
               << palette.disabled().brightText().rgb() << "U ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "disabled.setColor( QColorGroup::ButtonText, "
               << "QColor( QRgb( " 
               << palette.disabled().buttonText().rgb() << "U ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "disabled.setColor( QColorGroup::Base, "
               << "QColor( QRgb( " 
               << palette.disabled().base().rgb() << "U ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "disabled.setColor( QColorGroup::Background, "
               << "QColor( QRgb( " 
               << palette.disabled().background().rgb() << "U ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "disabled.setColor( QColorGroup::Shadow, "
               << "QColor( QRgb( " 
               << palette.disabled().shadow().rgb() << "U ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "disabled.setColor( QColorGroup::Highlight, "
               << "QColor( QRgb( " 
               << palette.disabled().highlight().rgb() << "U ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "disabled.setColor( QColorGroup::HighlightedText, "
               << "QColor( QRgb( " 
               << palette.disabled().highlightedText().rgb() << "U ) ) );" << endl;

        stream << srcIndent << srcIndent << "QColorGroup active;" << endl;
        stream << srcIndent << srcIndent
               << "active.setColor( QColorGroup::Foreground, "
               << "QColor( QRgb( " 
               << palette.active().foreground().rgb() << "U ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "active.setColor( QColorGroup::Button, "
               << "QColor( QRgb( " 
               << palette.active().button().rgb() << "U ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "active.setColor( QColorGroup::Light, "
               << "QColor( QRgb( " 
               << palette.active().light().rgb() << "U ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "active.setColor( QColorGroup::Midlight, "
               << "QColor( QRgb( " 
               << palette.active().midlight().rgb() << "U ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "active.setColor( QColorGroup::Dark, "
               << "QColor( QRgb( " 
               << palette.active().dark().rgb() << "U ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "active.setColor( QColorGroup::Mid, "
               << "QColor( QRgb( " 
               << palette.active().mid().rgb() << "U ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "active.setColor( QColorGroup::Text, "
               << "QColor( QRgb( " 
               << palette.active().text().rgb() << "U ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "active.setColor( QColorGroup::BrightText, "
               << "QColor( QRgb( " 
               << palette.active().brightText().rgb() << "U ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "active.setColor( QColorGroup::ButtonText, "
               << "QColor( QRgb( " 
               << palette.active().buttonText().rgb() << "U ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "active.setColor( QColorGroup::Base, "
               << "QColor( QRgb( " 
               << palette.active().base().rgb() << "U ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "active.setColor( QColorGroup::Background, "
               << "QColor( QRgb( " 
               << palette.active().background().rgb() << "U ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "active.setColor( QColorGroup::Shadow, "
               << "QColor( QRgb( " 
               << palette.active().shadow().rgb() << "U ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "active.setColor( QColorGroup::Highlight, "
               << "QColor( QRgb( " 
               << palette.active().highlight().rgb() << "U ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "active.setColor( QColorGroup::HighlightedText, "
               << "QColor( QRgb( " 
               << palette.active().highlightedText().rgb() << "U ) ) );" << endl;

	stream << srcIndent << srcIndent
               << "QPalette palette( normal, disabled, active );\n";
	stream << srcIndent << varName << "setPalette( palette );\n";
	stream << srcIndent << "}\n";
    }

    const QFont font = itsWidget->font();
    if( font != QApplication::font() )
    {
	stream << srcIndent << "{\n"
	       << srcIndent << srcIndent 
               << "QFont font(\"" << font.family() << "\", " 
               << font.pointSize() << ", "
	       << font.weight() << ", " ;
        SaveBool( stream, font.italic() );
        stream << ");\n";
	if( font.underline() )
	{
	    stream << srcIndent << srcIndent << "font.setUnderline(TRUE);\n";
	}
	if( font.strikeOut() )
	{
	    stream << srcIndent << srcIndent << "font.setStrikeOut(TRUE);\n";
	}
	if( font.fixedPitch() )
	{
	    stream << srcIndent << srcIndent << "font.setFixedPitch(TRUE);\n";
	}
	stream << srcIndent << srcIndent 
               << "font.setStyleHint((QFont::StyleHint)" << (int)font.styleHint() 
               << ");\n"
	       << srcIndent << srcIndent 
               << "font.setCharSet((QFont::CharSet)" << (int)font.charSet()
               << ");\n"
	       << srcIndent << varName << "setFont( font );\n"
	       << srcIndent << "}\n";
    }

    // Generate the background image code
    if( useBackgroundPixmap )
    {
	QString pixName = "pixmap_" + varName;
	stream << endl 
               << srcIndent << '{' << endl 
               << srcIndent << srcIndent << "QPixmap pixmap;\n";
	if( readPixmapFromData )
	{
	    if( addPixmapToProject )
	    {
		// We could expect the data somewhere in the binary
		stream << srcIndent << srcIndent 
                       << "pixmap.loadFromData(" << dataVarName << " , " 
                       << dataLenName << ");\n";
	    } else
	    {
		// Create the pixmap data right here and use it.
		QPixmap pixmap( backgroundPixmapsName );
		PixmapDraw::GeneratePixmap( &pixmap, dataVarName, dataLenName, (QFile *) stream.device() );
		stream << srcIndent << srcIndent 
                       << "pixmap.loadFromData(" << dataVarName << " , "
                       << dataLenName << ");\n";
	    }
	} else
	{
	    stream << srcIndent << srcIndent 
                   << "pixmap.load(\"" << backgroundPixmapsName << "\");\n";
	}
	if( itsWidget->inherits( "QButton" ) )
	    stream << srcIndent << varName << "setPixmap(pixmap);\n" 
                   << srcIndent << "}\n";
	else
	    stream << srcIndent << varName << "setBackgroundPixmap(pixmap);\n"
                   << srcIndent << "}\n";
    }

    stream << varName << "setFocusPolicy(QWidget::" 
           << FocusPolicyToStr( itsWidget->focusPolicy() ) 
           << ");\n";

    QWidget::BackgroundMode mode = itsWidget->backgroundMode();
    if( (mode != QWidget::FixedPixmap) && (mode != QWidget::FixedColor) )
    {
	stream << varName << "setBackgroundMode(QWidget::" 
               << BackgroundModeToStr( mode ) << ");\n";
    }

    stream << varName << "setBackgroundOrigin( QWidget::"
           << BackgroundOriginToStr( itsWidget->backgroundOrigin() )
           << " );" << endl;
}


void DlgWidget::GenerateSourceConnects(
    QTextStream& stream, 
    const QString& varName
    )
{
    QListIterator<SignalConnection> iter( GetConnectedSignalList() );
    SignalConnection* sig;
    QString dot = Dot();
 
    // remove the initial tab and the . or -> from the variable name to use
    // in the connect statement
    QString name = varName.right( varName.length() - srcIndent.length() );
    name = name.left( name.length() - dot.length() );

    while( (sig = iter()) )
    {
        stream << srcIndent 
               << "connect(" << name 
               << ", SIGNAL(" << sig->GetSignal() 
               << "), SLOT(" << sig->GetSlot() << "));\n";
    }
}


void DlgWidget::GenerateText(
    const QString&      text,
    const WidgetLayout& widgetLayout,
    QTextStream&        stream,
    bool                noTranslate
    )
{
    QString str = text;
    
    // Do \ first because the other tests will add \'s
    if( str.contains( '\\' ) )
    {
        static QRegExp backslash( "\\" );
        str = str.replace( backslash, "\\\\" );
    }
    
    if( str.contains( '\n' ) )
    {
        static QRegExp nl( "\n" );
        str = str.replace( nl, "\\n" );
    }
    
    if( str.contains( '\t' ) )
    {
        static QRegExp tab( "\t" );
        str = str.replace( tab, "\\t" );
    }
    
    if( str.contains( '"' ) )
    {
        static QRegExp quote( "\"" );
        str = str.replace( quote, "\\\"" );
    }

    if( !str.isEmpty() && !noTranslate && widgetLayout.IsTextTranslated() )
    {
        stream << "tr( \"" << str << "\" )";
    }
    else
    {
        stream << "\"" << str << "\"";
    }
}


QString DlgWidget::CreateName( 
    QString className, 
    const char *name, 
    int &number 
    )
{
    QString tmp;

    if( ! name )
    {
	tmp.setNum( number );
	tmp = className + QString( "_" ) + tmp;
	number++;
    } 
    else
    {
	tmp = name;
    }

    return tmp;
}


void DlgWidget::UpdateNextWidgetNumber( 
    const QString&	widgetName,
    int&		nextWidgetNum 
    )
{
  int pos = widgetName.findRev( '_' );

  if( pos >= 0 )
  {
    QString numStr = widgetName.mid( pos + 1, 
                                     widgetName.length() - pos - 1 );

    int num = numStr.toInt();
    if( num >= nextWidgetNum )
    {
	    nextWidgetNum = num + 1;
    }
  }    
}


const char* DlgWidget::Dot() const
{
    // all variables are now dynamically created
    //return IsVariableMapped() ? "." : "->";
    return "->";
}


void DlgWidget::GetSignalList( 
    QStrList& signalList
    ) const
{
    signalList.clear();
    
    AddSignalsToList( signalList );
}


void DlgWidget::AddSignalsToList( 
    QStrList&	// signalList
    ) const 
{
}


void DlgWidget::SetBackgroundData( 
    bool use, 
    bool adjust, 
    bool add, 
    bool read, 
    const QString &varName, 
    const QString &lenName, 
    const QString &fileName 
    )
{
    backgroundPixmapsName = fileName;
    
    useBackgroundPixmap = use;
    adjustSize = adjust;
    addPixmapToProject = add;
    readPixmapFromData = read;
    dataVarName = varName;
    dataLenName = lenName;
}


void DlgWidget::GetBackgroundData( 
    bool &use, 
    bool &adjust, 
    bool &add,
    bool &read, 
    QString &varName, 
    QString &lenName, 
    QString &fileName 
    )
{
    fileName = backgroundPixmapsName;

    use = useBackgroundPixmap;
    adjust = adjustSize;
    add = addPixmapToProject;
    read = readPixmapFromData;
    varName = dataVarName;
    lenName = dataLenName;
}


const QPixmap& DlgWidget::GetPixmap( DlgWidget::Type type )
{
    BitmapType bitmapType = Frame_bmp;

#define DW_PIXMAP( widget ) \
    case DW_##widget: bitmapType = widget##_bmp; break;
    
    // Translate widget type to bitmap type
    switch( type )
    {
	DW_PIXMAP( Button )
	DW_PIXMAP( ButtonGroup )
	DW_PIXMAP( CheckBox )
	DW_PIXMAP( ComboBox )
	DW_PIXMAP( Frame )
	DW_PIXMAP( Grid )
	DW_PIXMAP( GroupBox )
	DW_PIXMAP( HBox )
	DW_PIXMAP( HButtonGroup )
	DW_PIXMAP( HGroupBox )
	DW_PIXMAP( IconView )
	DW_PIXMAP( LCDNumber )
	DW_PIXMAP( Label )
	DW_PIXMAP( LineEdit )
	DW_PIXMAP( ListBox )
	DW_PIXMAP( ListView )
	DW_PIXMAP( MenuBar )
	DW_PIXMAP( MultiLineEdit )
	DW_PIXMAP( ProgressBar )
	DW_PIXMAP( PushButton )
	DW_PIXMAP( RadioButton )
	DW_PIXMAP( ScrollBar )
	DW_PIXMAP( ScrollView )
	DW_PIXMAP( Slider )
	DW_PIXMAP( SpinBox )
	DW_PIXMAP( Splitter )
	DW_PIXMAP( TabBar )
	DW_PIXMAP( TextBrowser )
	DW_PIXMAP( TextView )
	DW_PIXMAP( User )
	DW_PIXMAP( VBox )
	DW_PIXMAP( VButtonGroup )
	DW_PIXMAP( VGroupBox )

	case DW_None:
	default:
	{
	    break;
	}
    }
    
    return PixmapFactory::GetPixmap( bitmapType );
}


QRect DlgWidget::MapGeometryToOriginalParent(
    const QWidget*	theWidget
    ) const
{
    QRect rect = theWidget->rect();
    
    return QRect( 
	itsOriginalParent->mapFromGlobal( 
	    theWidget->mapToGlobal( rect.topLeft() ) ),
	itsOriginalParent->mapFromGlobal( 
	    theWidget->mapToGlobal( rect.bottomRight() ) ) );
}


DlgWidget * DlgWidget::GetParentForCodeGeneration(
    const QList<DlgWidget>& zOrdering
    )
{
    DlgWidget                *widget;
    QListIterator<DlgWidget> iter(zOrdering);

    //
    // A widget's parnt will be before it within the zOrder, and be capable of
    // having children, and will have a region that contains this widget.
    //
    while( (widget=iter()) )
        if(widget==this)
            break;
        else
            if ((widget->GetType()==DW_ButtonGroup ||
                 widget->GetType()==DW_GroupBox) &&
                 widget->GetWidget()->x()<=itsWidget->x() &&
                 widget->GetWidget()->y()<=itsWidget->y() &&
                 (widget->GetWidget()->x()+widget->GetWidget()->width())>=(itsWidget->x()+itsWidget->width())&&
                 (widget->GetWidget()->y()+widget->GetWidget()->height())>=(itsWidget->y()+itsWidget->height()))
                itsParentForSource=widget;

    return itsParentForSource;
}


QString DlgWidget::FocusPolicyToStr(
    QWidget::FocusPolicy	theFocusPolicy
    )
{
    QString str;
    
    switch( theFocusPolicy )
    {
	case QWidget::TabFocus:
	{
	    str = "TabFocus";
	    break;
	}
	case QWidget::ClickFocus:
	{
	    str = "ClickFocus";
	    break;
	}
	case QWidget::StrongFocus:
	{
	    str = "StrongFocus";
	    break;
	}
	case QWidget::WheelFocus:
	{
	    str = "WheelFocus";
	    break;
	}
	case QWidget::NoFocus:
	{
	    str = "NoFocus";
	    break;
	}
    }
    
    return str;
}


QWidget::FocusPolicy DlgWidget::StrToFocusPolicy(
    const QString&	theFocusPolicyStr
    )
{
    QWidget::FocusPolicy focusPolicy = QWidget::NoFocus;
    
    if( theFocusPolicyStr == "TabFocus" )
    {
	focusPolicy = QWidget::TabFocus;
    }
    else if( theFocusPolicyStr == "ClickFocus" )
    {
	focusPolicy = QWidget::ClickFocus;
    }
    else if( theFocusPolicyStr == "StrongFocus" )
    {
	focusPolicy = QWidget::StrongFocus;
    }
    else if( theFocusPolicyStr == "WheelFocus" )
    {
	focusPolicy = QWidget::WheelFocus;
    }
    else if( theFocusPolicyStr == "NoFocus" )
    {
	focusPolicy = QWidget::NoFocus;
    }
    
    return focusPolicy;
}


QString DlgWidget::BackgroundModeToStr(
    QWidget::BackgroundMode	theBackgroundMode
    )
{
    QString str;
    
    switch( theBackgroundMode )
    {
	case QWidget::FixedColor:
	{
	    str = "FixedColor";
	    break;
	}
	case QWidget::FixedPixmap:
	{
	    str = "FixedPixmap";
	    break;
	}
	case QWidget::NoBackground:
	{
	    str = "NoBackground";
	    break;
	}
	case QWidget::PaletteForeground:
	{
	    str = "PaletteForeground";
	    break;
	}
	case QWidget::PaletteBackground:
	{
	    str = "PaletteBackground";
	    break;
	}
	case QWidget::PaletteLight:
	{
	    str = "PaletteLight";
	    break;
	}
	case QWidget::PaletteMidlight:
	{
	    str = "PaletteMidlight";
	    break;
	}
	case QWidget::PaletteDark:
	{
	    str = "PaletteDark";
	    break;
	}
	case QWidget::PaletteMid:
	{
	    str = "PaletteMid";
	    break;
	}
	case QWidget::PaletteText:
	{
	    str = "PaletteText";
	    break;
	}
	case QWidget::PaletteBase:
	{
	    str = "PaletteBase";
	    break;
	}
        case QWidget::PaletteButton:
        {
            str = "PaletteButton";
            break;
        }
        case QWidget::PaletteBrightText:
        {
            str = "PaletteBrightText";
            break;
        }
        case QWidget::PaletteShadow:
        {
            str = "PaletteShadow";
            break;
        }
        case QWidget::PaletteHighlight:
        {
            str = "PaletteHighlight";
            break;
        }
        case QWidget::PaletteHighlightedText:
        {
            str = "PaletteHighlightedText";
            break;
        }
	default:
	{
            break;
	}
    }
    
    return str;
}
	

QWidget::BackgroundMode DlgWidget::StrToBackgroundMode(
    const QString&	theBackgroundModeStr
    )
{
    QWidget::BackgroundMode backgroundMode = QWidget::PaletteBackground;
    
    if( theBackgroundModeStr == "FixedColor" )
    {
	backgroundMode = QWidget::FixedColor;
    }
    else if( theBackgroundModeStr == "FixedPixmap" )
    {
	backgroundMode = QWidget::FixedPixmap;
    }
    else if( theBackgroundModeStr == "NoBackground" )
    {
	backgroundMode = QWidget::NoBackground;
    }
    else if( theBackgroundModeStr == "PaletteForeground" )
    {
	backgroundMode = QWidget::PaletteForeground;
    }
    else if( theBackgroundModeStr == "PaletteBackground" )
    {
	backgroundMode = QWidget::PaletteBackground;
    }
    else if( theBackgroundModeStr == "PaletteLight" )
    {
	backgroundMode = QWidget::PaletteLight;
    }
    else if( theBackgroundModeStr == "PaletteMidlight" )
    {
	backgroundMode = QWidget::PaletteMidlight;
    }
    else if( theBackgroundModeStr == "PaletteDark" )
    {
	backgroundMode = QWidget::PaletteDark;
    }
    else if( theBackgroundModeStr == "PaletteMid" )
    {
	backgroundMode = QWidget::PaletteMid;
    }
    else if( theBackgroundModeStr == "PaletteText" )
    {
	backgroundMode = QWidget::PaletteText;
    }
    else if( theBackgroundModeStr == "PaletteBase" )
    {
	backgroundMode = QWidget::PaletteBase;
    }
    else if( theBackgroundModeStr == "PaletteButton" )
    {
        backgroundMode = QWidget::PaletteButton;
    }
    else if( theBackgroundModeStr == "PaletteBrightText" )
    {
        backgroundMode = QWidget::PaletteBrightText;
    }
    else if( theBackgroundModeStr == "PaletteShadow" )
    {
        backgroundMode = QWidget::PaletteShadow;
    }
    else if( theBackgroundModeStr == "PaletteHighlight" )
    {
        backgroundMode = QWidget::PaletteHighlight;
    }
    else if( theBackgroundModeStr == "PaletteHighlightedText" )
    {
        backgroundMode = QWidget::PaletteHighlightedText;
    }

    return backgroundMode;
}


QString DlgWidget::AlignmentToStr(
    int	theAlignment
    )
{
    QStrList strList;

    if( theAlignment & Qt::AlignLeft )
    {
	strList.append( "AlignLeft" );
    }
    else if( theAlignment & Qt::AlignHCenter )
    {
	strList.append( "AlignHCenter" );
    }
    else if( theAlignment & Qt::AlignRight )
    {
	strList.append( "AlignRight" );
    }
    
    if( theAlignment & Qt::AlignTop )
    {
	strList.append( "AlignTop" );
    }
    else if( theAlignment & Qt::AlignVCenter )
    {
	strList.append( "AlignVCenter" );
    }
    else if( theAlignment & Qt::AlignBottom )
    {
	strList.append( "AlignBottom" );
    }

    if( theAlignment & Qt::SingleLine )
    {
        strList.append( "SingleLine" );
    }
    
    if( theAlignment & Qt::DontClip )
    {
        strList.append( "DontClip" );
    }
    
    if( theAlignment & Qt::ExpandTabs )
    {
        strList.append( "ExpandTabs" );
    }

    if( theAlignment & Qt::ShowPrefix )
    {
        strList.append( "ShowPrefix" );
    }
    
    if( theAlignment & Qt::WordBreak )
    {
        strList.append( "WordBreak" );
    }
    
    if( theAlignment & Qt::DontPrint )
    {
        strList.append( "DontPrint" );
    }
    
    QString alignStr;
    QStrListIterator iter( strList );
    char* str = 0;
    while( (str = iter()) )
    {
        if( !alignStr.isEmpty() )
        {
            alignStr += "|";
        }
        
        alignStr += str;
    }
    
    return alignStr;
}


int DlgWidget::StrToAlignment(
    const QString&	theAlignmentStr
    )
{
    int align = 0;
    QString str = theAlignmentStr;
    QString alignStr;
    while( !str.isEmpty() )
    {
        int pos = str.find( '|' );
        if( pos > -1 )
        {
            alignStr = str.left( pos ).stripWhiteSpace();
            str = str.mid( pos + 1, str.length() - pos - 1 ).stripWhiteSpace();
        }
        else
        {
            alignStr = str;
            str = "";
        }
	
        if( alignStr == "AlignLeft" )
        {
            align |= Qt::AlignLeft;
        }
        else if( alignStr == "AlignHCenter" )
        {
            align |= Qt::AlignHCenter;
        }
        else if( alignStr == "AlignRight" )
        {
            align |= Qt::AlignRight;
        }
        else if( alignStr == "AlignTop" )
        {
            align |= Qt::AlignTop;
        }
        else if( alignStr == "AlignVCenter" )
        {
	    align |= Qt::AlignVCenter;
        }
        else if( alignStr == "AlignBottom" )
        {
            align |= Qt::AlignBottom;
        }
        else if( alignStr == "SingleLine" )
        {
            align |= Qt::SingleLine;
        }
        else if( alignStr == "DontClip" )
        {
            align |= Qt::DontClip;
        }
        else if( alignStr == "ExpandTabs" )
        {
            align |= Qt::ExpandTabs;
        }
        else if( alignStr == "ShowPrefix" )
        {
            align |= Qt::ShowPrefix;
        }
        else if( alignStr == "WordBreak" )
        {
            align |= Qt::WordBreak;
        }
        else if( alignStr == "DontPrint" )
        {
            align |= Qt::DontPrint;
        }
    }

    return align;
}


QString DlgWidget::BackgroundOriginToStr(
    QWidget::BackgroundOrigin       origin
    )
{
    QString str;
    
    switch( origin )
    {
        case QWidget::WidgetOrigin:
        {
            str = "WidgetOrigin";
            break;
        }
        case QWidget::ParentOrigin:
        {
            str = "ParentOrigin";
            break;
        }
    }
    
    return str;
}


QWidget::BackgroundOrigin DlgWidget::StrToBackgroundOrigin(
    const QString&  str
    )
{
    QWidget::BackgroundOrigin origin = QWidget::WidgetOrigin;
    
    if( str == "WidgetOrigin" )
    {
        origin = QWidget::WidgetOrigin;
    }
    else if( str == "ParentOrigin" )
    {
        origin = QWidget::ParentOrigin;
    }
    
    return origin;
}


QString DlgWidget::TextFormatToStr(
    Qt::TextFormat  format
    )
{
    QString str;
    
    switch( format )
    {
        case Qt::PlainText:
        {
            str = "PlainText";
            break;
        }
        case Qt::RichText:
        {
            str = "RichText";
            break;
        }
        case Qt::AutoText:
        {
            str = "AutoText";
            break;
        }
    }
    
    return str;
}


Qt::TextFormat DlgWidget::StrToTextFormat(
    const QString&  str
    )
{
    Qt::TextFormat format = Qt::PlainText;
    
    if( str == "PlainText" )
    {
        format = Qt::PlainText;
    }
    else if( str == "RichText" )
    {
        format = Qt::RichText;
    }
    else if( str == "AutoText" )
    {
        format = Qt::AutoText;
    }
    
    return format;
}


QTextStream& operator <<( 
    QTextStream&	ostream,
    const QSize&	size 
    )
{
    ostream << size.width() << ' ' << size.height();
    
    return ostream;
}


QTextStream& operator >>( 
    QTextStream&	istream,
    QSize&		size 
    )
{
    istream >> size.rwidth() >> size.rheight();
    
    return istream;
}


QTextStream& operator <<( 
    QTextStream&	ostream,
    const QPoint&	point 
    )
{
    ostream << point.x() << ' ' << point.y();
    
    return ostream;
}


QTextStream& operator >>( 
    QTextStream&	istream,
    QPoint&		point
    )
{
    istream >> point.rx() >> point.ry();
    
    return istream;
}


QTextStream& operator <<(
    QTextStream&	ostream,
    const QRect&	rect
    )
{
    ostream << rect.left() << ' ' << rect.top() << ' ' <<
    rect.width() << ' ' << rect.height();
    
    return ostream;
}


QTextStream& operator >>(
    QTextStream&	istream,
    QRect&		rect
    )
{
    int left, top, width, height;
    
    istream >> left;
    istream >> top;
    istream >> width;
    istream >> height;
    
    rect.setRect( left, top, width, height );
    
    return istream;
}


// write palette to text stream
QTextStream& operator <<(
    QTextStream&	ostream,
    const QPalette&	palette
    )
{
    ostream << 
	palette.normal() << ' ' <<
	palette.disabled() << ' ' <<
	palette.active();
    
    return ostream;
}


// read palette from stream
QTextStream& operator >>(
    QTextStream&	istream,
    QPalette&		palette
    )
{
    QColorGroup normal, disabled, active;
    
    istream >> normal >> disabled >> active;
    
    palette.setNormal( normal );
    palette.setDisabled( disabled );
    palette.setActive( active );

    return istream;
}


QTextStream& operator <<(
    QTextStream&	ostream,
    const QColorGroup&	group
    )
{
    ostream << 
	group.foreground().rgb() << ' ' <<
	group.button().rgb() << ' ' <<
	group.light().rgb() << ' ' <<
	group.dark().rgb() << ' ' <<
	group.mid().rgb() << ' ' <<
	group.text().rgb() << ' ' <<
	group.base().rgb() << ' ' <<
	group.background().rgb() << ' ' <<
	group.midlight().rgb() << ' ' <<
	group.brightText().rgb() << ' ' <<
	group.buttonText().rgb() << ' ' <<
	group.shadow().rgb() << ' ' <<
	group.highlight().rgb() << ' ' <<
	group.highlightedText().rgb();
    
    return ostream;
}


QTextStream& operator >>(
    QTextStream&	istream,
    QColorGroup&	group
    )
{
    QRgb fore, button, light, dark, mid, text, base, background, 
        midlight, brightText, buttonText, shadow, highlight, highlightedText;
    
    istream >> fore >> button >> light >> dark >> mid >> text >> base 
            >> background >> midlight >> brightText >> buttonText 
            >> shadow >> highlight >> highlightedText;
    
    group.setColor( QColorGroup::Foreground, QColor( fore ) );
    group.setColor( QColorGroup::Button, QColor( button ) );
    group.setColor( QColorGroup::Light, QColor( light ) );
    group.setColor( QColorGroup::Midlight, QColor( midlight ) );
    group.setColor( QColorGroup::Dark, QColor( dark ) );
    group.setColor( QColorGroup::Mid, QColor( mid ) );
    group.setColor( QColorGroup::Text, QColor( text ) );
    group.setColor( QColorGroup::BrightText, QColor( brightText ) );
    group.setColor( QColorGroup::ButtonText, QColor( buttonText ) );
    group.setColor( QColorGroup::Base, QColor( base ) );
    group.setColor( QColorGroup::Background, QColor( background ) );
    group.setColor( QColorGroup::Shadow, QColor( shadow ) );
    group.setColor( QColorGroup::Highlight, QColor( highlight ) );
    group.setColor( QColorGroup::HighlightedText, QColor( highlightedText ) );
    
    return istream;
}
  

QTextStream& operator <<(
    QTextStream&	ostream,
    const QFont&	font
    )
{
    ostream << '\"' << font.family() << "\" " <<
	font.pointSize() << ' ' <<
	font.weight() << ' ' <<
	(int)font.italic() << ' ' <<
	(int)font.underline() << ' ' <<
	(int)font.strikeOut() << ' ' <<
	(int)font.fixedPitch() << ' ' <<
	(int)font.styleHint() << ' ' <<
	(int)font.charSet();

    return ostream;
}


QTextStream& operator >>(
    QTextStream&	istream,
    QFont&		font
    )
{
    QString family, str;
    
    // since family name might have spaces in it, loop until you get the 
    // closing quote which encloses the name
    do
    {
	istream >> str;
	family += str;
    } while( str[((int)str.length()) - 1] != '\"' );
    
    // remove leading and trailing "'s
    family = family.mid( 1, family.length()-2 );
    
    font.setFamily( family );

    int n;
    
    istream >> n;
    font.setPointSize( n );
    
    istream >> n;
    font.setWeight( n );
    
    istream >> n;
    font.setItalic( n );
    
    istream >> n;
    font.setUnderline( n );

    istream >> n;
    font.setStrikeOut( n );
    
    istream >> n;
    font.setFixedPitch( n );
    
    istream >> n;
    font.setStyleHint( (QFont::StyleHint)n );
    
    istream >> n;
    font.setCharSet( (QFont::CharSet)n );

    return istream;
}


void SaveBool(
    QTextStream&	ostream,
    const bool&	b
    )
{
    ostream << (b ? "TRUE" : "FALSE");
}


void RestoreBool(
    QTextStream&	istream,
    bool&		b
    )
{
    QString aStr;

    istream >> aStr;
    if( aStr == "TRUE" )
	b = TRUE;
    else
	b = FALSE;
}


