/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: SignalConnection.cpp,v 2.2 2000/07/23 20:38:01 jeffharris Exp $

*******************************************************************************/


#include <qregexp.h>

#include "widget/SignalConnection.h"


SignalConnection::SignalConnection()
    :
    itsScope( Protected )
{
}


SignalConnection::SignalConnection
(
    Scope		scope,
    const QString&	signal,
    const QString&	slot,
    const QString&	args 
)
    :
    itsScope( scope ),
    itsSignal( signal ),
    itsSlot( slot ),
    itsArgs( args )
{
}


QString SignalConnection::GetSignal( void ) const
{
    QString str = itsSignal + "(" + itsArgs + ")";
    return str;
}


void SignalConnection::SetSignal
(
    const QString&	sig
)
{
    QString str = sig.copy();
    QRegExp args( "(.*)" );
    int len;
    int pos = args.match( str, 0, &len );
    
    if( len >= 0 )
    {
	itsArgs = str.mid( pos+1, len-2 );
	itsSignal = str.replace( args, "" );
    }
    else
    {
	itsArgs = "";
	itsSignal = str;
    }
}
    

const QString SignalConnection::GetSlot( void ) const
{
    QString str = itsSlot + "(" + itsArgs + ")";
    return str;
}


QString SignalConnection::ToString( void ) const
{
    QString str;
    
    str = "[";
    
    switch( itsScope )
    {
	case BaseClass:
	{
	    str += "BaseClass";
	    break;
	}
	case Protected:
	{
	    str += "Protected";
	    break;
	}
	case Public:
	{
	    str += "Public";
	    break;
	}
    }
    
    str += "]  " + itsSignal + " --> " + itsSlot + "  (" + itsArgs + ")";
    
    return str;
}


void SignalConnection::FromString( const QString& from )
{
    itsScope = Protected;
    itsSignal = "";
    itsSlot = "";

    QRegExp scopeReg( "^\\[.*\\]" );
    int len;
    
    int pos = scopeReg.match( from, 0, &len );
    if( pos >= 0 )
    {
	QString scope = from.mid( pos+1, len-2 );
	if( scope == "Protected" )
	{
	    itsScope = Protected;
	}
	else if( scope == "Public" )
	{
	    itsScope = Public;
	}
	else if( scope == "BaseClass" )
	{
	    itsScope = BaseClass;
	}
	
	QRegExp signalReg( ".*-->" );
	pos = signalReg.match( from, pos + len, &len );
	if( pos >= 0 )
	{
	    itsSignal = from.mid( pos, len-3 );
	    itsSignal = itsSignal.stripWhiteSpace();
	    
	    QRegExp slotReg( "-->.*(" );
	    pos = slotReg.match( from, pos + len - 3, &len );
	    
	    if( pos >= 0 )
	    {
		itsSlot = from.mid( pos + 3, len - 4 );
		itsSlot = itsSlot.stripWhiteSpace();

		QRegExp argReg( ".*)" );
		pos = argReg.match( from, pos + len, &len );
		
		if( pos >= 0 )
		{
		    itsArgs = from.mid( pos, len - 1 );
		    itsArgs = itsArgs.stripWhiteSpace();
		}
	    }
	}
    }
}






