/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgClass.cpp,v 2.5 2000/07/23 20:38:01 jeffharris Exp $

*******************************************************************************/


#include "widget/DlgCCC.h"

#include "prop/CCCPage.h"

#include "GuiTools.h"

#include "xml/QtArchExpat.h"
#include "xml/QtArchGenerator.h"

#define Inherited DlgDDD

QStrList DlgCCC::itsSignals;
int DlgCCC::itsNextWidgetNum = 1;

DlgCCC::DlgCCC() :
    Inherited( DlgWidget::DW_CCC )
{
}


DlgCCC::DlgCCC(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgCCC::~DlgCCC()
{
}


void DlgCCC::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new CCCPage( this, parent, "CCC" ) );

    labels.append( "CCC" );
}


void DlgCCC::Save(
    QtArchGenerator&    generator
    )
{
    generator.WriteStartBlock( "CCC" );
    
    QCCC* widget = GetCCC();

    // Insert saving code here.
    // generator.WriteXMLValue( "property", value );

    Inherited::Save( generator );
    
    generator.WriteEndBlock();
}


void DlgCCC::RestoreXML( 
    XMLValue&   value
    )
{
    // Sanity check
    if( value.GetTypeString() != "CCC" )
        return;

    XMLValueMap map = value.AsNested();

    // Create if necessary
    if( !GetCCC() )
    {
        CreateQtWidget( 0 );
    }
  
    QCCC* widget = GetCCC();

    // Load properties
    //widget->setProperty( map["property"].AsType() );
  
    // We inherit from 
    Inherited::RestoreXML( map["DDD"] );
}


void DlgCCC::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    const QCCC* widget = GetCCC();

    // Insert source generation code here.
}


void DlgCCC::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgCCC::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
	// Add signals here.
    }

    return itsSignals;
}


void DlgCCC::CreateQtWidget(
    const char*	theName
    )
{
    QCCC* widget = 
	new QCCC( GetParent(),
		  CreateName( "CCC", 
			      theName, 
			      itsNextWidgetNum ) );

    SetQtWidget( widget );
}
