/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: QtArchExpat.h,v 1.5 2000/07/23 20:38:02 jeffharris Exp $

*******************************************************************************/

#ifndef QtArchExpat_included
#define QtArchExpat_included

#include "expat/xmlparse/xmlparse.h"
#include <qobject.h>
#include <qmap.h>
#include <qstack.h>
#include "XMLValue.h"

typedef void* UData;

typedef QMap<QString,QString> StringMap;

class QtArchExpat : public QObject
{
  Q_OBJECT

public:
  QtArchExpat( const QString &filename );
  ~QtArchExpat();

  // Parse the file. Structures found will get emitted using signals
  bool Parse();

  // Retrieve the dlg type, no signals will get emitted
  QString GetType();
  
  // The following function are for internal use only
  // They are called by the expat callback only
  void StartElement( UData, const XML_Char *, const XML_Char ** );
  void EndElement( UData, const XML_Char * );
  void CharacterData( UData, const XML_Char *, int );
  void ProcessingInstruction( UData, const XML_Char *, const XML_Char * );
  void Comment( UData, const XML_Char * );

  signals:
  void NewDlgObject( const StringMap & );
  void NewType( const XMLValue & );

private slots:
  void GetTypeSlot( const StringMap & );  
  
private: // Attributes
  QString theFileName;

  XML_Parser theParser;

  bool parsedEnough;
  QString qtArchType;
  
  QStack<bool> nested;
  QStack<bool> list;
  QStack<QString> types;
  QStack<QString> complex;
  QStack<QString> widgets;
  QStack<QString> superType;

  bool layoutFlag;
  
  QStack<XMLValueMap> simpleMap;
  QStack<XMLValueList> simpleList;
  
  QString currentElement;
  QString currentData;
  QString currentType;
  QString currentWidget;
};


#endif
