/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: QtArchGenerator.h,v 1.5 2000/07/23 20:38:02 jeffharris Exp $

*******************************************************************************/

#ifndef QtArchGenerator_included
#define QtArchGenerator_included

#include <qobject.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qstack.h>
#include "XMLValue.h"

// Exception class
struct GeneratorError
{
  GeneratorError( const QString &msg )
    {
      message = msg;
    };

  QString message;
};

class QtArchGenerator : public QObject
{

  Q_OBJECT

public:
  QtArchGenerator( bool printDefault = false );
  ~QtArchGenerator();

  // Initialize generator: open file, write header, etc.
  void WriteStart( const QString &fileName );
  void WriteStart( QTextStream *stream );
  
  // Start a nesting block with given name: <name> and attributes
  // If nested == true, the attribute "nested=true" will be appended
  // nested is used by the parser to determine wether this block will
  // contain only text or children
  void WriteStartBlock( const QString &, bool nested = TRUE, const QString& = "" );

  // Convinience method for creating lists 
  void WriteStartList( const QString &, const QString& = "" );
  
  // End the last opened block: </name>
  void WriteEndBlock();

  // Write a single line: <name> value </name>
  void WriteSingleLine( const QString &name, 
                        const QString &value,
                        const QString& = "" );

  // Write integer, string, bool, ...
  void WriteXMLValue( const QString &name, const XMLValue &value );
  
  // Close file
  void WriteEnd();

private:
  int indentionLevel;

  QFile file;
  QTextStream *stream;
  bool ourStream;

  QStack<QString> elementStack;

  bool printDefaults;
  
private: // methods
  void BreakDownXMLValue( const XMLValue &value );
  
  QString indent();

    QString EncodeText( const QString& text );
    // Encode a text node, escaping XML symbols <, >, etc.
};
  
#endif
