/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: XMLValue.cpp,v 1.11 2000/07/23 20:38:02 jeffharris Exp $

*******************************************************************************/

#include <qtextstream.h>

#include "XMLValue.h"
#include "QtArchGenerator.h"


XMLValueMap::XMLValueMap()
{
}


XMLValueMap::XMLValueMap( const XMLValueMap& map ) :
    QMap<QString,XMLValue>( map )
{
    begin();
}

    
XMLValueMap& XMLValueMap::operator=( const XMLValueMap& map )
{
    if( this != &map )
    {
        QMap<QString,XMLValue>::operator=( map );
        begin();
    }
    
    return *this;
}


XMLValueList::XMLValueList()
{
    setAutoDelete( TRUE );
}


XMLValueList::XMLValueList( const XMLValueList& list ) :
    QList<XMLValue>()
{
    operator=( list );

    setAutoDelete( TRUE );
}


XMLValueList& XMLValueList::operator=( const XMLValueList& list )
{
    if( this != &list )
    {
        //QList<XMLValue>::operator=( list );

        clear();
        
        XMLValue* value;
        QListIterator<XMLValue> iter( list );
        while( (value = iter()) )
        {
            append( new XMLValue( *value ) );
        }

    }
    
    return *this;
}


XMLValue::XMLValue()
{
  type = XMLNoType;
  value = 0;
  complexType = FALSE;
}

XMLValue::XMLValue( const XMLValue &xmlValue ) :
    value( 0 )
{
    operator=( xmlValue );
    
/*
  type = xmlValue.type;
  typeString = xmlValue.typeString;
  name = xmlValue.name;
  complexType = xmlValue.complexType;

  value = 0;
  if( xmlValue.value != 0 )
  {
      value = new Value;
      *value = *(xmlValue.value);
  }
*/
}

XMLValue::XMLValue( const int i ) :
    value( 0 )
{
  SetInt( i );
}

XMLValue::XMLValue( const bool b ) :
    value( 0 )
{
  SetBool( b );
}

XMLValue::XMLValue( const double d ) :
    value( 0 )
{
    SetDouble( d );
}

XMLValue::XMLValue( const QString &str ) :
    value( 0 )
{
  SetQString( str );
}

XMLValue::XMLValue( const QSize &size ) :
    value( 0 )
{
  SetQSize( size );
}

XMLValue::XMLValue( const QRect &rect ) :
    value( 0 )
{
  SetQRect( rect );
}

XMLValue::XMLValue( const QPoint &point ) :
    value( 0 )
{
  SetQPoint( point );
}

XMLValue::XMLValue( const QPalette &pal ) :
    value( 0 )
{
  SetQPalette( pal );
}

XMLValue::XMLValue( const QFont &font ) :
    value( 0 )
{
  SetQFont( font );
}

XMLValue::XMLValue( const QColor &color ) :
    value( 0 )
{
  SetQColor( color );
}

XMLValue::XMLValue( const QColorGroup &colGrp ) :
    value( 0 )
{
  SetQColorGroup( colGrp );
}

XMLValue::XMLValue( const QString &ts, const QString &val ) :
    value( 0 )
{
  SetTypValue( ts, val );
}

XMLValue::XMLValue( const QString &ts,
                    const XMLValueMap &map,
                    XMLValueType t ) :
    value( 0 )
{
  SetTypValue( ts, map, t );
}

XMLValue::XMLValue( const QString &ts,
                    const XMLValueList &list ) :
    value( 0 )
{
  SetTypValue( ts, list );
}

XMLValue::~XMLValue()
{
    delete value;
    value = 0;
}

XMLValue& XMLValue::operator=( const XMLValue& copyValue )
{
    if( &copyValue != this )
    {
        type = copyValue.type;
        typeString = copyValue.typeString;
        complexType = copyValue.complexType;
        name = copyValue.name;
        
        delete value;
        value = 0;
        
        if( copyValue.value )
        {
            value = new Value( *copyValue.value );
        }
    }
    
    return *this;
}


XMLValue::XMLValueType XMLValue::Type() const
{
  return type;
}

void XMLValue::SetType( XMLValueType newType )
{
  if( type != XMLNoType )
    delete value;
  value = 0;
  type = newType;
}

void XMLValue::SetTypValue( const QString &ts, const QString &val )
{
    InitValue();

  if( ts == GetTypeString( XMLint ) )
  {
    type = XMLint;
    value->intValue = val.toInt();
  } 
  else if( ts == GetTypeString( XMLbool ) )
  {
    type = XMLbool;
    value->boolValue = val == "true";
  } 
  else if( ts == GetTypeString( XMLdouble ) )
  {
      type = XMLdouble;
      value->doubleValue = val.toDouble();
  }
  else if( ts == GetTypeString( XMLQString ) )
  {
    type = XMLQString;
    value->qstringValue = val;
  } 
  else 
  {
    type = XMLQString;
    value->qstringValue = val;
  } 
}

void XMLValue::SetTypValue( const QString &ts,
                    const XMLValueMap &map,
                    XMLValueType t )
{
    InitValue();

  if( t == XMLNested )
  {
    if( ts == GetTypeString( XMLQSize ) )
    {
      type = XMLQSize;

      int w, h;
    
      value->qsizeValue.setWidth( w = map["Width"].AsInt() );
      value->qsizeValue.setHeight( h = map["Height"].AsInt() );

#ifdef DEBUGEXPAT
      debug( "QSize constructed, %d, %d", w, h );
#endif
    } 
    else if( ts == GetTypeString( XMLQRect ) )
    {
      type = XMLQRect;

      value->qrectValue.setX( map["X"].AsInt() );
      value->qrectValue.setY( map["Y"].AsInt() );
      value->qrectValue.setWidth( map["Width"].AsInt() );
      value->qrectValue.setHeight( map["Height"].AsInt() );    
    } 
    else if( ts == GetTypeString( XMLQPoint ) )
    {
      type = XMLQPoint;

      value->qpointValue.setX( map["X"].AsInt() );
      value->qpointValue.setY( map["Y"].AsInt() );
    } 
    else if( ts == GetTypeString( XMLQPalette ) )
    {
      type = XMLQPalette;
    
      value->qpaletteValue.setActive( map["Active"].AsQColorGroup() );
      value->qpaletteValue.setDisabled( map["Disabled"].AsQColorGroup() );
      value->qpaletteValue.setInactive( map["Inactive"].AsQColorGroup() );
    
    } 
    else if( ts == GetTypeString( XMLQFont ) )
    {
      type = XMLQFont;

      value->qfontValue.setFamily( map["Family"].AsQString() );
      value->qfontValue.setPointSize( map["PointSize"].AsInt() );
      value->qfontValue.setWeight( map["Weight"].AsInt() );
      value->qfontValue.setItalic( map["Italic"].AsBool() );
      value->qfontValue.setUnderline( map["Underline"].AsBool() );
      value->qfontValue.setStrikeOut( map["StrikeOut"].AsBool() );
      value->qfontValue.setFixedPitch( map["FixedPitch"].AsBool() );
      value->qfontValue.setStyleHint( (QFont::StyleHint) map["StyleHint"].AsInt() );
      value->qfontValue.setCharSet( (QFont::CharSet) map["CharSet"].AsInt() );
    } 
    else if( ts == GetTypeString( XMLQColor ) )
    {
      type = XMLQColor;
    
      int r, g, b;
      r = map["Red"].AsInt();
      b = map["Blue"].AsInt();
      g = map["Green"].AsInt();

      value->qcolorValue.setRgb( r, g, b );
    } 
    else if( ts == GetTypeString( XMLQColorGroup ) )
    {
      type = XMLQColorGroup;
    
      value->qcolorgroupValue.setColor( QColorGroup::Foreground,
                                        map["Foreground"].AsQColor() );
      value->qcolorgroupValue.setColor( QColorGroup::Button,
                                        map["Button"].AsQColor() );
      value->qcolorgroupValue.setColor( QColorGroup::Light,
                                        map["Light"].AsQColor() );
      value->qcolorgroupValue.setColor( QColorGroup::Midlight,
                                        map["MidLight"].AsQColor() );
      value->qcolorgroupValue.setColor( QColorGroup::Dark,
                                        map["Dark"].AsQColor() );
      value->qcolorgroupValue.setColor( QColorGroup::Mid,
                                        map["Mid"].AsQColor() );
      value->qcolorgroupValue.setColor( QColorGroup::Text,
                                        map["Text"].AsQColor() );
      value->qcolorgroupValue.setColor( QColorGroup::BrightText,
                                        map["BrightText"].AsQColor() );
      value->qcolorgroupValue.setColor( QColorGroup::ButtonText,
                                        map["ButtonText"].AsQColor() );
      value->qcolorgroupValue.setColor( QColorGroup::Base,
                                        map["Base"].AsQColor() );
      value->qcolorgroupValue.setColor( QColorGroup::Background,
                                        map["Background"].AsQColor() );
      value->qcolorgroupValue.setColor( QColorGroup::Shadow,
                                        map["Shadow"].AsQColor() );
      value->qcolorgroupValue.setColor( QColorGroup::Highlight,
                                        map["HighLighted"].AsQColor() );
      value->qcolorgroupValue.setColor( QColorGroup::HighlightedText,
                                        map["HighLightedText"].AsQColor() );
    
    } else
    {
      // It's a special kind of nested type
      type = t;
      typeString = ts;
      value->map = map;
    }
  }
  else  
  {
    // It's a special kind of nested type
    type = t;
    typeString = ts;
    value->map = map;
  }
}

void XMLValue::SetTypValue( const QString &ts,
                    const XMLValueList &list )
{
  InitValue();
  type = XMLList;
  typeString = ts;
  value->list = list;
}

void XMLValue::SetInt( const int i )
{
  type = XMLint;
  complexType = FALSE;
  InitValue();
  value->intValue = i;
}

void XMLValue::SetBool( const bool b )
{
  type = XMLbool;
  complexType = FALSE;
  InitValue();
  value->boolValue = b;
}

void XMLValue::SetDouble( const double d )
{
    type = XMLdouble;
    complexType = FALSE;
    InitValue();
    value->doubleValue = d;
}

void XMLValue::SetQString( const QString &str )
{
  type = XMLQString;
  complexType = FALSE;
  InitValue();
  value->qstringValue = str;
}

void XMLValue::SetNested( const XMLValueMap &map, XMLValueType t )
{
  type = t;
  complexType = TRUE;
  InitValue();
  value->map = map;
}

#define SETVAL( t, var ) \
void XMLValue::Set##t( const t &val ) \
{\
  type = XML##t;\
  complexType = TRUE;\
  InitValue();\
  value->var = val;\
}

SETVAL( QSize, qsizeValue );
SETVAL( QRect, qrectValue );
SETVAL( QPoint, qpointValue );
SETVAL( QPalette, qpaletteValue );
SETVAL( QColorGroup, qcolorgroupValue );
SETVAL( QColor, qcolorValue );
SETVAL( QFont, qfontValue );

int XMLValue::AsInt() const
{
  if( type == XMLint )
    return value->intValue;
  return 0;
}

bool XMLValue::AsBool() const
{
  if( type == XMLbool )
    return value->boolValue;
  return FALSE;
  
}

double XMLValue::AsDouble() const
{
    if( type == XMLdouble )
    {
        return value->doubleValue;
    }
    else
    {
        return 0.0;
    }
}

QSize XMLValue::AsQSize() const
{
  if( type == XMLQSize )
    return value->qsizeValue;
  return QSize( 0, 0 );
  
}

QRect XMLValue::AsQRect() const
{
#ifdef DEBUGEXPAT
  debug( "As QRect(%d,%d) (%d,%d)",
         value->qrectValue.x(), value->qrectValue.y(),
         value->qrectValue.width(), value->qrectValue.height() );
#endif

  if( type == XMLQRect )
    return value->qrectValue;
  return QRect();
}

QPoint XMLValue::AsQPoint() const
{
  if( type == XMLQPoint )
    return value->qpointValue;
  return QPoint();
}

QPalette XMLValue::AsQPalette() const
{
  if( type == XMLQPalette )
    return value->qpaletteValue;
  return QPalette();
}

QFont XMLValue::AsQFont() const
{
  if( type == XMLQFont )
    return value->qfontValue;
  return QFont();
  
}

QColor XMLValue::AsQColor() const
{
  if( type == XMLQColor )
    return value->qcolorValue;
  return QColor();
  
}

QColorGroup XMLValue::AsQColorGroup() const
{
  if( type == XMLQColorGroup )
    return value->qcolorgroupValue;
  return QColorGroup();
  
}

XMLValueMap XMLValue::AsNested() const
{
  XMLValueMap retMap;
  if( value )
    retMap = value->map;
  
  return retMap;
}

XMLValueList XMLValue::AsList() const
{
  if( ! value )
    return XMLValueList();
  
  return value->list;
}

QString XMLValue::AsQString() const
{
    QString retString;

    switch( type )
    {
        case XMLint :
            retString.setNum( value->intValue );
            break;
        case XMLbool :
            retString = value->boolValue ? "true" : "false";
            break;
        case XMLdouble:
            retString.setNum( value->doubleValue );
            break;
        case XMLQString :
            retString = value->qstringValue;
            break;
        default:
            retString = "";
            break;
    }

#ifdef DEBUGEXPAT
    debug( "As QString: %s", (const char*) retString );
#endif

    return retString;
}

bool XMLValue::ComplexType() const
{
  return complexType;
}

XMLValueMap XMLValue::BreakDown() const
{
  XMLValueMap map;

  // Breakdown is only possible for complex types
  // Simple types are accessible using AsQString
  if( ! complexType )
    return map;
  
  switch( type )
  {
    case XMLQSize :
      map["Width"] = XMLValue( value->qsizeValue.width() );
      map["Height"] = XMLValue( value->qsizeValue.height() );
      break;
    case XMLQRect :
      map["X"] = XMLValue( value->qrectValue.x() );
      map["Y"] = XMLValue( value->qrectValue.y() );
      map["Width"] = XMLValue( value->qrectValue.width() );
      map["Height"] = XMLValue( value->qrectValue.height() );
      break;
    case XMLQPoint :
      map["X"] = XMLValue( value->qpointValue.x() );
      map["Y"] = XMLValue( value->qpointValue.y() );
      break;
    case XMLQFont :
      map["Family"] = XMLValue( value->qfontValue.family() );
      map["PointSize"] = XMLValue( value->qfontValue.pointSize() );
      map["Weight"] = XMLValue( value->qfontValue.weight() );
      map["Italic"] = XMLValue( value->qfontValue.italic() );
      map["Underline"] = XMLValue( value->qfontValue.underline() );
      map["StrikeOut"] = XMLValue( value->qfontValue.strikeOut() );
      map["FixedPitch"] = XMLValue( value->qfontValue.fixedPitch() );
      map["StyleHint"] = XMLValue( value->qfontValue.styleHint() );
      map["CharSet"] = XMLValue( value->qfontValue.charSet() );
      break;
    case XMLQPalette :
      map["Active"] = XMLValue( value->qpaletteValue.active() );
      map["Inactive"] = XMLValue( value->qpaletteValue.inactive() );
      map["Disabled"] = XMLValue( value->qpaletteValue.disabled() );
      break;
    case XMLQColor :
      map["Red"] = XMLValue( value->qcolorValue.red() );
      map["Green"] = XMLValue( value->qcolorValue.green() );
      map["Blue"] = XMLValue( value->qcolorValue.blue() );
      break;
    case XMLQColorGroup :
      map["Foreground"] = XMLValue( value->qcolorgroupValue.foreground() );
      map["Button"] = XMLValue( value->qcolorgroupValue.button() );
      map["Light"] = XMLValue( value->qcolorgroupValue.light() );
      map["Dark"] = XMLValue( value->qcolorgroupValue.dark() );
      map["Mid"] = XMLValue( value->qcolorgroupValue.mid() );
      map["Text"] = XMLValue( value->qcolorgroupValue.text() );
      map["Base"] = XMLValue( value->qcolorgroupValue.base() );
      map["Background"] = XMLValue( value->qcolorgroupValue.background() );
      map["MidLight"] = XMLValue( value->qcolorgroupValue.midlight() );
      map["BrightText"] = XMLValue( value->qcolorgroupValue.brightText() );
      map["ButtonText"] = XMLValue( value->qcolorgroupValue.buttonText() );
      map["Shadow"] = XMLValue( value->qcolorgroupValue.shadow() );
      map["HighLighted"] = XMLValue( value->qcolorgroupValue.highlight() );
      map["HighLightedText"] =
        XMLValue( value->qcolorgroupValue.highlightedText() );
      break;
    default :
      break;
  }

  return map;
}
      
QString XMLValue::GetTypeString( XMLValueType t ) const
{
  QString retString;

  // Use this type aa default
  if( t == XMLNoType )
    t = type;
  
  switch( t )
  {
    case XMLint : retString = "integer"; break;
    case XMLbool : retString = "boolean"; break;
    case XMLdouble : retString = "double"; break;
    case XMLQSize : retString = "qsize"; break;
    case XMLQRect : retString = "qrect"; break;
    case XMLQPoint : retString = "qpoint"; break;
    case XMLQPalette : retString = "qpalette"; break;
    case XMLQFont : retString = "qfont"; break;
    case XMLQColor : retString = "qcolor"; break;
    case XMLQColorGroup : retString = "qcolorgroup"; break;
    case XMLNested : retString = typeString; break;
    case XMLList : retString = typeString; break;
    case XMLDlgWidget : retString = typeString; break;
    case XMLQString:
    default:
      retString = "qstring"; break;
  }
  return retString;
}

bool XMLValue::IsDefault() const
{
    bool retBool = FALSE;

    if( ! value || type == XMLNoType )
        return FALSE;
  
    switch( type )
    {
        case XMLint :
            retBool = value->intValue == 0 ? TRUE : FALSE;
            break;
        case XMLbool :
            retBool = value->boolValue ? FALSE : TRUE;
            break;
        case XMLdouble:
            retBool = (value->doubleValue == 0.0);
            break;
        case XMLQSize :
            retBool = value->qsizeValue == QSize( 0, 0 ) ? TRUE : FALSE;
            break;
        case XMLQRect :
            retBool = value->qrectValue == QRect() ? TRUE : FALSE;
            break;
        case XMLQPoint :
            retBool = value->qpointValue == QPoint() ? TRUE : FALSE;
            break;
        case XMLQPalette : 
            retBool = value->qpaletteValue == QPalette() ? TRUE : FALSE;
            break;
        case XMLQFont :
            retBool = value->qfontValue == QFont() ? TRUE : FALSE;
            break;
        case XMLQColor :
            retBool = value->qcolorValue == QColor() ? TRUE : FALSE;
            break;
        case XMLQColorGroup :
            retBool = value->qcolorgroupValue == QColorGroup() ? TRUE : FALSE;
            break;
        case XMLNested :
        case XMLList :
        case XMLDlgWidget :
            retBool = FALSE;
            break;
        case XMLQString:
            retBool = value->qstringValue.isEmpty() ? TRUE : FALSE;
            break;
        default:
            retBool = FALSE;
            break;
    }
    return retBool;
}

void XMLValue::Dump( const QString &fileName )
{
  QtArchGenerator g;
  g.WriteStart( fileName );

  _Dump( &g );
  
  g.WriteEnd();
}

void XMLValue::_Dump( QtArchGenerator *dumpGenerator ) const
{
  QString n;
  n = (name.isEmpty() ? QString("NoName") : name);
  
  switch( type )
  {
    // Simple and complex types can be handled by the generator
    case XMLint:
    case XMLbool:
    case XMLdouble:
    case XMLQString:
    case XMLQSize:
    case XMLQRect:
    case XMLQPoint:
    case XMLQPalette:
    case XMLQFont:
    case XMLQColor:
    case XMLQColorGroup:
      dumpGenerator->WriteXMLValue( n, *this );
      break;
    case XMLNested:
      {
        XMLValueMap::Iterator it;
        dumpGenerator->WriteStartBlock( n );
        for( it = value->map.begin(); it != value->map.end(); ++it )
        {
          it.data()._Dump( dumpGenerator );
        }
        dumpGenerator->WriteEndBlock();
      }
      break;
    case XMLList:
      {
        dumpGenerator->WriteStartList( n );
        XMLValue* v;
        for( v = value->list.first(); v; v = value->list.next() )
        {
          v->_Dump( dumpGenerator );
        }
        dumpGenerator->WriteEndBlock();
      }
      break;
    case XMLDlgWidget:
      dumpGenerator->WriteStartBlock( n );
      dumpGenerator->WriteEndBlock();
      break;
    case XMLNoType:
      break;
  }
}

